/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its
documentation for educational, research and non-profit purposes,
without fee, and without a written agreement is hereby granted,
provided that the above copyright notice, this paragraph and the
following five paragraphs appear in all copies.

Permission to incorporate this software into commercial products may
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate
receiving any enhancements and modifications of this software
made by others, and reserves the right to include them in future
releases. The Institute also recommends that this software not
be redistributed by third parties, but be obtained by all directly
from the ICSI server, so that all users of the software may
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* $Id: MBoneMedia.h,v 1.3 1996/08/12 11:36:14 whd Exp $ */

#ifndef _MBoneMedia_h
#define _MBoneMedia_h

#include <String.h>

extern "C" {
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <stdlib.h>
#include <unistd.h>
}

#include "VCRDisplay.h"
#include "VCRFile.h"
#include "VCRTcl.h"
#include "MediaTimer.h"
#include "MediaSocket.h"
#include "VCRPacket.PHPQ.h"
#include "SrcInfo.AVLSet.h"
#include "it_depends.h"

enum media_fmts		{ NONE	=0,
			  RTP   =1,
			  VAT   =2,
			  WB    =3,
			  /* FMT_ERROR must always be at the end! */
			  FMT_ERROR =4
			};

enum media_states	{ INVALID,
			  IDLE,
			  PLAYING,
			  RECORDING,
			  FF,
			  REW
			};

enum media_rcs	 	{ MEDIA_LAST_PACKET		=1,
			  MEDIA_OK			=2,

			  MEDIA_BOF			=11,
			  MEDIA_EOF			=12,

			  MEDIA_NOT_READY		=21,
			  MEDIA_INVALID_STATE		=22,
			  MEDIA_ERROR			=23,

			  MEDIA_GONE			=31,

			  MEDIA_PACKET_PENDING		=41,
			  MEDIA_NO_PACKET_PENDING	=42,

			  MEDIA_UPDATE_TIME		=51,

			  MEDIA_IS_READ_ONLY		=61,
			  MEDIA_EMPTY			=62,

			  MEDIA_IDX_NOT_FOUND		=71,

			  MEDIA_IDX_TOO_LATE		=81,

			  MEDIA_RECORDING_STARTED	=91,

			  MEDIA_NO_DATA_TIMEOUT		=101
			};

enum msg_codes		{ SEND_ID,
			  SEND_DONE
			};

#define MAX_SRC

#define UPDATE_TIME 10000

class 	MBoneSession;
typedef void (MBoneSession::*MediaEvtHdlr)(int, media_rcs, u_int32_t);

class 	MBoneMedia;
typedef MBoneMedia *P2MBoneMedia;

class 	MBoneMedia {

public:

  MBoneMedia(String ="", int =0,
	     String ="224.4.4.4", int =4444,
	     u_int16_t =0, u_int8_t =1,
	     int =1, int =0, int =0, int =0, int =0);
  virtual ~MBoneMedia();

  inline media_rcs set_mute(int m) {
    mute=m;
    switch (media_state) {
    case RECORDING:
      if (mute) {
	media_data_socket.deactivate_read_hdlr();
	if (nb_of_sockets==2) media_session_socket.deactivate_read_hdlr();
      } else {
	media_data_socket.activate_read_hdlr();
	if (nb_of_sockets==2) media_session_socket.activate_read_hdlr();
      };
      break;
    case PLAYING:
      media_timeout();
      break;
    default:
      break;
    };
    return MEDIA_OK;
  };
  inline int get_mute() {
    return mute;
  };
  inline media_rcs toggle_mute() {
    mute=(mute!=1);
    return MEDIA_OK;
  };
  inline media_states get_media_state() {
    return media_state;
  };
  inline media_fmts get_media_fmt() {
    return media_fmt;
  };
  inline String get_media_ipaddr() {
    return media_ipaddr;
  };
  inline String get_media_filename() {
    return media_filename;
  };
  inline int get_media_port() {
    return media_port;
  };
  inline u_int16_t get_media_confid() {
    return media_confid;
  };
  inline u_int8_t get_media_ttl() {
    return media_ttl;
  };
  inline String get_media_name() {
    return media_name;
  };
  inline int get_media_nb() {
    return media_number;
  };

  inline media_rcs init_media_buffer(char **buffer, int size) {
    free_media_buffer(buffer);
    Media_buffer_size=size;
    if (size==0) {
      Media_buffer=*buffer=NULL;
    } else {
      Media_buffer=*buffer=(char *) malloc(Media_buffer_size);
    };
    return MEDIA_OK;
  };

  inline media_rcs change_media_buffer(char **buffer, char **new_buffer, int new_size) {
    free_media_buffer(buffer);
    Media_buffer_size=new_size;
    if (new_size==0) {
      Media_buffer=*buffer=*new_buffer=NULL;
    } else {
      Media_buffer=*buffer=*new_buffer;
    };
    return MEDIA_OK;
  };

  inline media_rcs free_media_buffer(char **buffer) {
    if ((*buffer==Media_buffer) && Media_buffer && Media_buffer_size) free(Media_buffer);
    return MEDIA_OK;
  };
  inline int media_buffer_size() { return Media_buffer_size; };

  inline media_rcs init_session_buffer(char **buffer, int size) {
    free_session_buffer(buffer);
    Session_buffer_size=size;
    if (size==0) {
      Session_buffer=*buffer=NULL;
    } else {
      Session_buffer=*buffer=(char *) malloc(Session_buffer_size);
    };
    return MEDIA_OK;
  };

  inline media_rcs free_session_buffer(char **buffer) {
    if ((*buffer==Session_buffer) && Session_buffer && Session_buffer_size) free(Session_buffer);
    return MEDIA_OK;
  };
  inline int session_buffer_size() { return Session_buffer_size; };

  inline media_rcs init_id_buffer(char **buffer, int size) {
    free_id_buffer(buffer);
    Id_buffer_size=size;
    if (size==0) {
      Id_buffer=*buffer=NULL;
    } else {
      Id_buffer=*buffer=(char *) malloc(Id_buffer_size);
    };
    return MEDIA_OK;
  };

  inline media_rcs free_id_buffer(char **buffer) {
    if ((*buffer==Id_buffer) && Id_buffer && Id_buffer_size) free(Id_buffer);
    return MEDIA_OK;
  };
  inline int id_buffer_size() { return Id_buffer_size; };

  /* Media specific playback functions: */
  virtual media_rcs start_playing();
  virtual media_rcs stop_playing();
  virtual media_rcs read_next_sync_unit(u_int32_t *, float =1, int =0) =0;
  virtual media_rcs send_pending_sync_unit() =0;

  /* Media specific recording functions: */
  virtual media_rcs start_recording();
  virtual media_rcs stop_recording();

  virtual media_rcs send_msg(msg_codes code =0, String ="");
  virtual media_rcs send_ctrl();

  media_rcs media_change_state(media_states, u_int32_t =0, int =0);

  inline media_rcs read_pending_sync_unit(u_int32_t *ts) {
    if (media_packet_pending) {
      *ts=media_hdr.ts;
      return MEDIA_OK;
    } else {
      *ts=0;
      return MEDIA_NO_PACKET_PENDING;
    };
  };

  inline media_rcs goto_start() {
    return media_return(media_file.goto_start());
  };

  inline media_rcs goto_end() {
    return media_return(media_file.goto_end());
  };

  inline media_rcs goto_ts(u_int32_t ts) {
    vcr_file_rc rc;
    rc=media_file.goto_ts(ts);
    /* send msg and set new timer */
    media_timeout();
    return media_return(rc);
  };

  media_rcs set_index(u_int8_t , VCR_Index* );

  inline media_rcs delete_index(u_int32_t pos, u_int8_t idx) {
    return media_return(media_file.delete_index(pos, idx));
  };

  inline media_rcs delete_all_indices(u_int8_t idx, int start =0) {
    return media_return(media_file.delete_all_indices(idx));
  };

  inline media_rcs get_first_index(VCR_Index* vcr_idx, u_int8_t idx, int start =1) {
    return media_return(media_file.get_first_index(vcr_idx, idx, start));
  };

  inline media_rcs get_next_index(VCR_Index* vcr_idx_p) {
    return media_return(media_file.get_next_index(vcr_idx_p));
  };

  inline media_rcs goto_next_index(u_int8_t idx) {
    vcr_file_rc rc;
    rc=media_file.goto_next_index(idx);
    /* send msg and set new timer */
    media_timeout();
    return media_return(rc);
  };

  inline media_rcs goto_prev_index(u_int8_t idx) {
    vcr_file_rc rc;
    rc=media_file.goto_prev_index(idx);
    /* send msg and set new timer */
    media_timeout();
    return media_return(rc);
  };

  inline media_rcs goto_next_blank(u_int32_t blank_time, u_int32_t *from =NULL) {
    vcr_file_rc rc;
    rc=media_file.goto_next_blank(blank_time, from);
    /* send msg and set new timer */
    media_timeout();
    return media_return(rc);
  };

  inline media_rcs goto_prev_blank(u_int32_t blank_time, u_int32_t *from =NULL) {
    vcr_file_rc rc;
    rc=media_file.goto_prev_blank(blank_time, from);
    /* send msg and set new timer */
    media_timeout();
    return media_return(rc);
  };

  inline const char *get_error_msg() {
    return (const char *) error_msg;
  };

  inline u_int32_t get_ts() {
    return media_file.timestamp();
  };

  inline u_int32_t get_length() {
    return media_file.length();
  };
  inline media_rcs install_event_hdlr(MBoneSession *ps, MediaEvtHdlr eh, int si) {
    p2session=ps; event_hdlr=eh; session_index=si; return MEDIA_OK;
  };

  inline int is_writable() {
    return media_file.is_writable();
  };

  inline int rec_enabled() {
    return recording_enabled;
  };

  inline media_rcs set_rec_enabled(int rec) {
    if (!is_writable() && rec==1) return MEDIA_IS_READ_ONLY;
    if (recording_enabled!=rec) recording_enabled=rec;
    return MEDIA_OK;
  };

  inline media_rcs set_rec_timeout(int timeout =0) {
    rec_timeout=timeout;
    return MEDIA_OK;
  };

  inline media_rcs set_recording_started(int i) {
    recording_started=i;
    return MEDIA_OK;
  };

  inline int is_recording() {
    return recording_started;
  };

  inline int record_now() {
    return media_state=RECORDING;
  };

  inline  int 	is_timed_out() {
      return timed_out;
  };


protected:
  String	error_msg;
  char		*MEDIA_MAGIC;
  String	media_ipaddr;
  int		media_port;
  u_int16_t	media_confid;
  u_int8_t	media_ttl;
  String	media_filename;
  media_states	media_state;
  media_fmts	media_fmt;
  float		media_speed;
  String        media_name;
  int		media_number;

  MediaTimer	media_timer;
  int		refresh_media_msg;
  String	media_msg;

  MediaTimer	ctrl_timer;
  int32_t	ctrl_packet_size;
  int32_t	avg_ctrl_packet_size;
  int		send_ctrl_period;

  MediaTimer	no_data_timer;
  int		rec_timeout;

  int		nb_of_sockets;
  MediaSocket   media_data_socket;
  MediaSocket   media_session_socket;
  VCRFile	media_file;
  VCR_Hdr	media_hdr;

  char		*media_buffer;
  char		*session_buffer;
  char		*id_buffer;

  int 		media_packet_pending;

  VCRPacketPHPQ	data_queue;
  VCRPacketPHPQ	session_queue;
  SrcInfoAVLSet	src_set;
  Pix		sync_src, next_sync_src;

  u_int32_t	last_ts;

  virtual void 	media_timeout();
  virtual void 	ctrl_timeout();
  virtual void 	no_data_timeout();

  void		data_handle(eventtype);
  virtual void 	media_data_handle(struct sockaddr_in *, int);
  void		session_handle(eventtype);
  virtual void 	media_session_handle(struct sockaddr_in *, int);

  inline int call_event_hdlr(media_rcs event, u_int32_t cargo =0) {
    if (event_hdlr) {
      (p2session->*event_hdlr)(session_index, event, cargo);
    } else {
      cout << "MBoneMedia::call_event_hdlr: event_hdlr == NULL !\n";
    };
  };

  inline media_rcs media_return(vcr_file_rc frc) {
    switch(frc) {
    case VCR_FILE_LAST_PACKET:
      return MEDIA_LAST_PACKET;
      break;
    case VCR_FILE_OK:
      return MEDIA_OK;
      break;
    case VCR_FILE_EOF:
      return MEDIA_EOF;
      break;
    case VCR_FILE_BOF:
      return MEDIA_BOF;
      break;
    case VCR_FILE_ERROR:
      return MEDIA_ERROR;
      break;
    case VCR_FILE_IDX_NOT_FOUND:
      return MEDIA_IDX_NOT_FOUND;
      break;
    default:
      return MEDIA_ERROR;
      break;
    };
    return MEDIA_ERROR;
  };

private:
  int		Media_buffer_size;
  char		*Media_buffer;
  int		Session_buffer_size;
  char		*Session_buffer;
  int		Id_buffer_size;
  char		*Id_buffer;
  MediaEvtHdlr	event_hdlr;
  MBoneSession 	*p2session;
  int		session_index;
  int		recording_enabled;
  int		recording_started;
  int		timed_out;
  int		mute;
};
#endif

