/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* $Id: MCSocket.h,v 1.2 1996/08/02 15:25:47 whd Exp $ */

#ifndef _MCSocket_h
#define _MCSocket_h

extern "C" {
#include <tk.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
}

#include <String.h>
#ifdef HAVE_SYS_SOCKETBUF_H
#include <sys/socketbuf.h>
#endif
#include "it_depends.h"

#define DEFAULT_ADDR	"224.2.2.2"
#define DEFAULT_PORT	8000
#define DEFAULT_TTL	1
#define DEFAULT_LOOP	1
#define DEFAULT_REUSE	1

enum mcs_state { MCS_CLOSED=0, MCS_OPEN=1 }; 

enum eventtype { MCS_READ, MCS_WRITE, MCS_EXCEPTION };

class MCSocket {
public:
  MCSocket();
  virtual ~MCSocket();
  
  int 		set_ipaddr(String i);
  int 		set_port(int p);
  int		set_ttl(u_int8_t t);
  int 		set_loop(u_int8_t l);
  int 		set_reuse(int t);

  inline String		get_ipaddr()		{ return ipaddr; 	};
  inline int		get_port()		{ return port; 		};
  inline u_int8_t	get_ttl()		{ return ttl; 		};
  inline u_int8_t	get_loop()		{ return loop; 		};
  inline int 		get_reuse()		{ return reuseaddr; 	};
  inline mcs_state 	get_state()		{ return my_state; 	};
  inline mcs_state 	set_state(mcs_state s)	{ return my_state=s; 	};
  inline int		get_rfd()		{ return mcs_read; 	};

  inline int read_hdlr_active() { 
    return (read_hdlr_mask&TK_READABLE); 
  };
  inline int write_hdlr_active() { 
    return (write_hdlr_mask&TK_WRITABLE); 
  };
  inline int exception_hdlr_active() { 
    return ((read_hdlr_mask&TK_EXCEPTION)&&(write_hdlr_mask&TK_EXCEPTION)); 
  };

  int open(String, int, u_int8_t, u_int8_t, int);

  inline open() { 
    open(get_ipaddr(),get_port(),get_ttl(),get_loop(),get_reuse()); 
  };

  inline open(String i) { 
    open(i,get_port(),get_ttl(),get_loop(),get_reuse()); 
  };

  inline open(String i, int p) { 
    open(i,p,get_ttl(),get_loop(),get_reuse()); 
  };

  inline open(String i, int p, u_int8_t t) { 
    open(i,p,t,get_loop(),get_reuse()); 
  };

  inline open(String i, int p, u_int8_t t, u_int8_t l) { 
    open(i,p,t,l,get_reuse()); 
  };

  int close();
  inline int is_open() {return (my_state==MCS_OPEN);};
  inline int read_is_open() {return (read_state==MCS_OPEN);};
  inline int write_is_open() {return (write_state==MCS_OPEN);};

  int read_str(String *);
  int write_str(String);
  int mcs_send(char *, int);
  int mcs_recv(char **, int, struct sockaddr_in *);

  int activate_read_hdlr();
  int activate_write_hdlr();
  int activate_exception_hdlr();

  int deactivate_read_hdlr();  
  int deactivate_write_hdlr();
  int deactivate_exception_hdlr();

 protected:
  virtual void handle(eventtype);
  
private:
  String	ipaddr;
  int		port;
  u_int8_t	ttl;
  u_int8_t	loop;
  int		reuseaddr;
  int		mcs_write,mcs_read;
  Tcl_File	mcs_write_tcl, mcs_read_tcl;
  int		use_mcast;
  fd_set	read_fds;
  struct timeval tvs;
  struct timeval *tv;
  mcs_state	my_state;
  mcs_state	read_state;
  mcs_state	write_state;
  int		read_hdlr_mask;
  int		write_hdlr_mask;

  int reopen();
  int open_read();
  int open_write();
  int close_read();
  int close_write();
  static void event_handler(ClientData, int);
};
#endif
