// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Dirk Grunwald (grunwald@cs.uiuc.edu)
    adapted for libg++ by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef MediaSchedulePHPQ_h
#ifdef __GNUG__
#pragma interface
#endif
#define MediaSchedulePHPQ_h 1

#include "MediaSchedule.PQ.h"

#ifndef MediaSchedulePHPQIndex
#define MediaSchedulePHPQIndex unsigned short
#endif

struct MediaSchedulePHPQNode
{
  MediaSchedulePHPQIndex   sibling;
  MediaSchedulePHPQIndex   children;
  MediaSchedule            item;
  char           valid;
};


class MediaSchedulePHPQ : public MediaSchedulePQ
{
  MediaSchedulePHPQNode*  storage;        // table -- freelist in storage[0].sibling
  int           root;
  int           size;

  void          prealloc(int);
  int           check_sibling_list(int);

public:

                MediaSchedulePHPQ(int sz = DEFAULT_INITIAL_CAPACITY);
                MediaSchedulePHPQ(MediaSchedulePHPQ&);
  inline               ~MediaSchedulePHPQ();

  Pix           enq(MediaSchedule& item);
  inline MediaSchedule           deq(); 

  inline MediaSchedule&          front();
  void          del_front();

  inline int           contains(MediaSchedule& item);

  void          clear(); 

  Pix           first(); 
  void          next(Pix& i);
  inline MediaSchedule&          operator () (Pix i);
  void          del(Pix i);
  Pix           seek(MediaSchedule& item);

  int           OK();                    // rep invariant
};


inline MediaSchedulePHPQ::~MediaSchedulePHPQ()
{
  delete [] storage;
}


inline MediaSchedule MediaSchedulePHPQ::deq()
{
  if (count == 0) error("deq of empty PQ");
  MediaSchedule x = storage[root].item;
  del_front();
  return x;
}


inline MediaSchedule& MediaSchedulePHPQ::front()
{
  if (count == 0) error("front of empty PQ");
  return storage[root].item;
}

inline int MediaSchedulePHPQ::contains(MediaSchedule& item)
{
  return seek(item) != 0;
}

inline MediaSchedule& MediaSchedulePHPQ::operator() (Pix p)
{
  if (p == 0) error("null Pix");
  return storage[int(p)].item;
}

#endif
