/*
 * RTP.h
 *
 * Constants and structures based on RFC 1889 
 * This information is still subject to change.
 *
 */

/* $Id: RTP.h,v 1.2 1996/08/02 15:19:47 whd Exp $ */

#ifndef _RTP_H_
#define _RTP_H_

#define RTPv0 0x0000  /* RTP Version O initially implemented in the vat audio tool */
#define RTPv1 0x4000  /* RTP Version 1 */
#define RTPv2 0x8000  /* RTP Version 2 as specified in RFC 1889 */

/* RTP standard content encodings for audio */
#define RTP_PT_PCMU		0	/* 8kHz PCM mu-law mono */
#define RTP_PT_1016		1	/* 8kHz CELP (Fed Std 1016) mono */
#define RTP_PT_G721		2	/* 8kHz G.721 ADPCM mono */
#define RTP_PT_GSM		3	/* 8kHz GSM mono */
#define RTP_PT_G723		4	/* 8kHz G.723 ADPCM mono */
#define RTP_PT_DVI		5	/* 8kHz Intel DVI ADPCM mono */
#define RTP_PT_L16_16		6	/* 16kHz 16-bit linear mono */
#define RTP_PT_L16_44_2		7	/* 44.1kHz 16-bit linear stereo */

/* RTP standard content encodings for video */
#define RTP_PT_SCR		23	/* simple conditional replenishment */
#define RTP_PT_MPEG		24 	/* MPEG II? */
#define RTP_PT_CELLB		25 	/* Sun CellB */
#define RTP_PT_JPEG		26	/* JPEG */
#define RTP_PT_CUSEEME		27	/* Cornell CU-SeeMe */
#define RTP_PT_NV		28	/* Xerox PARC nv */
#define RTP_PT_DVC		29	/* BBN dvc */
#define RTP_PT_BOLT		30	/* Bolter */
#define RTP_PT_H261		31	/* CCITT H.261 */

/* backward compat hack for decoding RTPv1 ivs streams */
#define RTP_PT_H261_COMPAT	127

/* Offset from UNIX's epoch to the NTP epoch in seconds (NTP's JAN_1970) */
#define RTP_EPOCH_OFFSET	2208988800UL

#define RTP_T	0xC000
#define RTP_P	0x2000
#define RTP_X	0x1000
#define RTP_CC	0x0F00
#define RTP_M	0x0080
#define RTP_PT	0x007F

/* Basic RTP header */
struct rtphdr {
	u_int16_t  rh_flags;		/* V:2 P:1 X:1 CC:4 M:1 PT:7 */
	u_int16_t  rh_seqno;		/* sequence number */
	u_int32_t  rh_ts;		/* time stamp (middle of NTP timestamp) */
	u_int32_t  rh_ssrc;		/* synchronization src id */
	/* data sources follow per cc */
};

#define RTCP_T	0xC000
#define RTCP_P	0x2000
#define RTCP_RC	0x1F00
#define RTCP_PT	0x00FF

struct rtcphdr {
	u_int16_t rh_flags;	/* T:2 P:1 RC:5 PT:8 */
	u_int16_t rh_len;	/* length of message (in bytes) */
	u_int32_t rh_ssrc;	/* synchronization src id */
};

typedef struct {
	u_int32_t upper;	/* more significant 32 bits */
	u_int32_t lower;	/* less significant 32 bits */
} ntp64;

/*
 * Sender report.
 */
struct rtcp_sr {
	ntp64 sr_ntp;		/* 64-bit ntp timestamp */
	u_int32_t sr_ts;	/* reference media timestamp */
	u_int32_t sr_np;	/* no. packets sent */
	u_int32_t sr_nb;	/* no. bytes sent */
};

/*
 * Receiver report.
 * Time stamps are middle 32-bits of ntp timestamp.
 */
struct rtcp_rr {
	u_int32_t rr_srcid;	/* sender being reported */
	u_int32_t rr_loss;	/* loss stats (8:fraction, 24:cumulative)*/
	u_int32_t rr_ehsr;	/* ext. highest seqno received */
	u_int32_t rr_dv;	/* jitter (delay variance) */
	u_int32_t rr_lsr;	/* orig. ts from last rr from this src  */
	u_int32_t rr_dlsr;	/* time from recpt of last rr to xmit time */
};

#define RTCP_PT_SR	200	/* sender report */
#define RTCP_PT_RR	201	/* receiver report */
#define RTCP_PT_SDES	202	/* source description */
#define 	RTCP_SDES_CNAME	1	/* official name (mandatory) */
#define 	RTCP_SDES_NAME	2	/* personal name (optional) */
#define 	RTCP_SDES_EMAIL	3	/* e-mail addr (optional) */
#define 	RTCP_SDES_PHONE	4	/* telephone # (optional) */
#define 	RTCP_SDES_LOC	5	/* geographical location */
#define 	RTCP_SDES_TOOL	6	/* name/(vers) of app */
#define 	RTCP_SDES_NOTE	7	/* transient messages */
#define 	RTCP_SDES_PRIV	8	/* private SDES extensions */
#define RTCP_PT_BYE	203	/* end of participation */
#define RTCP_PT_APP	204	/* application specific functions */

#define		RTCP_SDES_MIN	1
#define		RTCP_SDES_MAX	7


/*
 * Parameters controling the RTCP report rate timer.
 */
#define RTCP_SESSION_BW_FRACTION (0.05)
#define RTCP_MIN_RPT_TIME (5.)
#define RTCP_SENDER_BW_FRACTION (0.25)
#define RTCP_RECEIVER_BW_FRACTION (1. - RTCP_SENDER_BW_FRACTION)
#define RTCP_SIZE_GAIN (1./8.)

/*
 * Parameters controling the RTCP report rate timer.
 */
/*
 * Fraction of the rtcp bandwidth to be shared among active senders.
 * (This fraction was chosen so that in a typical session with one or
 * two active senders, the computed report time would be roughly
 * equal to the min report time so that we don't unnecessarily slow
 * down receiver reports.)  The receiver fraction must be 1 - the
 * sender fraction.
 */
#define RTCP_SESSION_BW_FRACTION (0.05)
#define RTCP_SENDER_BW_FRACTION (0.25)
#define RTCP_RECEIVER_BW_FRACTION (1. - RTCP_SENDER_BW_FRACTION)

/*
 * Minimum time between RTCP packets from this site (in seconds).
 * This time prevents the reports from `clumping' when sessions are
 * small and the law of large numbers isn't helping to smooth out
 * the traffic.  it also keeps the report interval from becoming
 * ridiculously small during transient outages like a network
 * partition.
 */
#define RTCP_MIN_RPT_TIME (5.)

/*
 * Gain (smoothing constant) for the low-pass filter that estimates
 * the average rtcp packet size.
 */
#define RTCP_SIZE_GAIN (1./8.)

/*
 * Largest (user-level) packet size generated by our rtp applications.
 * Individual audio/video formats may use smaller mtu's.
 */
#define RTP_MTU 1024

#define MAXHDR 24

/*
 * Motion JPEG encapsulation.
 *
 * 0                   1                   2                   3
 * 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
/*
 * Parameters controling the RTCP report rate timer.
 */
#define RTCP_SESSION_BW_FRACTION (0.05)
#define RTCP_MIN_RPT_TIME (5.)
#define RTCP_SENDER_BW_FRACTION (0.25)
#define RTCP_RECEIVER_BW_FRACTION (1. - RTCP_SENDER_BW_FRACTION)
#define RTCP_SIZE_GAIN (1./8.)

/*
 * Largest (user-level) packet size generated by our rtp applications.
 * Individual video formats may use smaller mtu's.
 */
#define RTP_MTU 1024

#define MAXHDR 24

/*
 * Motion JPEG encapsulation.
 *
 * 0                   1                   2                   3
 * 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |      MBZ      |                frag offset                    |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |      Type     |       Q       |     Width     |     Height    |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 
 * Type = Index into a table of predefined JPEG parameters
 * Width = Width of input in 8-pixel blocks
 * Height = Height of input in 8-pixel blocks
 * Q = Quality factor (0-100 = std, >100 = custom)
 * Frag offset = The byte offset into the frame for the data in 
 * this packet
 */
struct jpeghdr {
	u_int32_t off;		/* fragment offset */
	u_int8_t type;		/* id of jpeg decoder params */
	u_int8_t q;		/* quantization factor (or table id) */
	u_int8_t width;		/* 1/8 frame width */
	u_int8_t height;	/* 1/8 frame height */
};

/*
 * NV encapsulation.
 */
struct nvhdr {
	u_int16_t width;
	u_int16_t height;
	/* nv data */
};

/*
 * CellB encapsulation.
 */
struct cellbhdr {
	u_int16_t x;
	u_int16_t y;
	u_int16_t width;
	u_int16_t height;
	/* cells */
};

/*
 * H.261 encapsulation.
 * See Internet draft.
 *
 *  0                   1                   2                   3
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |SBIT |EBIT |I|V| GOBN  |  MBAP   |  QUANT  |  HMVD   |  VMVD   |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */
#ifdef notdef
struct h261hdr {
	u_int16_t	flags;
	u_int16_t off;
};
#endif

struct bvchdr {
	u_int8_t version;
	u_int8_t pad;
	u_int8_t width;
	u_int8_t height;
	u_int32_t quant;
	u_int8_t sbit;
	u_int8_t ebit;
	u_int16_t blkno;
};

#endif /*_RTP_H*/






