/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* $Id: Timer.h,v 1.1 1996/07/26 15:02:20 whd Exp $ */

#ifndef _TIMER_H_
#define _TIMER_H_

extern "C" {
#include <tk.h>
#include <sys/time.h>
}

#include <DLList.h>

class TimerToken {
public:
  Tk_TimerToken	token;
  ClientData	cd;
  Pix		index;
};

class Timer {

public:
  Timer();
  virtual ~Timer();

  inline void msched(int msecs) { add_to_token_list(msecs); };
  inline void usched(int usecs) { add_to_token_list(usecs/1000); };
  inline void ssched(int secs)  { add_to_token_list(secs*1000); };

  inline void cancel() {
    for (Pix i=token_list.first(); i!=0; token_list.next(i))
    Tk_DeleteTimerHandler(token_list(i).token);
    token_list.clear();
  };

  inline int is_active() { return(token_list.length()>0); };

protected:
  virtual void ttimeout() {};

private:
  DLList<TimerToken>	token_list;

  inline void add_to_token_list(int msecs) {
    static TimerToken tt;
    tt.cd=(ClientData) this;
    token_list.append(tt);
    token_list(token_list.last()).index=token_list.last();
    token_list(token_list.last()).token=Tk_CreateTimerHandler(
		  		        msecs, 
				        dispatch,
				        &(token_list(token_list.last())));
  };

  static void dispatch(ClientData cd) {
    struct TimerToken *ttp = (TimerToken *) cd;
    Timer *timer = (Timer *) ttp->cd;
    timer->token_list.del(ttp->index);
    timer->ttimeout();
  };
  
};
#endif







