/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static const char rcsid[]="$Id: VCRDisplay.cc,v 1.1 1996/07/26 15:02:20 whd Exp $";

extern "C" {
#include <sys/time.h>
#include <math.h>
#include <stdlib.h>
}

#include "VCRTcl.h"
#include "VCRDisplay.h"
#include "it_depends.h"

VCRDisplay* VCRDisplay::instance_;

void VCRDisplay::init() {
  if (instance_!=NULL) {
    cerr << "VCRDisplay::init() VCRDisplay already initialized!\n";
    exit;
  }
  instance_ = new VCRDisplay;
  VCRDisplay& display = VCRDisplay::instance();
  display.init_time();
};

void VCRDisplay::init_time() {
  if (instance_==NULL) {
    cerr << "VCRDisplay::init_time() VCRDisplay not yet initialized!\n";
    exit;
  }
  display_time=0;
  display_offset=0;
  display_start=0;
  auto_scale=0;
  display_direction=0;
  display_speed=1;
  display_on=0;
  ts=0;
  slider_length=0;
};

void VCRDisplay::start(u_int32_t time, float speed, int set_auto_scale) {
  if (instance_==NULL) {
    cerr << "VCRDisplay::start() VCRDisplay not yet initialized!\n";
    exit;
  }
  if (display_on) {
    #ifdef DEBUG1
    cerr << "VCRDisplay::start(" << time << "," << speed << ") => VCRDisplay already on ...\n";
    #endif
  } else {
    #ifdef DEBUG2
    cout << "VCRDisplay::start(" << time << "," << speed << ")\n";
    #endif
    display_on=1;
    auto_scale=set_auto_scale;
    display_start=time;

    set(time, speed);
    UpdateVCRDisplay((ClientData) NULL);
  };
};

void VCRDisplay::stop() {
  if (instance_==NULL) {
    cerr << "VCRDisplay::stop() VCRDisplay not yet initialized!\n";
    exit;
  }
  if (display_on) {
    #ifdef DEBUG2
    cout << "VCRDisplay::stop()\n";
    #endif
    Tk_DeleteTimerHandler(display_token);
    display_time=display_time+display_offset;
    display_start=0;
    display_offset=0;
    display_on=0;
  } else {
    #ifdef DEBUG2
    cout << "VCRDisplay::stop() => VCRDisplay not on ...\n";
    #endif
  };
  auto_scale=0;
};

u_int32_t VCRDisplay::update(float speed) {
  if (instance_==NULL) {
    cerr << "VCRDisplay::update() VCRDisplay not yet initialized!\n";
    exit;
  }

  if (!display_on) return 0;

  #ifdef DEBUG2
  cout << "VCRDisplay::update(" << speed << ")\n";
  #endif

  display_speed=speed;
  if (display_speed<0) display_speed*=-1;

  if (display_start==0) {
    ts=0;
  } else {
    ts=real_time()-display_start;
  };

  if (speed<0) {
    display_direction=-1;
    display_time = u_int32_t(ts*display_speed);
  } else {
    display_direction=1;
    display_time = u_int32_t(ts*display_speed);
  };

  if ((!auto_scale) && (display_time/1000)>slider_length) {
    stop();
  } else {
    set_tcl(display_time);
  };

  return display_time;
};

int VCRDisplay::set_time(u_int32_t time, float speed) {

  #ifdef DEBUG2
  cout << "VCRDisplay::set_time(" << time << "," << speed << ")\n";
  #endif

  if (instance_==NULL) {
    cerr << "VCRDisplay::set_time() VCRDisplay not yet initialized!\n";
    exit;
  }

  display_start=real_time();
  if (display_on) {
    display_speed=speed;
    if (display_speed<0) display_speed*=-1;
    display_direction=(speed<0?-1:1);
  };
  display_offset=time;
  display_time=0;
  return 1;
};

int VCRDisplay::set(u_int32_t time, float speed) {
  #ifdef DEBUG2
  cout << "VCRDisplay::set(" << time << "," << speed << ")\n";
  #endif

  if (instance_==NULL) {
    cerr << "VCRDisplay::set() VCRDisplay not yet initialized!\n";
    exit;
  }

  set_time(time, speed);
  set_tcl(display_time);
  return 1;
};

int VCRDisplay::set_tcl(u_int32_t time) {
  static char cmd[200];
  static int sign;
  static int32_t disp=0, last=0;

  #ifdef DEBUG2
  cout << "VCRDisplay::set_tcl(" << time << "," << speed() << ")\n";
  #endif
  
  if (instance_==NULL) {
    cerr << "VCRDisplay::set_tcl() VCRDisplay not yet initialized!\n";
    exit;
  }

  VCRTcl& tcl = VCRTcl::instance();

  disp=time+display_offset;
  sign=(disp<0);
  disp=abs(disp);

  if((auto_scale) && (u_int32_t(disp/1000)>slider_length)) update_total_length(disp);

  ;
  if ((int32_t) disp/1000 != last) {
    last=(int32_t) disp/1000;
    sprintf(cmd,"SetNumDisplay %s %d %d %d %d %d %d %d", tcl.var("num_display"),
	    sign, 
	    ms2hour(disp)/10, 
	    ms2hour(disp)%10, 
	    ms2mod_min(disp)/10, 
	    ms2mod_min(disp)%10, 
	    ms2mod_sec(disp)/10, 
	    ms2mod_sec(disp)%10);
    tcl.eval(cmd);
    sprintf(cmd,".vcr.position.slider set %d", last);
    tcl.eval(cmd);
    #ifdef DEBUG2
    cout << "VCRDisplay::set_tcl(): " << ms2hour(disp) << ":" << ms2mod_min(disp) << ":" << ms2mod_sec(disp) << "\n";
    #endif
  };
  return 1;
};

void VCRDisplay::update_total_length(u_int32_t ms) {
  static int hours,mins,secs;
  static int ticks;
  static char cmd[200];

  #ifdef DEBUG2
  cout << "VCRDisplay::update_total_length: " << ms << "\n";
  #endif

  if (instance_==NULL) {
    cerr << "VCRDisplay::update_total_length() VCRDisplay not yet initialized!\n";
    exit;
  }

  VCRTcl& tcl = VCRTcl::instance();

  slider_length=u_int32_t(ms/1000);

  hours=ms2hour(ms);
  mins=ms2mod_min(ms);
  secs=ms2mod_sec(ms);
  sprintf(cmd,".vcr.position.total_length.value configure -text \"%.2d:%.2d:%.2d\"",
	  hours,mins,secs);
  tcl.eval(cmd);
  
  if (slider_length==0) 
    ticks=0;
  else if (slider_length<=10) 
    ticks=1;
  else if (slider_length<=20) 
    ticks=2;
  else
    ticks=int((int(slider_length/pow(10,int(log10(slider_length)))) * 
	      pow(10,int(log10(slider_length))))/10);

  sprintf(cmd,".vcr.position.slider configure -to %d -tickinterval %d", slider_length, ticks);
  tcl.eval(cmd);

};

void VCRDisplay::UpdateVCRDisplay(ClientData cd) {
  if (instance_==NULL) {
    cerr << "VCRDisplay::UpdateVCRDisplay() VCRDisplay not yet initialized!\n";
    exit;
  }
  VCRDisplay& display = VCRDisplay::instance();
  display.set_token(Tk_CreateTimerHandler(int(1000/display.speed()), UpdateVCRDisplay, cd));
  display.update(display.speed());
};


