/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* $Id: VCRDisplay.h,v 1.1 1996/07/26 15:02:20 whd Exp $ */

#ifndef _VCRDISPLAY_H_
#define _VCRDISPLAY_H_

extern "C" {
#include <sys/time.h>
#include <tk.h>
}
#include <String.h>
#include "it_depends.h"

class VCRDisplay {

 public:
  static void	init();
  static inline VCRDisplay& instance() { 
    if (instance_==NULL) {
      cerr << "VCRDisplay::instance() VCRDisplay not yet initialized!\n"; 
      exit(1);
    };
    return (*instance_); 
  }

  inline u_int32_t	real_time(struct timeval *tvp) {
    if (instance_==NULL) {
      cerr << "VCRDisplay::real_time() VCRDisplay not yet initialized!\n"; 
      exit(1);
    };
    gettimeofday(tvp,NULL);
    return(tv2ms((*tvp)));
  };

  inline u_int32_t	real_time() {
    if (instance_==NULL) {
      cerr << "VCRDisplay::real_time() VCRDisplay not yet initialized!\n"; 
      exit(1);
    };
    static struct timeval tv;
    gettimeofday(&tv,NULL);
    return tv2ms(tv);
  };
  
  inline u_int32_t get_time() {
    if (instance_==NULL) {
      cerr << "VCRDisplay::get_time() VCRDisplay not yet initialized!\n"; 
      exit(1);
    };
    return (u_int32_t)((display_on)?
	    ((real_time()-display_start)*display_speed+display_offset):
	    (display_time+display_offset));
  }; 

  void 		start(u_int32_t, float =1, int =0);
  void 		stop();
  
  int 		set_time(u_int32_t, float =1);
  int 		set(u_int32_t, float =1);
  void		update_total_length(u_int32_t);
  inline float	speed() { return display_speed; };
  void		set_token(Tk_TimerToken token) { display_token=token; };
  u_int32_t	update(float =1);
  int 		set_tcl(u_int32_t);
  void		init_time();
  
 private:
  static VCRDisplay* instance_;

  Tk_TimerToken display_token;

  u_int32_t	ts;
  u_int32_t	display_time;
  u_int32_t	display_offset;
  u_int32_t	display_start;
  u_int32_t	slider_length;
  int		auto_scale;
  int		display_on;
  int		display_direction;
  float		display_speed;

  static void UpdateVCRDisplay(ClientData);

};

#endif
