/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static const char rcsid[]="$Id: VCRFile.cc,v 1.2 1996/08/08 17:28:43 whd Exp $";

extern "C" {
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>
}
#include "VCRDisplay.h"
#include "VCRFile.h"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
VCRFile::VCRFile(int init_resync_buffer_size) 
{
  #ifdef DEBUG1
  cerr << "VCRFile::VCRFile()\n";
  #endif

  rec_index=0;
  length_ok=VCR_FILE_FALSE;
  size_ok=VCR_FILE_FALSE;
  flush_ok=VCR_FILE_FALSE;
  idx_size=sizeof(VCR_Index);
  index.l=0;
  index.ch=INVALID_CHANNEL;
  index.ts=0;
  index.idx=0;
  read_only=0;
  file_length=0;
  idx_file_size=0;
  filename="";
  indexname="";
  writable=0;
  last_index_pos=0;
  last_index_value=0;
  fp=NULL;
  ip=NULL;
  set_resync_buffer_size(init_resync_buffer_size);
  idx_magic_size=strlen(IDX_MAGIC);
  rec_magic_size=strlen(REC_MAGIC);
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
VCRFile::~VCRFile() {
  #ifdef DEBUG1
  cerr << "VCRFile::~VCRFile()\n";
  #endif
  if (is_open()) close();
};


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::open(String fn, int read_only) {
  int file_mode;
  int no_index=0, no_data=0;

  #ifdef DEBUG2
  cerr << "VCRFile::open().\n";
  #endif

  filename = fn+".rec";
  indexname= fn+".idx";

  ip=NULL;
  fp=NULL;
  
  file_mode=check_stat(&no_index, &no_data);

  if (read_only) {
    file_mode=VCR_FILE_READ_ONLY;
  };

  if (file_mode==VCR_FILE_READ_ONLY) {
    /* open read only */
    read_only=1;
    ip=fopen((const char *) indexname, "r");
    if (ip==NULL) {
      #ifdef DEBUG1
      cerr << "VCRFile::open() cannot read index file (" << indexname <<")\n";
      #endif
      perror("VCRFile:open()/fopen(index)");
      return VCR_FILE_CANNOT_READ_INDEX;
    };
    fp=fopen((const char *) filename, "r");
    if (fp==NULL) {
      fclose(ip);
      #ifdef DEBUG1
      cerr << "VCRFile::open() cannot read data file (" << filename <<")\n";
      #endif
      perror("VCRFile:open()/fopen(data)");
      return VCR_FILE_CANNOT_READ_DATA;
    };
  } else {
    /* open read/write */
    read_only=0;
    if (no_index && no_data) {
      /* create files */
      ip=fopen((const char *) indexname, "w+");
      if (ip==NULL) {
        #ifdef DEBUG2
	cerr << "VCRFile::open() cannot create index file (" << indexname <<")\n";
        #endif
	perror("VCRFile:open()/fopen(data)");
	return VCR_FILE_CANNOT_CREATE_INDEX;
      };

      fwrite(IDX_MAGIC, idx_magic_size, 1, ip);
      char eof = 27;
      fwrite(&eof,1,1,ip);
 
      fp=fopen((const char *) filename, "w+");
      if (fp==NULL) {
        #ifdef DEBUG1
	cerr << "VCRFile::open() cannot create data file (" << filename <<")\n";
        #endif
	fclose(ip);
	ip=NULL;
	perror("VCRFile:open()/fopen(data)");
	return VCR_FILE_CANNOT_CREATE_DATA;
      };

      fwrite(REC_MAGIC, rec_magic_size, 1, fp);
      fwrite(&eof,1,1,fp);
 
    } else if (!no_index && !no_data) {
      /* open existing files */
      ip=fopen((const char *) indexname, "r+");
      if (ip==NULL) {
        #ifdef DEBUG1
	cerr << "VCRFile::open() cannot open index file (" << indexname <<")\n";
        #endif
	perror("VCRFile:open()/fopen(data)");
	return VCR_FILE_CANNOT_OPEN_INDEX;
      };
      fp=fopen((const char *) filename, "r+");
      if (fp==NULL) {
        #ifdef DEBUG1
	cerr << "VCRFile::open() cannot open data file (" << filename <<")\n";
        #endif
	fclose(ip);
	ip=NULL;
	perror("VCRFile:open()/fopen(data)");
	return VCR_FILE_CANNOT_OPEN_DATA;
      };
    } else {
      /* only one file exists ?? */ 
      if (no_data) {
        #ifdef DEBUG1
	cerr << "VCRFile::open() data file (" << filename <<") not found but \n";
	cerr << "                index file (" << indexname << ") exists.\n";
	cerr << "                (remove index file to create)\n";
        #endif
	return VCR_FILE_INDEX_NO_DATA;
      } else if (no_index) {
        #ifdef DEBUG1
	cerr << "VCRFile::open() index file (" << indexname <<") not found but \n";
	cerr << "                data file (" << filename << ") exists.\n";
	cerr << "                (remove data file to create)\n";
        #endif
	return VCR_FILE_DATA_NO_INDEX;
      };
    };
  };

  fseek(ip,0,0);
  fseek(fp,0,0);

  char *check_magic;
  check_magic=(char *) malloc(idx_magic_size+1);
  fread(check_magic, idx_magic_size, 1, ip);
  check_magic[idx_magic_size]='\0';
  if (strcmp(check_magic, IDX_MAGIC)) {
    free(check_magic); 
    return VCR_FILE_IDX_MAGIC_ERROR;
  };
  free(check_magic); 
  /* jump over the "EOF" char */
  fseek(ip,1,1);

  check_magic=(char *) malloc(rec_magic_size+1);
  fread(check_magic, rec_magic_size, 1, fp);
  check_magic[rec_magic_size]='\0';
  if (strcmp(check_magic, REC_MAGIC)) {
    free(check_magic); 
    return VCR_FILE_IDX_MAGIC_ERROR;
  };
  free(check_magic);
  /* jump over the "EOF" char */
  fseek(fp,1,1);

  if (get_file_size()>idx_magic_size+1) {
    if (fread((char *) &index, idx_size, 1, ip) != 1) {
      #ifdef DEBUG1
      cerr << "VCRFile::open() Error reading index.\n";
      #endif
      index.ts=0;
      return VCR_FILE_INDEX_READ_ERROR;
    };
  } else {
    index.ts=htonl(0);
    index.ch=INVALID_CHANNEL;
    index.l=htonl(0);
    index.idx=0;
    index.offset=htonl(0);
  };
  return vcr_file_rc(file_mode);

};


vcr_file_rc VCRFile::check_stat(int *no_index, int *no_data) {
  int uid;
  int gid;
  int ret;
  int file_mode;
  struct stat fbuf,dbuf;
  extern int errno;
  errno=0;

  #ifdef DEBUG2
  cerr << "VCRFile::check_stat().\n";
  #endif

  writable=0;
  uid=geteuid();
  gid=getegid();

  if (indexname.length()==0 || filename.length()==0) return VCR_FILE_ERROR;

  /* Check Index File */
  if (ip==NULL) 
    ret=stat((const char *) indexname, &fbuf);
  else 
    ret=fstat(int(ip), &fbuf);

  if (ret==-1) {
    if (errno==ENOENT) {
      String basename;
      int i;
      i=indexname.index("/",-1);
      if (i==0) {
	basename=indexname;
      } else {
	basename=indexname.at(0,i);
      };

      if ((ret=stat((const char *) basename, &dbuf))==-1) return VCR_FILE_PATH_NOT_FOUND;
      if (!S_ISDIR(dbuf.st_mode))  return VCR_FILE_PATH_NOT_FOUND;
      if (((dbuf.st_uid==uid) && !((dbuf.st_mode&S_IRWXU)&S_IXUSR)) &&
	  ((dbuf.st_gid==gid) && !((dbuf.st_mode&S_IRWXG)&S_IXGRP)) &&
	  !((dbuf.st_mode&S_IRWXO)&S_IXOTH)) return VCR_FILE_NO_X_PERM;

      if (((dbuf.st_uid==uid) && !((dbuf.st_mode&S_IRWXU)&S_IRUSR)) &&
	  ((dbuf.st_gid==gid) && !((dbuf.st_mode&S_IRWXG)&S_IRGRP)) &&
	  !((dbuf.st_mode&S_IRWXO)&S_IROTH)) return VCR_FILE_NO_READ_PERM;

      if (((dbuf.st_uid==uid) && !((dbuf.st_mode&S_IRWXU)&S_IWUSR)) &&
	  ((dbuf.st_gid==gid) && !((dbuf.st_mode&S_IRWXG)&S_IWGRP)) &&
	  !((dbuf.st_mode&S_IRWXO)&S_IWOTH)) return VCR_FILE_NO_WRITE_PERM;

      if (no_index) *no_index=1;
      file_mode=VCR_FILE_READ_WRITE;

    } else {
      perror("VCRFile::open()/stat(index)");
      return VCR_FILE_STAT_ERROR;
    };
  } else {
    if (no_index) *no_index=0;
    if (((fbuf.st_uid==uid) && !((fbuf.st_mode&S_IRWXU)&S_IRUSR)) &&
	((fbuf.st_gid==gid) && !((fbuf.st_mode&S_IRWXG)&S_IRGRP)) &&
	!((fbuf.st_mode&S_IRWXO)&S_IROTH)) return VCR_FILE_NO_READ_PERM;

    if (((fbuf.st_uid==uid) && ((fbuf.st_mode&S_IRWXU)&S_IWUSR)) ||
	((fbuf.st_gid==gid) && ((fbuf.st_mode&S_IRWXG)&S_IWGRP)) ||
	((fbuf.st_mode&S_IRWXO)&S_IWOTH)) {
      file_mode=VCR_FILE_READ_WRITE;
    } else {
      file_mode=VCR_FILE_READ_ONLY;
    };
  };


  /* Check Data File */
  if (fp==NULL) 
    ret=stat((const char *) filename, &fbuf);
  else 
    ret=fstat(int(fp), &fbuf);

  if (ret==-1) {
    if (errno==ENOENT) {

      if (file_mode!=VCR_FILE_READ_WRITE) return  VCR_FILE_NO_WRITE_PERM;

      String basename;
      int i;
      i=filename.index("/",-1);
      if (i==0) {
	basename=filename;
      } else {
	basename=filename.at(0,i);
      };

      if ((ret=stat((const char *) basename, &dbuf))==-1) return VCR_FILE_PATH_NOT_FOUND;
      if (!S_ISDIR(dbuf.st_mode))  return VCR_FILE_PATH_NOT_FOUND;
      if (((dbuf.st_uid==uid) && !((dbuf.st_mode&S_IRWXU)&S_IXUSR)) &&
	  ((dbuf.st_gid==gid) && !((dbuf.st_mode&S_IRWXG)&S_IXGRP)) &&
	  !((dbuf.st_mode&S_IRWXO)&S_IXOTH)) return VCR_FILE_NO_X_PERM;

      if (((dbuf.st_uid==uid) && !((dbuf.st_mode&S_IRWXU)&S_IRUSR)) &&
	  ((dbuf.st_gid==gid) && !((dbuf.st_mode&S_IRWXG)&S_IRGRP)) &&
	  !((dbuf.st_mode&S_IRWXO)&S_IROTH)) return VCR_FILE_NO_READ_PERM;

      if (((dbuf.st_uid==uid) && !((dbuf.st_mode&S_IRWXU)&S_IWUSR)) &&
	  ((dbuf.st_gid==gid) && !((dbuf.st_mode&S_IRWXG)&S_IWGRP)) &&
	  !((dbuf.st_mode&S_IRWXO)&S_IWOTH)) return VCR_FILE_NO_WRITE_PERM;

      if (no_data) *no_data=1;
      file_mode=VCR_FILE_READ_WRITE;

    } else {
      perror("VCRFile::open()/stat(data)");
      return VCR_FILE_STAT_ERROR;
    };
  } else {
    if (no_data) *no_data=0;
    if (((fbuf.st_uid==uid) && !((fbuf.st_mode&S_IRWXU)&S_IRUSR)) &&
	((fbuf.st_gid==gid) && !((fbuf.st_mode&S_IRWXG)&S_IRGRP)) &&
	!((fbuf.st_mode&S_IRWXO)&S_IROTH)) return VCR_FILE_NO_READ_PERM;

    if (((fbuf.st_uid==uid) && ((fbuf.st_mode&S_IRWXU)&S_IWUSR)) ||
	((fbuf.st_gid==gid) && ((fbuf.st_mode&S_IRWXG)&S_IWGRP)) ||
	((fbuf.st_mode&S_IRWXO)&S_IWOTH)) {
      /* don't change file_mode (if it was READ_ONLY than it can't be changed
         if it is already READ_WRITE than it's fine...) */
    } else {
      file_mode=VCR_FILE_READ_ONLY;
    };
  };

  if (file_mode==VCR_FILE_READ_WRITE) writable=1;
  return vcr_file_rc(file_mode);
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::close() {
  
  #ifdef DEBUG2
  cerr << "VCRFile::close().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  flush(1);
  if (fp) fclose(fp);
  if (ip) fclose(ip);

  index.l=0;
  index.ch=INVALID_CHANNEL;
  index.ts=0;
  index.idx=0;

  read_only=1;
  writable=0;
  fp=NULL;
  ip=NULL;
  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* protected */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
long VCRFile::get_file_size() {

  #ifdef DEBUG2
  cerr << "VCRFile::get_file_size().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  if (size_ok==VCR_FILE_FALSE) {
    flush();
    long old_pos=ftell(ip);
    fseek(ip, 0, 2);
    idx_file_size=ftell(ip);
    fseek(ip, old_pos, 0);
    size_ok=VCR_FILE_TRUE;
  };
  return idx_file_size;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
u_int32_t VCRFile::length() {

  #ifdef DEBUG2
  cerr << "VCRFile::length().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  if (length_ok==VCR_FILE_FALSE) {
    if (get_file_size()>=(idx_size+idx_magic_size+1)) {
      flush();
      long old_pos=ftell(ip);
      goto_end();
      file_length=ntohl(index.ts);
      if (old_pos>idx_magic_size+1) fseek(ip,old_pos-idx_size,0);
      if (fread((char *) &index, idx_size, 1, ip) != 1) {
        #ifdef DEBUG1
	cerr << "VCRFile::length() Error reading index.\n";
        #endif
	index.ts=0;
	return VCR_FILE_INDEX_READ_ERROR;
      };
      if (old_pos==idx_magic_size+1) fseek(ip,idx_magic_size+1,0);
    } else {
      file_length=0;
    };
    length_ok=VCR_FILE_TRUE;
  };

  return file_length;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::goto_start() {
  if (ip==NULL) return VCR_FILE_NOT_OPEN;

  #ifdef DEBUG2
  cerr << "VCRFile::goto_start().\n";
  #endif

  flush();

  fseek(ip,idx_magic_size+1,0);

  if (get_file_size()>idx_magic_size+1) {
    if (fread((char *) &index, idx_size, 1, ip) != 1) {
      #ifdef DEBUG1
      cerr << "VCRFile::goto_start() Error reading index.\n";
      #endif
      index.ts=0;
      return VCR_FILE_INDEX_READ_ERROR;
    };
  } else {
    #ifdef DEBUG1
    cerr << "VCRFile::goto_start() Indexfile seems to be empty?.\n";
    #endif
    index.ts=htonl(0);
    index.ch=INVALID_CHANNEL;
    index.l=htons(0);
    index.idx=0;
    index.offset=htonl(0);
  };
  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::goto_end() {

  #ifdef DEBUG2
  cerr << "VCRFile::goto_end().\n";
  #endif

  flush();

  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  fseek(ip,0,2);
  if (ftell(ip) >= (idx_size+idx_magic_size+1)) { 
    fseek(ip,-idx_size, 1);
    if (fread((char *) &index, idx_size, 1, ip)!= 1) {
      #ifdef DEBUG1
      cerr << "VCRFile::goto_end() Error reading index.\n";
      #endif
      index.ts=0;
      return VCR_FILE_INDEX_READ_ERROR;
    };
    return VCR_FILE_OK;
  } else {
    index.ts=0;
    index.l=0;
    index.ch=INVALID_CHANNEL;
    index.idx=0;
    return VCR_FILE_OK;
  };
};


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::read(VCR_Hdr *p, char **data) {
  vcr_file_rc rc;

  #ifdef DEBUG2
  cerr << "VCRFile::read().\n";
  #endif

  flush();

  *data=NULL;

  p->ts=ntohl(index.ts);
  p->l=ntohs(index.l);
  p->ch=index.ch;
  p->idx=index.idx;

  if (ntohs(index.l) > 0) {
    if (*data) free(*data);
    (*data) = (char *) malloc(ntohs(index.l));
    rc=read_data(data);
    if (rc>VCR_FILE_OK) {
      return rc;
    };
  };

  /* read the next header so we have immediate access to it */
  return read_next_hdr();

};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::write(VCR_Hdr *new_hdr, char *data) {

  VCRPacket p;

  #ifdef DEBUG2
  cerr << "VCRFile::write().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  
  if (new_hdr!=NULL) {
    
    flush_ok=VCR_FILE_FALSE;

    if (resync_buffer_size && (pq.length()>=resync_buffer_size)) {
      while (pq.length()>=resync_buffer_size) {
	p=pq.deq();
	write_pq(&p);
      };
    };

    memcpy(&(p.vcr_hdr), new_hdr, sizeof(p.vcr_hdr));

    /* if we were asked to write an index by the function set_index() do it now */
    if (rec_index) {
      p.vcr_hdr.idx |= rec_index;
      rec_index = 0;
    };

    if (data && p.vcr_hdr.l) { 
      p.data=(char *) malloc(p.vcr_hdr.l); 
      memcpy(p.data, data, p.vcr_hdr.l); 
    } else {
      p.data=NULL;
    }

    if (resync_buffer_size) { 
      pq.enq(p);
    } else {
      write_pq(&p);
    };
  };

  return VCR_FILE_OK;
};


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* write the data to disk ... */
vcr_file_rc VCRFile::write_pq(VCRPacket *p) {

  #ifdef DEBUG2
  cerr << "VCRFile::write_pq().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;

  /* right now: write only at the end... */
  fseek(fp, 0, 2);
  
  if (index.ch==TIME_CHANNEL) {
    if (p->vcr_hdr.ch==TIME_CHANNEL) {
      /* if the previous index packet is a "dummy" and so is this, overwrite it.        */
      /* However, if the old information is more accurate, just throw the new one away. */ 
      if (ntohl(index.ts)<p->vcr_hdr.ts) fseek(ip, -idx_size, 1); else return VCR_FILE_OK;
    } else {
      /* the new packet is data, so just overwrite the "dummy" index */
      fseek(ip, -idx_size, 1);
    };
  };
  
  index.ts=htonl(p->vcr_hdr.ts);
  index.ch=p->vcr_hdr.ch;
  index.idx=p->vcr_hdr.idx;
  index.l=htons(p->vcr_hdr.l);
  index.offset=htonl(ftell(fp));
  /* write index ... */
  if (fwrite((char *) &index, idx_size, 1, ip) != 1) {
    #ifdef DEBUG1
    cerr << "VCRFile::write_pq() error writing vcr-index!\n";
    #endif
    return VCR_FILE_INDEX_WRITE_ERROR;
  }

  if ((p->vcr_hdr.l>0) && (p->data!=NULL)) {
    /* write data ... */
    if (fwrite(p->data, 1, p->vcr_hdr.l, fp) != p->vcr_hdr.l) {
      #ifdef DEBUG1
      cerr << "VCRFile:: error writing vcr-data!\n";
      #endif
      free(p->data);
      return VCR_FILE_DATA_WRITE_ERROR;
    };
    /* free the memory allocated earlier */
    free(p->data);
    #ifdef DEBUG1
  } else {
    cout << "VCRFile::write_pq(): p->data==NULL\n";
    #endif
  };
  
  file_length=p->vcr_hdr.ts;
  length_ok=VCR_FILE_TRUE;
  idx_file_size=ftell(ip);
  size_ok=VCR_FILE_TRUE;
  flush_ok=VCR_FILE_FALSE;

  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* flush the priority queue ... */
vcr_file_rc VCRFile::flush_pq() {
  VCRPacket p;

  #ifdef DEBUG2
  cerr << "VCRFile::flush_pq().\n";
  #endif

  while (pq.length()>0) {
    p=pq.deq();
    write_pq(&p);
  };
  pq.clear();
  return VCR_FILE_OK;
};


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* skip to the next header */
vcr_file_rc VCRFile::skip_to_next_hdr() {
  #ifdef DEBUG2
  cerr << "VCRFile::skip_to_next_hdr().\n";
  #endif
  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  if (feof(ip)) {
#ifdef DEBUG1
      cerr << "VCRFile::skip_to_next_header(): EOF" << endl;
#endif
    return VCR_FILE_EOF;
    }
  flush();
  if (fseek(ip, idx_size, 1)==0) return VCR_FILE_OK;
  return VCR_FILE_ERROR;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* skip to the prev header */
vcr_file_rc VCRFile::skip_to_prev_hdr() {
  #ifdef DEBUG2
  cerr << "VCRFile::skip_to_prev_hdr().\n";
  #endif
  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  flush();
  if ((ftell(ip)-idx_size)<(idx_magic_size+1)) return VCR_FILE_BOF;
  if ((fseek(ip, -idx_size, 1)==0) && (ftell(ip)>=(idx_magic_size+1))) return VCR_FILE_OK;
  fseek(ip,idx_magic_size+1,0);
  return VCR_FILE_BOF;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::read_next_hdr() {

  #ifdef DEBUG2
  cerr << "VCRFile::read_next_hdr().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;

  flush();
  if (fread((char *) &index, idx_size, 1, ip) != 1) {
    if (feof(ip)) {
#ifdef DEBUG1
      cerr << "VCRFile::read_next_header(): EOF" << endl;
#endif
      return VCR_FILE_EOF;
    }
    #ifdef DEBUG1
    cerr << "VCRFile::read_next_header() Error reading index.\n";
    #endif
    return VCR_FILE_INDEX_READ_ERROR;
  };
  if (feof(ip)) {
#ifdef DEBUG1
      cerr << "VCRFile::read_next_hdr(): LAST_PACKET" << endl;
#endif
    return VCR_FILE_LAST_PACKET;
  }
  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::read_prev_hdr() {

  #ifdef DEBUG2
  cerr << "VCRFile::read_prev_hdr().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  flush();

  if ((fseek(ip, -2*idx_size, 1)==0) && (ftell(ip)>=(idx_magic_size+1))) return read_next_hdr();

  fseek(ip,idx_magic_size+1,0);
  return VCR_FILE_BOF;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::read_data(char **data) {

  #ifdef DEBUG2
  cerr << "VCRFile::read_data().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;

  if (ntohs(index.l)==0) return VCR_FILE_OK;
  
  fseek(fp, ntohl(index.offset), 0);

  if (fread((*data), ntohs(index.l), 1, fp) != 1) {
    if (feof(fp)) {
#ifdef DEBUG1
      cerr << "VCRFile::read_data(): EOF" << endl;
#endif
      return VCR_FILE_EOF;
    }
    return VCR_FILE_DATA_READ_ERROR;
  }; 
  if (feof(fp)) {
#ifdef DEBUG1
      cerr << "VCRFile::read_data(): LAST_PACKET" << endl;
#endif
    return VCR_FILE_LAST_PACKET;
  }
  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::write_index(u_int8_t idx, VCR_Index *vcr_idx_p) {
  vcr_file_rc rc;

  #ifdef DEBUG2
  cerr << "VCRFile::write_index().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  if (is_read_only()) return VCR_FILE_READ_ONLY;
  
  if ((rc=skip_to_prev_hdr())!=VCR_FILE_OK) return rc;

  index.idx |= idx;

  if (fwrite((char *) &index, idx_size, 1, ip) != 1) {
    return VCR_FILE_INDEX_WRITE_ERROR;
  };
  
  vcr_idx_p->ts=ntohl(index.ts);
  vcr_idx_p->offset=ntohl(index.offset);
  vcr_idx_p->l=ntohs(index.l);
  vcr_idx_p->ch=index.ch;
  vcr_idx_p->idx=index.idx;

  flush_ok=VCR_FILE_FALSE;
  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::set_index(u_int8_t idx, VCR_Index *vcr_idx_p) {

  #ifdef DEBUG2
  cerr << "VCRFile::set_index().\n";
  #endif
  
  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  if (is_read_only()) return VCR_FILE_READ_ONLY;

  if (pq.length() == 0) {
    rec_index |= idx;
    VCRDisplay& display = VCRDisplay::instance();
    vcr_idx_p->ts=display.get_time();
    vcr_idx_p->offset=0;
    vcr_idx_p->l=0;
    vcr_idx_p->ch=0;
    vcr_idx_p->idx=rec_index;
  } else {
    pq.front().vcr_hdr.idx|=idx;
    vcr_idx_p->ts=pq.front().vcr_hdr.ts;
    vcr_idx_p->idx=pq.front().vcr_hdr.idx;
    vcr_idx_p->offset=0;
    vcr_idx_p->l=0;
    vcr_idx_p->ch=0;
  };

  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::delete_index(u_int32_t pos, u_int8_t idx) {
  vcr_file_rc rc;

  #ifdef DEBUG2
  cerr << "VCRFile::delete_index().\n";
  #endif
  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  if (is_read_only()) return VCR_FILE_READ_ONLY;

  if ((rc=goto_ts(pos)) != VCR_FILE_OK) return rc;
  if (skip_to_prev_hdr()!=VCR_FILE_OK) return VCR_FILE_ERROR;
  index.idx = (index.idx ^ idx) & index.idx;

  if (fwrite((char *) &index, idx_size, 1, ip) != 1) {
    #ifdef DEBUG1
    cerr << "VCRFile::delete_index()" << "error writing vcr-index!\n";
    #endif
    return VCR_FILE_INDEX_WRITE_ERROR;
  };

  flush(1);

  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::delete_all_indices(u_int8_t idx, int start) {
  int32_t old_pos, next_pos;

  #ifdef DEBUG2
  cerr << "VCRFile::delete_all_indices().\n";
  #endif
  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  if (is_read_only()) return VCR_FILE_READ_ONLY;
  
  old_pos=ftell(ip);
  if (start) fseek(ip,idx_magic_size+1,0);
  
  while (read_next_hdr()==VCR_FILE_OK) {
    /* ... give X a chance ... */
    Tk_DoOneEvent(TK_X_EVENTS|TK_DONT_WAIT);

    if (index.idx & idx) {
      next_pos=ftell(ip);
      if (skip_to_prev_hdr()!=VCR_FILE_OK) {
        #ifdef DEBUG1
	cerr << "VCRFile::delete_all_indices()" << "error skiping to prev hdr!\n";
        #endif
	return VCR_FILE_ERROR;
      };
      index.idx = (index.idx ^ idx) & index.idx;
      #ifdef DEBUG2
      cerr << "VCRFile::delete_all_indices() idx at ts: " << ntohl(index.ts) << " deleted.\n";
      #endif
      if (fwrite((char *) &index, idx_size, 1, ip) != 1) {
        #ifdef DEBUG1
	cerr << "VCRFile::delete_all_indices()" << "error writing vcr-index!\n";
        #endif
	return VCR_FILE_INDEX_WRITE_ERROR;
      };
      fseek(ip,next_pos,0);
    };
  };
  fseek(ip,old_pos,0);
  skip_to_prev_hdr();
  read_next_hdr();
  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::get_first_index(VCR_Index *vcr_idx_p, u_int8_t idx, int start) {

  #ifdef DEBUG2
  cerr << "VCRFile::get_first_index(vcr_idx, " << int(idx) << ", " << start << ").\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  
  if (start==1) {
    last_index_pos=idx_magic_size+1;
  } else {
    last_index_pos=ftell(ip);
  };
  last_index_value=idx;
  return get_next_index(vcr_idx_p);
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::get_next_index(VCR_Index *vcr_idx) {
  vcr_file_rc rc;
  long old_pos;
  struct VCR_Index old_index;

  #ifdef DEBUG2
  cerr << "VCRFile::get_next_index().\n";
  #endif

  vcr_idx->ts=0;
  vcr_idx->offset=0;
  vcr_idx->l=0;
  vcr_idx->ch=0;
  vcr_idx->idx=0;
  
  if (ip==NULL) return VCR_FILE_NOT_OPEN;

  old_pos=ftell(ip);
  bcopy((char *) &index, (char *) &old_index, sizeof(index));
  fseek(ip,last_index_pos,0);
  rc=read_next_hdr();

  while (rc == VCR_FILE_OK) { 

    /* everything that matches last_index_value is ok */
    if (index.idx & last_index_value) {
      vcr_idx->ts=ntohl(index.ts);
      vcr_idx->offset=ntohl(index.offset);
      vcr_idx->l=ntohs(index.l);
      vcr_idx->ch=index.ch;
      vcr_idx->idx=index.idx;
      
      last_index_pos=ftell(ip);
      fseek(ip,old_pos,0);
      bcopy((char *) &old_index, (char *) &index,sizeof(index));
      
      return VCR_FILE_OK;
    }
    
    rc=read_next_hdr();
    
  }
  
  last_index_pos=ftell(ip);
  fseek(ip,old_pos,0);
  bcopy((char *) &old_index,(char *) &index,sizeof(index));
  
  return rc;

};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::goto_next_index(u_int8_t idx) {
  vcr_file_rc rc;
  long old_pos;
  struct VCR_Index old_index;

  #ifdef DEBUG2
  cerr << "VCRFile::goto_next_index().\n";
  #endif
  if (ip==NULL) return VCR_FILE_NOT_OPEN;

  old_pos=ftell(ip);
  bcopy((char *) &index, (char *) &old_index, sizeof(index));

  while ((rc=read_next_hdr())==VCR_FILE_OK) {
    if (ntohl(index.ts)>=length()) {
      /* actually this should never happen... */
      rc=VCR_FILE_LAST_PACKET;
      break;
    } else if (index.idx & idx) {
      return VCR_FILE_OK;
    };
  };

  fseek(ip, old_pos, 0);
  bcopy((char *) &old_index, (char *) &index,sizeof(index));

  if (rc==VCR_FILE_LAST_PACKET) {
    return VCR_FILE_IDX_NOT_FOUND;
  };
  return rc;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::goto_prev_index(u_int8_t idx) {
  vcr_file_rc rc;
  long old_pos;
  struct VCR_Index old_index;

  #ifdef DEBUG2
  cerr << "VCRFile::goto_prev_index().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;

  old_pos=ftell(ip);
  bcopy((char *) &index, (char *) &old_index,sizeof(index));

  while ((rc=read_prev_hdr())==VCR_FILE_OK) {
    if (ntohl(index.ts)<=0) {
      /* actually this should never happen... */
      rc=VCR_FILE_BOF;
      break;
    } else if (index.idx & idx) {
      return VCR_FILE_OK;
    };
  };

  fseek(ip, old_pos, 0);
  bcopy((char *) &old_index, (char *) &index,sizeof(index));

  if (rc==VCR_FILE_BOF) {
    return VCR_FILE_IDX_NOT_FOUND;
  };
  return rc;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::goto_next_blank(u_int32_t blank_time, u_int32_t *from) {
  vcr_file_rc rc;
  long old_pos;
  u_int32_t last_ts, start_at;
  struct VCR_Index old_index;
  

  #ifdef DEBUG2
  cerr << "VCRFile::goto_next_blank().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;

  old_pos=ftell(ip);
  bcopy((char *) &index, (char *) &old_index,sizeof(index));

  if (from==NULL) {
    start_at=ntohl(index.ts);
  } else {
    start_at=*from;
    goto_ts(start_at);
  };

  last_ts=ntohl(index.ts);
  read_prev_hdr();

  /* if we are already at the end of a gap we rather jump to the next one */
  if (last_ts-ntohl(index.ts)>=blank_time) {
    cout << "was already at the end of a gap\n";
    read_next_hdr();
  };

  last_ts=ntohl(index.ts);
  while ((rc=read_next_hdr())<=VCR_FILE_OK) {
    if (ntohl(index.ts)>=length()) {
      /* actually this should never happen... */
      rc=VCR_FILE_LAST_PACKET;
      break;
    } else if ((ntohl(index.ts)-last_ts) >= blank_time) {
      return VCR_FILE_OK;
    };
    last_ts=ntohl(index.ts);
  };

  if (rc==VCR_FILE_LAST_PACKET) {
    fseek(ip, old_pos, 0);
    bcopy((char *) &old_index, (char *) &index,sizeof(index));
    return VCR_FILE_IDX_NOT_FOUND;
  };

  return rc;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
vcr_file_rc VCRFile::goto_prev_blank(u_int32_t blank_time, u_int32_t *from) {
  vcr_file_rc rc;
  long old_pos;
  u_int32_t last_ts, start_at;
  struct VCR_Index old_index;

  #ifdef DEBUG2
  cerr << "VCRFile::goto_prev_blank().\n";
  #endif

  if (ip==NULL) return VCR_FILE_NOT_OPEN;
  
  old_pos=ftell(ip);
  bcopy((char *) &index, (char *) &old_index,sizeof(index));

  if (from==NULL) {
    start_at=ntohl(index.ts);
  } else {
    start_at=*from;
    goto_ts(start_at);
  };

  last_ts=ntohl(index.ts);
  if (read_prev_hdr()==VCR_FILE_BOF) {
    return VCR_FILE_IDX_NOT_FOUND;
  };
  
  /* if we are already at the end of a gap we rather jump to the previous one */
  if (last_ts-ntohl(index.ts)>=blank_time) { 
    #ifdef DEBUG2
    cout << "VCRFile::goto_prev_blank(): already at the end of a gap\n";
    #endif
    read_prev_hdr();
  };

  last_ts=ntohl(index.ts);
  while ((rc=read_prev_hdr())<=VCR_FILE_OK) {
    if (ntohl(index.ts)<=0) {
      /* actually this should never happen... */
      rc=VCR_FILE_BOF;
      break;
    } else if ((last_ts-ntohl(index.ts)) >= blank_time) {
      if (read_next_hdr()>VCR_FILE_OK) {
	fseek(ip, old_pos, 0);
	bcopy((char *) &old_index, (char *) &index, sizeof(index));
	return VCR_FILE_ERROR;
      };
      return VCR_FILE_OK;
    };
    last_ts=ntohl(index.ts);
  };

  if (rc==VCR_FILE_BOF) {
    fseek(ip, old_pos, 0);
    bcopy((char *) &old_index, (char *) &index,sizeof(index));
    return VCR_FILE_IDX_NOT_FOUND;
  };

  return rc;
};


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* goto a specific timestamp (or nearby) */
vcr_file_rc VCRFile::goto_ts(u_int32_t ts, int32_t *diffp) {
  long first,last,i;
  int found;
  vcr_file_rc rc;

  #ifdef DEBUG2
  cerr << "VCRFile::goto_ts().\n";
  #endif
  if (ip==NULL) return VCR_FILE_NOT_OPEN;

  if (ntohl(index.ts)==ts) return VCR_FILE_OK;
  if (ts==0) return goto_start();
  if (ts==length()) return goto_end();
  if (ts>length()) {
    #ifdef DEBUG1
    cerr << "VCRFile::goto_ts(" << ts << ") ts > length (" << length() << ").\n";
    #endif
    goto_end();
    return VCR_FILE_ERROR;
  };

  if (ntohl(index.ts)<ts) {
    first=long((ftell(ip)-(idx_magic_size+1))/idx_size);
    last=long((get_file_size()-(idx_magic_size+1))/idx_size);
  } else {
    first=1;
    last=long((ftell(ip)-(idx_magic_size+1))/idx_size);
  };
    
  found=0;

  /* perform binary search: */  
  while (first<=last && !found) {
    i=long((first+last)/2);
    fseek(ip,(idx_magic_size+1)+((i-1)*idx_size),0);
    if (fread((char *) &index, idx_size, 1, ip)!= 1) {
      #ifdef DEBUG1
      cerr << "VCRFile::goto_ts() Error reading index.\n";
      #endif
      index.ts=0;
      return VCR_FILE_INDEX_READ_ERROR;
    };
    if (ts==ntohl(index.ts)) found=1;
    else if (ts<ntohl(index.ts)) last=i-1;
    else first=i+1;
  };

  /* goto the very first packet after the packet with the timestamp we are looking for */
  if (ntohl(index.ts)<ts) {
    do {
      rc = read_next_hdr();
    } while ((rc<=VCR_FILE_OK) && (ntohl(index.ts)<ts));

    if (rc>VCR_FILE_OK) {
      if (rc==VCR_FILE_LAST_PACKET) return goto_end();
      if (rc==VCR_FILE_BOF) return goto_start();
      return rc;
    };
  }; 

  if (diffp) *diffp=int32_t(ntohl(index.ts)-ts);
  #ifdef DEBUG2
  cerr << "goto_ts ts: " << ts << " index.ts: " << ntohl(index.ts) << 
          " diff: " << ntohl(index.ts)-ts << "\n";
  #endif
  
  return VCR_FILE_OK;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



