/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* $Id: VCRFile.h,v 1.1 1996/07/26 15:02:20 whd Exp $ */

#ifndef _VCRFILE_H
#define _VCRFILE_H

#define MBONE_VCR_MAGIC "<MBONE_VCR 1.0>"
#define IDX_MAGIC "<MBONE_VCR_IDX 1.0>"
#define REC_MAGIC "<MBONE_VCR_REC 1.0>"

#include <String.h>

extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <errno.h>
}

#include "VCRPacket.PHPQ.h"
#include "it_depends.h"

typedef struct VCR_Index {
  u_int32_t	ts;
  u_int32_t	offset;
  u_int16_t	l;
  u_int8_t	ch;
  u_int8_t	idx;
};

enum vcr_file_rc { VCR_FILE_READ_ONLY		= 1,
		   VCR_FILE_READ_WRITE		= 2,
		   VCR_FILE_LAST_PACKET		= 3,
		   VCR_FILE_OK			= 4,

		   VCR_FILE_TRUE		= 10,
		   VCR_FILE_FALSE		= 11,

		   VCR_FILE_EOF			= 20, 
		   VCR_FILE_BOF			= 21, 
		   VCR_FILE_IDX_NOT_FOUND	= 22,

		   VCR_FILE_NOT_OPEN		= 30, 
		   VCR_FILE_EMPTY		= 31, 

		   VCR_FILE_PATH_NOT_FOUND	= 40,
		   VCR_FILE_INDEX_FILE_NOT_FOUND= 41,
		   VCR_FILE_DATA_FILE_NOT_FOUND	= 42,

		   VCR_FILE_NO_READ_PERM	= 50,
		   VCR_FILE_NO_WRITE_PERM	= 51,
		   VCR_FILE_NO_X_PERM		= 52,

		   VCR_FILE_CANNOT_OPEN_INDEX	= 60,
		   VCR_FILE_CANNOT_OPEN_DATA	= 61,
		   VCR_FILE_CANNOT_CREATE_INDEX	= 62,
		   VCR_FILE_CANNOT_CREATE_DATA	= 63,
		   VCR_FILE_CANNOT_READ_INDEX	= 64,
		   VCR_FILE_CANNOT_READ_DATA	= 65,

		   VCR_FILE_INDEX_NO_DATA	= 70,
		   VCR_FILE_DATA_NO_INDEX	= 71,

		   VCR_FILE_INDEX_READ_ERROR	= 80,
		   VCR_FILE_DATA_READ_ERROR	= 81,
		   VCR_FILE_INDEX_WRITE_ERROR	= 82,
		   VCR_FILE_DATA_WRITE_ERROR	= 83,

		   VCR_FILE_STAT_ERROR		= 90,
		   VCR_FILE_IDX_MAGIC_ERROR	= 91,
		   VCR_FILE_REC_MAGIC_ERROR	= 92,

		   VCR_FILE_ERROR		= 100
		 };

enum vcr_file_idx { VCR_FILE_NEWTS	=1,		    
		    VCR_FILE_NEWSP	=2,
		    VCR_FILE_NEWFMT	=4,
		    VCR_FILE_USER1	=8,
		    VCR_FILE_USER2	=16,
		    VCR_FILE_USER3	=32,
		    VCR_FILE_USER4	=64,
		    VCR_FILE_USER5	=128
		  };

class VCRFile {
public:
  VCRFile(int =100);
  ~VCRFile();
  
  vcr_file_rc open(String, int);
  vcr_file_rc close();

  vcr_file_rc check_stat(int* =NULL, int* =NULL);

  inline vcr_file_rc flush(int force =0) {
#ifdef DEBUG2
    cout << "VCRFile::flush()\n";
#endif
    
    if (ip==NULL) return VCR_FILE_NOT_OPEN;
    if (flush_ok!=VCR_FILE_TRUE || force) {
#ifdef DEBUG2
      cout << "VCRFile::flush() done.\n";
#endif
      flush_pq();
      fflush(fp);
      fflush(ip);
      flush_ok=VCR_FILE_TRUE;
    };
    return VCR_FILE_OK;
  };

  inline vcr_file_rc eof() {
    if (ip==NULL) return VCR_FILE_NOT_OPEN;
    return (feof(ip)?VCR_FILE_TRUE:VCR_FILE_FALSE);
  };
  inline vcr_file_rc bof() {
    if (ip==NULL) return VCR_FILE_NOT_OPEN;
    return (ftell(ip)==0?VCR_FILE_TRUE:VCR_FILE_FALSE);
  }
  inline vcr_file_rc is_open() {
    return (ip!=NULL)?VCR_FILE_TRUE:VCR_FILE_FALSE;
  };

  vcr_file_rc goto_start();
  vcr_file_rc goto_end();

  vcr_file_rc read(VCR_Hdr *, char **);
  vcr_file_rc write(VCR_Hdr * =NULL, char * =NULL);
  vcr_file_rc write_pq(VCRPacket *);
  vcr_file_rc flush_pq();



  vcr_file_rc read_next_hdr();
  vcr_file_rc read_prev_hdr();
  vcr_file_rc read_data(char **);

  vcr_file_rc skip_to_next_hdr();
  vcr_file_rc skip_to_prev_hdr();

  vcr_file_rc get_first_index(VCR_Index*, u_int8_t, int =1);
  vcr_file_rc get_next_index(VCR_Index*);

  vcr_file_rc goto_next_index(u_int8_t);
  vcr_file_rc goto_prev_index(u_int8_t);
  vcr_file_rc goto_next_blank(u_int32_t, u_int32_t* =NULL);
  vcr_file_rc goto_prev_blank(u_int32_t, u_int32_t* =NULL);
  vcr_file_rc set_index(u_int8_t, VCR_Index *);
  vcr_file_rc write_index(u_int8_t, VCR_Index *);
  vcr_file_rc delete_index(u_int32_t, u_int8_t);
  vcr_file_rc delete_all_indices(u_int8_t, int =1);

  vcr_file_rc goto_ts(u_int32_t, int32_t * =NULL);


  u_int32_t length();
  inline int is_writable()	{return writable;};
  inline int is_read_only()	{return read_only;}
  inline u_int16_t data_size()	{
    if (pq.length()==0) {
      return ntohs(index.l);
    } else {
      return pq.front().vcr_hdr.l;
    };
  };
  inline u_int32_t timestamp() {
    if (pq.length()==0) {
      return ntohl(index.ts);
    } else {
      return pq.front().vcr_hdr.ts;
    };
  };

  inline u_int8_t idx() {
    if (pq.length()==0) {
      return index.idx;
    } else {
      return pq.front().vcr_hdr.idx;
    };
  };

  inline u_int8_t channel() {
    if (pq.length()==0) {
      return index.ch;
    } else {
      return pq.front().vcr_hdr.ch;
    };
  };

  inline long get_pos()	{if (ip==NULL) return 0; return ftell(ip);};

  inline vcr_file_rc goto_pos(u_int32_t pos) { 
    if (ip==NULL) return VCR_FILE_NOT_OPEN;
    if (pos>idx_size) {
      fseek(ip, pos-idx_size, 0); 
      return read_next_hdr();
    } else {
      return VCR_FILE_ERROR;
    };
  };

  inline int get_resync_buffer_size()		{return resync_buffer_size;};
  inline int set_resync_buffer_size(int x)	{
    flush();
    return resync_buffer_size=x;
  };

protected:
  long get_file_size();

private:
  FILE		  	*fp, *ip;
  struct VCR_Index	index;
  int			read_only;
  int			idx_magic_size;
  int			rec_magic_size;

  String	  	filename;
  String		indexname;

  long			idx_file_size;
  u_int32_t		file_length;
  long			last_index_pos;
  u_int8_t		last_index_value;

  VCRPacketPHPQ		pq;
  int			max_in_pq;

  int			idx_size;
  vcr_file_rc		length_ok;
  vcr_file_rc		size_ok;
  vcr_file_rc		flush_ok;
  int			writable;
  int			resync_buffer_size;
  u_int8_t		rec_index;
  
};
#endif

