/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* $Id: VCRPacket.h,v 1.1 1996/07/26 15:04:10 whd Exp $ */

#ifndef _VCRPACKET_DEFS_H
#define _VCRPACKET_DEFS_H

#include <stddef.h>
#include <sys/types.h>
#include "it_depends.h"

#define PQ_DEFAULT_INITIAL_CAPACITY 20

enum channel_types {
  SESSION_CHANNEL = 0,
  DATA_CHANNEL    = 1,
  TIME_CHANNEL    = 6,
  INVALID_CHANNEL = 7
};

typedef struct VCR_Hdr {
  u_int32_t	ts;
#ifndef LITTLE_BITFIELDS
  u_int8_t	ch;
  u_int8_t	idx;
#else
  u_int8_t	idx;
  u_int8_t	ch;
#endif
  u_int16_t	l;
  u_int16_t	seq;
};

class VCRPacket {
 public:

  inline VCRPacket() {
    vcr_hdr.ts=0;
    vcr_hdr.ch=0;
    vcr_hdr.idx=0;
    vcr_hdr.l=0;
    vcr_hdr.seq=0;
    data=NULL;
  };

  inline int operator==(const VCRPacket& a) {
    return ((vcr_hdr.ts==a.vcr_hdr.ts) && 
	    (vcr_hdr.seq==a.vcr_hdr.seq) && 
	    (vcr_hdr.ch==a.vcr_hdr.ch));
  };
  
  inline int operator<(const VCRPacket& a) {
    return ((vcr_hdr.ts==a.vcr_hdr.ts)?
	    ((vcr_hdr.seq==a.vcr_hdr.seq)?
	     (vcr_hdr.ch<a.vcr_hdr.ch):
	     (vcr_hdr.seq<a.vcr_hdr.seq)):
	    (vcr_hdr.ts<a.vcr_hdr.ts));
  };
  
  inline int operator<=(const VCRPacket& a) {
    return ((vcr_hdr.ts==a.vcr_hdr.ts)?
	    ((vcr_hdr.seq==a.vcr_hdr.seq)?
	     (vcr_hdr.ch<=a.vcr_hdr.ch):
	     (vcr_hdr.seq<=a.vcr_hdr.seq)):
	    (vcr_hdr.ts<=a.vcr_hdr.ts));
  };
  
  inline int operator>(const VCRPacket& a) {
    return ((vcr_hdr.ts==a.vcr_hdr.ts)?
	    ((vcr_hdr.seq==a.vcr_hdr.seq)?
	     (vcr_hdr.ch>a.vcr_hdr.ch):
	     (vcr_hdr.seq>a.vcr_hdr.seq)):
	    (vcr_hdr.ts>a.vcr_hdr.ts));
  };
  
  inline int operator>=(const VCRPacket& a) {
    return ((vcr_hdr.ts==a.vcr_hdr.ts)?
	    ((vcr_hdr.seq==a.vcr_hdr.seq)?
	     (vcr_hdr.ch>=a.vcr_hdr.ch):
	     (vcr_hdr.seq>=a.vcr_hdr.seq)):
	    (vcr_hdr.ts>=a.vcr_hdr.ts));
  };

  struct VCR_Hdr vcr_hdr;
  char *data;
};




#endif
