/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* $Id: VCRTcl.h,v 1.2 1996/08/01 16:32:31 whd Exp $ */

#ifndef _VCRTCL_H_
#define _VCRTCL_H_

extern "C" {
#include <stdlib.h>
}
#include <String.h>
#include "Tcl.h"

class VCRTcl : public Tcl {

 private:
  static VCRTcl* instance_;

 public:
  VCRTcl(const char* =NULL);
  static void init(const char* program);
  static inline VCRTcl& instance() { return (*instance_); }

  inline char* var(const char* varname, int flags = TCL_GLOBAL_ONLY) {
    /* GCC makes static strings readonly.  When Tcl_GetVar parses a 
       variable name string "array(index)" it temporarily replaces the 
       "(" with a NUL byte '\0'. This causes a segmentation fault.  
       Therefore we have to put the string temporally into a read/write 
       variable. 
       */
    char *tmp = (char *) malloc(strlen(varname)+1);
    strcpy(tmp, varname);
    char *x=Tcl_GetVar(interp(), tmp, flags);
    free(tmp);
    if (x==NULL) {
      fprintf(stderr,"Tcl:var(): Tcl_GetVar returns NULL pointer for var \"%s\".\n",
	      varname);
      return NULL;
    } else {
      return x;
    }
  };
  
  inline int int_var(const char* varname, int flags = TCL_GLOBAL_ONLY) {
    
    char *x=var(varname, flags);
    if (x==NULL) {
      fprintf(stderr,"Tcl:int_var(): Tcl_GetVar returns NULL pointer for var \"%s\".\n",
	      varname);
      return -1;
    } else {
      int value=0;
      if (Tcl_GetInt(interp(), x, &value)==TCL_ERROR) {
	fprintf(stderr,"Tcl:int_var(): Tcl_GetInt returns TCL_ERROR for \"%s\".\n",x);
	return -1;
      }
      return value;
    };
  };

  inline int press_button(String button) {
    char cmd[200];
    release_button();
    sprintf(cmd,"press_button %s", (const char *) button);
    eval(cmd);
  };
  
  inline int release_button(String button ="") {
    char cmd[200];
    sprintf(cmd,"release_button %s", (const char *) button);
    eval(cmd);
  };
  
  inline int ErrorMsg(String msg) {
    char cmd[500];
    char *ver;
    sprintf(cmd, "AlertBox \"\n $VERSION error: \n\n %s \n\n\" \"\" \"\" \"$VERSION (Error)\" Ok", (const char *) msg);
    eval("global VERSION");
    eval(cmd);
  };

  inline int WarningMsg(String msg) {
    char cmd[500];
    sprintf(cmd, "AlertBox \"\n $VERSION Warning: \n\n %s \n\n\" \"\" \"\" \"$VERSION (Warning)\" Ok Cancel", (const char *) msg);
    eval("global VERSION");
    eval(cmd);
    return int_var("LAST_ALERT_RETURN");
  };

  inline void set_msg(String msg, int msecs) {
    char cmd[500];
    sprintf(cmd,"print_msg \"%s\"", (const char *) msg);
    eval(cmd);
    if (msecs>0) {
      sprintf(cmd,"after %d print_msg \"\"", msecs);
      eval(cmd);
    };
  };

  inline void set_led(int media_nb =0, int state =0) {
    char cmd[500];
    if (media_nb) {
      sprintf(cmd,"set_led %d %d", media_nb, state);
      eval(cmd);
    };
  };

  inline void clear_led(int media_nb =0) {
    char cmd[500];
    if (media_nb) {
      sprintf(cmd,"clear_led %d", media_nb);
      eval(cmd);
    }
  };

};

#endif
