/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* $Id: it_depends.h,v 1.1 1996/07/26 15:02:20 whd Exp $ */

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include <sys/time.h>

#if defined(__osf__) && defined(__alpha__)
#include <alpha/endian.h>
#endif

#if defined(sgi) || defined(__bsdi__) || defined(__FreeBSD__)
#include <sys/types.h>
#elif defined(linux)
#include <sys/bitypes.h>
#else
typedef signed char int8_t;
typedef unsigned char u_int8_t;
typedef short int16_t;
typedef unsigned short u_int16_t;
typedef unsigned int u_int32_t;
typedef int int32_t;
#ifdef __alpha
typedef long int64_t;
typedef unsigned long u_int64_t;
#endif
#endif

#if (defined(__hpux) || defined(_AIX)) && defined(__cplusplus)
#include <stdlib.h>
#include <unistd.h>
#include <time.h>		/* For clock_t */
extern "C" {
#include <arpa/inet.h>
int strcasecmp(const char *, const char *);
clock_t clock(void);
int gethostid(void);
#ifndef _AIX41
void srandom(int);
#endif
long random(void);
time_t time(time_t *);
char *ctime(const time_t *);
}
#endif

#if defined(NEED_SUNOS_PROTOS) && defined(__cplusplus)
extern "C" {
struct timeval;
struct timezone;
int ioctl(int fd, int request, ...);
int close(int);
int strcasecmp(const char*, const char*);
int srandom(int);
int random();
int socket(int, int, int);
int setsockopt(int s, int level, int optname, void* optval, int optlen);
struct sockaddr;
int connect(int s, sockaddr*, int);
int bind(int s, sockaddr*, int);
struct msghdr;
int send(int s, void*, int len, int flags);
int sendmsg(int, msghdr*, int);
int recv(int, void*, int len, int flags);
int recvfrom(int, void*, int len, int flags, sockaddr*, int);
int gethostid();
int getpid();
int gethostname(char*, int);
}
#endif

extern "C" {
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#ifdef HAVE_SYS_STREAM_H
#include <sys/stream.h>
#endif

#if !defined(sgi) && !defined(linux) && !defined(FREEBSD) && !defined(bsdi)
extern int gethostid(void);
#endif

#if defined(sgi) || defined(sun) || defined (hp) || defined(AIX) && OS_VER == 4
extern void bcopy(char *, char *, int);
#if defined(sgi) || defined(AIX)
extern void bzero(char *, int);
#endif
#endif

extern double drand48(void);
extern int getopt(int, char *const *, const char*);

#if defined(OSF) && OS_VER < 3
extern int select(int, void *, void *, void *, struct timeval *);
#endif


#if defined(sun) 
  extern void bzero(char *, int);
  extern int connect(int, struct sockaddr *, int);
  extern int setsockopt(int, int, int, const char*, int);
  extern int socket(int, int, int);
#endif

#if defined(SUNOS)
  extern void (*signal(int, void (*)(int)))();
  extern int bind(int, const void *, int);
  extern int recvfrom(int, void *, int, int, void *, int *);
  extern int select(int, void *, void *, void *, struct timeval *);
  extern int send(int, const void *, int, int);
#endif

#if defined(AIX) && OS_VER < 4
  extern int setsockopt(int, int, int, const char*, int);
  extern int socket(int, int, int);
  extern int bind(int, const void *, int);
  extern int connect(int, struct sockaddr *, int);
  extern int send(int, const void *, int, int);
  extern int recvfrom(int, void *, int, int, void *, int *);
#endif

#ifdef HAVE_BSDGETTIMEOFDAY
#define gettimeofday BSDgettimeofday
#endif
#ifdef GETTOD_NOT_DECLARED
extern int gettimeofday(struct timeval *tp, struct timezone *tzp);
#endif

/*
 * The following macro defines the type of the mask arguments to
 * select:
 */

#ifndef NO_FD_SET
#define SELECT_MASK fd_set
#else
#if !defined(AIX)
typedef long fd_mask;
#endif
#if defined(_IBMR2)
#define SELECT_MASK void
#else
#define SELECT_MASK int
# endif
#endif

/* define some macros

#ifndef MAX
#define MAX(x,y) (((x)>(y))?(x):(y))
#endif

/* convert ms to something modulo the unit */
#define ms2mod_hour(ms)	(((ms)/3600000)%24)
#define ms2mod_min(ms)	(((ms)/60000)%60)
#define ms2mod_sec(ms)	(((ms)/1000)%60)
#define ms2mod_usec(ms)	(((ms)%1000)%1000000)

/* convert ms to another unit */
#define ms2hour(ms)	((ms)/3600000)
#define ms2min(ms)	((ms)/60000)
#define ms2sec(ms)	((ms)/1000)
#define ms2usec(ms)	((ms)%1000)

/* convert rtp to ms and vice versa */
/* NOTE: ms2rtp(rtp2ms(x)) is not necessarily equal x   */
/*       rtp2ms(ms2rtp(x)) is equal to x (if x < 2**16) */
#define rtp2ms(rtp)	((((rtp)&0xFFFF0000)>>16)*1000+((rtp)&0xFFFF)/65)
#define ms2rtp(ms)	((((ms)/1000)<<16)|(((ms)%1000)*65))

/* convert tv to ms */
#define tv2ms(tv)	((tv.tv_sec&0x3FFFFF)*1000+(tv.tv_usec)/1000)

}

#endif
