/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
Copyright (c) 1995 International Computer Science Institute
All Rights Reserved.

Created by Wieland Holfelder (whd@icsi.berkeley.edu)
Tenet Group
International Computer Science Institute

Permission to use, copy, modify, and distribute this software and its 
documentation for educational, research and non-profit purposes, 
without fee, and without a written agreement is hereby granted, 
provided that the above copyright notice, this paragraph and the 
following five paragraphs appear in all copies.  

Permission to incorporate this software into commercial products may 
be obtained from the International Computer Science Institute,
1947 Center Street, Suite 600, Berkeley, CA 94704.  

IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE 
LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE 
USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS 
ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE 
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

The International Computer Science Institute will appreciate 
receiving any enhancements and modifications of this software 
made by others, and reserves the right to include them in future 
releases. The Institute also recommends that this software not 
be redistributed by third parties, but be obtained by all directly 
from the ICSI server, so that all users of the software may 
receive bug notifications and other relevant announcements.

The German Academic Exchange Service has supported the development 
of this software. Their support is gratefully acknowledged.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static const char rcsid[]="$Id: random.c,v 1.1 1996/07/26 15:02:20 whd Exp $";

/*
* Generate a random 32-bit quantity.
*/
extern "C" {
#include <stdio.h>          /* printf() */
#include <stdlib.h>
#include <sys/time.h>       /* gettimeofday() */
#include <unistd.h>         /* get..() */
#if defined(sun) && defined(__svr4__)
#include <sys/systeminfo.h> /* sysinfo */
#endif
#include "it_depends.h"
}

#include <global.h>         /* from RFC 1321 */
#include <md5.h>            /* from RFC 1321 */

#include "it_depends.h"

#define MD_CTX MD5_CTX
#define MDInit MD5Init
#define MDUpdate MD5Update
#define MDFinal MD5Final

static u_int32_t md_32(unsigned char *string, int length)
{
  MD_CTX context;
  union {
    char   c[16];
    u_int32_t x[4];
  } digest;
  u_int32_t r;
  int i;

  MDInit (&context);
  MDUpdate (&context, string, length);
  MDFinal ((unsigned char *)&digest, &context);
  r = 0;
  for (i = 0; i < 3; i++) {
    r ^= digest.x[i];
  }
  return r;
} /* md_32 */


/*
* Return random unsigned 32-bit quantity.
*/
u_int32_t random32()
{
  #if defined(sun) && defined(__svr4__) 
  char buf[100];
  #endif
  struct {
    struct timeval tv;
    pid_t pid;
    u_int32_t hostid;
    uid_t  uid;
    gid_t  gid;
    char   name[8];
  } s;

  gettimeofday(&s.tv, 0);
  s.pid    = getpid();
  #if defined(sun) && defined(__svr4__) 
  sysinfo(SI_HW_SERIAL,buf,100);
  s.hostid = atoi(buf);
  #else
  s.hostid = gethostid();
  #endif
  s.uid    = getuid();
  s.gid    = getgid();
  gethostname(s.name, sizeof(s.name));

  return md_32((unsigned char *)&s, sizeof(s));
} /* random32 */

static u_int16_t md_16(unsigned char *string, int length)
{
  MD_CTX context;
  union {
    char   c[16];
    u_int16_t x[8];
  } digest;
  u_int16_t r;
  int i;

  MDInit (&context);
  MDUpdate (&context, string, length);
  MDFinal ((unsigned char *)&digest, &context);
  r = 0;
  for (i = 0; i < 3; i++) {
    r ^= digest.x[i];
  };
  return r;
} /* md_16 */


/*
* Return random unsigned 16-bit quantity.
*/
u_int16_t random16()
{
  #if defined(sun) && defined(__svr4__)
  char buf[100];
  #endif
  struct {
    struct timeval tv;
    pid_t pid;
    u_int32_t hostid;
    uid_t  uid;
    gid_t  gid;
    char   name[8];
  } s;

  gettimeofday(&s.tv, 0);
  s.pid    = getpid();
  #if defined(sun) && defined(__svr4__)
  sysinfo(SI_HW_SERIAL,buf,100);
  s.hostid = atoi(buf);
  #else
  s.hostid = gethostid();
  #endif
  s.uid    = getuid();
  s.gid    = getgid();
  gethostname(s.name, sizeof(s.name));

  return md_16((unsigned char *)&s, sizeof(s));
} /* random16 */


