#/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#
# Copyright (c) 1995 International Computer Science Institute
# All Rights Reserved.
#
# Created by Wieland Holfelder (whd@icsi.berkeley.edu)
# Tenet Group
# International Computer Science Institute
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for educational, research and non-profit purposes,
# without fee, and without a written agreement is hereby granted,
# provided that the above copyright notice, this paragraph and the
# following five paragraphs appear in all copies.
#
# Permission to incorporate this software into commercial products may
# be obtained from the International Computer Science Institute,
# 1947 Center Street, Suite 600, Berkeley, CA 94704.
#
# IN NO EVENT SHALL THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE BE
# LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
# CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE
# USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE INTERNATIONAL
# COMPUTER SCIENCE INSTITUTE  HAS BEEN ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# THE INTERNATIONAL COMPUTER SCIENCE INSTITUTE SPECIFICALLY DISCLAIMS
# ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
# PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE INTERNATIONAL
# COMPUTER SCIENCE INSTITUTE  HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
#
# The International Computer Science Institute will appreciate
# receiving any enhancements and modifications of this software
# made by others, and reserves the right to include them in future
# releases. The Institute also recommends that this software not
# be redistributed by third parties, but be obtained by all directly
# from the ICSI server, so that all users of the software may
# receive bug notifications and other relevant announcements.
#
# The German Academic Exchange Service has supported the development
# of this software. Their support is gratefully acknowledged.
#
#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#
# $Id: vcr.tcl,v 1.7 1996/10/11 09:20:16 whd Exp $ (ICSI)
#
#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

# dummy proc just in case we don't have the real stuff from vcr
proc vcr_ctrl {{arg1 ""} {arg2 ""} {arg3 ""} {arg4 ""} {arg5 ""} {arg6 ""} {arg7 ""}} {
    puts "vcr_ctrl $arg1 $arg2 $arg3 $arg4 $arg5 $arg6 $arg7"

    if {$arg1 == "read"}	{return "1"}
    if {$arg1 == "delete"}	{return "0"}
    if {$arg1 == "play"}	{return "playing"}
    if {$arg1 == "stop"}	{return "idle"}
    if {$arg1 == "rec"}		{return "recording"}
    if {$arg1 == "ff"}		{return "forwarding"}
    if {$arg1 == "rew"}		{return "rewinding"}
    if {$arg1 == "is_writable"}	{return "1"}
    if {$arg1 == "set_rec_enabled"}	{return "1"}
    if {$arg1 == "set_rec_disabled"}	{return "0"}

    return ""
}

# procedure to show window vcr.
proc InitMainWindow {} {
    global VERSION READ_ONLY num_display bmp interactive session colors session_file 
    global cmd_start_time font index_vaues init

    # set Xbitmaps
    SetXbm

    # Window manager configurations
    global tkVersion
    wm positionfrom . "program"
    wm sizefrom . "program"
    wm minsize . 640 380
    wm geometry . 640x380
    wm title . "$VERSION"
    wm iconname . "vcr"
    wm protocol . WM_DELETE_WINDOW {quit_vcr}

    # bindings
    bind . <Enter> { focus %W }
    bind . <q> { adios }
    bind . <Control-c> { adios }
    bind . <Control-d> { adios }
    bind . <Control-i> { toggle_index_ctrl }
    bind . <Control-s> { start_tools_win }
    bind . <Control-c> { close_session }
    bind . <Control-r> { reread_session }
    bind . <Control-R> { blank_session }
    bind . <b> { set session(state) [vcr_ctrl goto start] }
    bind . <e> { set session(state) [vcr_ctrl goto end] }
    bind . <s> { stop }
    bind . <r> { rec }
    bind . <p> { play }
    bind . <l> { .vcr.top.options.m invoke 0 }

    # build widget .title
    frame .title \
	    -borderwidth {2} \
	    -relief {raised}

    # build widget .title.title
    label .title.title \
	    -highlightthickness {0} \
	    -borderwidth {1} \
	    -font $font(vcr_title) \
	    -foreground {black} \
	    -text "$VERSION"

    # pack widget .title
    pack append .title \
	    .title.title {left frame center expand fillx}

    # build widget .vcr
    frame .vcr \
	    -highlightthickness {0} \
	    -borderwidth {5} \
	    -relief {raised}

    # build widget .vcr.top
    frame .vcr.top \
	    -highlightthickness {0} \
	    -borderwidth {1} \
	    -relief {raised}

    # build widget .vcr.top.session
    menubutton .vcr.top.session \
	    -foreground {black} \
	    -menu {.vcr.top.session.m} \
	    -text {Session} \
	    -underline {0}

    # build widget .vcr.top.session.m
    menu .vcr.top.session.m -foreground {black} -tearoff 0

    .vcr.top.session.m add command \
	    -label {New Session ...} \
	    -underline {0} \
	    -command {
	if {[demo_version] == "0"} {
	    if { [check_macro] == "1" } {return}
	    open_session 1 "$VERSION (New Session)"
	}
    }

    .vcr.top.session.m add command \
	    -label {Open Session ...} \
	    -underline {0} \
	    -command {
	global VERSION
	if { [check_macro] == "1" } {return}
	open_session 0 "$VERSION (Open Session)"
    }

    .vcr.top.session.m add command \
	    -label {Import SD Session ...} \
	    -underline {0} \
	    -command {
	global VERSION
	if { [check_macro] == "1" } {return}
	if { [file exists "~/.sd_cache"] == 0 } {
	    AlertBox "\n $VERSION: \n\n Cannot import sessions from SD. \n\
		    File ~/.sd_cache not found. \n\n"\
		    "" "" "$VERSION (Import Session)" Ok
	    return 0
	}
	if { [close_session] > 0 } {
	    sd_import 1
	}
    }

    .vcr.top.session.m add separator

    .vcr.top.session.m add command \
	    -label {Save Session} \
	    -underline {0} \
	    -command {
	if {[check_macro] == "1"} {return}
	if {[demo_version] == "0"} {
	    save_session
	    update_main_win
	}
    }

    .vcr.top.session.m add command \
	    -label {Save Session As ...} \
	    -underline {13} \
	    -command {
	global session
	if {[check_macro] == "1"} {return}
	if {[demo_version] == "0"} {
	    set name [FSBox "Save Session As:" \
		    [file tail $session(filename)] "" "" "*.vcr" "$VERSION (Save Session As)"]
	    if { ($name != "") && ([IsADir $name] == 0) } {
		set old_session_filename $session(filename)
		set session(filename) [check_file_ext $name]
		if { [save_session $old_session_filename] == "1" } {
		    update_session
		} else {
		    close_session
		}
		update_main_win
	    }
	}
    }

    .vcr.top.session.m add separator

    .vcr.top.session.m add command \
	    -label {Edit Session ...}\
	    -underline {0}\
	    -command {
	    if {[check_macro] == "1"} {return}
	    edit_session "0"
    }

    .vcr.top.session.m add command \
	    -label {Close Session} \
	    -underline {0} \
	    -command {
	if {[check_macro] == "1"} {return}
	close_session
    }

    .vcr.top.session.m add separator

    .vcr.top.session.m add command \
	    -label {Reread Session}\
	    -underline {0}\
	    -command {
	if {[check_macro] == "1"} {return}
	reread_session
    }

    .vcr.top.session.m add command \
	    -label {Blank Session}\
	    -underline {0}\
	    -command {
	if {[check_macro] == "1"} {return}
	blank_session
    }

    .vcr.top.session.m add separator

    .vcr.top.session.m add command \
	    -label {Quit} \
	    -underline {0} \
	    -command {
	if {[check_macro] == "1"} {return}
	quit_vcr
    }

    # build widget .vcr.top.options
    menubutton .vcr.top.options \
	    -foreground {black} \
	    -menu {.vcr.top.options.m} \
	    -text {Options} \
	    -underline {0}

    # build widget .vcr.top.options.m
    menu .vcr.top.options.m -tearoff 0

    set session(loop_mode) $init(loop_mode)
    .vcr.top.options.m add checkbutton \
	    -label {Loop Mode} \
	    -variable {session(loop_mode)} \
	    -underline {0}

    .vcr.top.options.m add checkbutton \
	    -label {Blinking buttons} \
	    -variable {session(blink_mode)} \
	    -underline {0} \
	    -command {
	if { $session(start_blinking) != "" } {
	    eval "$session(start_blinking)"
	}
    }

    .vcr.top.options.m add checkbutton \
	    -label {Show Record/Playback LEDs} \
	    -variable {session(indicate)} \
	    -underline {0} \
	    -command { update_leds }

    .vcr.top.options.m add separator

    .vcr.top.options.m add cascade \
	    -label {FF/REW factor ...} \
	    -menu {.vcr.top.options.m.ff_rew_factor} \
	    -underline {7}

    menu .vcr.top.options.m.ff_rew_factor -tearoff 0

    foreach i { 2 5 10 25 50 100} {
	.vcr.top.options.m.ff_rew_factor add radiobutton \
		-label "x $i" \
		-variable {session(ff_rew_factor)} \
		-value "$i" \
		-command "vcr_ctrl set_ff_rew_factor $i"
    }

    # build widget .vcr.top.speed
    menubutton .vcr.top.speed \
	    -menu {.vcr.top.speed.selection} \
	    -text {Speed} \
	    -underline {1}

    # build widget .vcr.top.speed.selection
    menu .vcr.top.speed.selection -foreground {black} -tearoff 0

    .vcr.top.speed.selection add radiobutton \
	    -label {normal} \
	    -state {active} \
	    -value {1} \
	    -variable {session(speed)} \
	    -underline {0} \
	    -command {
	if {[check_macro] == "1"} {return}
	vcr_ctrl speed $session(speed)
    }

    .vcr.top.speed.selection add radiobutton \
	    -label {double} \
	    -value {2} \
	    -variable {session(speed)} \
	    -underline {0} \
	    -command {
	if {[check_macro] == "1"} {return}
	vcr_ctrl speed $session(speed)
    }

    .vcr.top.speed.selection add radiobutton \
	    -label {half} \
	    -value {0.5} \
	    -variable {session(speed)} \
	    -underline {0} \
	    -command {
	if {[check_macro] == "1"} {return}
	vcr_ctrl speed $session(speed)
    }

    # build widget .vcr.top.record
    menubutton .vcr.top.record \
	    -foreground {black} \
	    -menu {.vcr.top.record.m} \
	    -text {Record} \
	    -underline {0}

    # build widget .vcr.top.record.m
    menu .vcr.top.record.m -tearoff 0

    .vcr.top.record.m add checkbutton \
	    -label {Recording enabled} \
	    -underline {0} \
	    -onvalue {1} \
	    -offvalue {0} \
	    -variable {session(rec_enabled)} \
	    -command {
	if {[check_macro] == "1"} {return}
	if {$session(rec_enabled) == "1"}  {
	    enable_recording
	} else {
	    disable_recording
	}
    }

    .vcr.top.record.m add checkbutton \
	    -label {Recording without signal} \
	    -variable {session(start_recording)} \
	    -underline {10}

    .vcr.top.record.m add cascade \
	    -label {Recording timeout ...} \
	    -menu {.vcr.top.record.m.rec_timeout} \
	    -underline {10}

    menu .vcr.top.record.m.rec_timeout -tearoff 0

    .vcr.top.record.m.rec_timeout add radiobutton \
	    -label "No timeout" \
	    -variable {session(rec_timeout)} \
	    -value "0" \
	    -command "vcr_ctrl set_rec_timeout 0"

    foreach i { 2 5 10 30 60 120 180 240 300} {
	.vcr.top.record.m.rec_timeout add radiobutton \
		-label "$i secs" \
		-variable {session(rec_timeout)} \
		-value "$i" \
		-command "vcr_ctrl set_rec_timeout $i"
    }

   # build widget .vcr.top.macro
    menubutton .vcr.top.macro \
	    -foreground {black} \
	    -menu {.vcr.top.macro.m} \
	    -text {Macro} \
	    -underline {0}

    # build widget .vcr.top.macro.m
    menu .vcr.top.macro.m -foreground {black} -tearoff 0

  .vcr.top.macro.m add command \
	    -label {Run Macro Cmd File} \
	    -underline {0} \
	    -command {
	global VERSION
	if {[check_macro] == "1"} {return}
	set name [FSBox "Select an MBone VCR Macro File:" \
		"" "" "" "*.cmd" "$VERSION (Read Macro Cmd File)"]
	if { $name != "" } {
	    if { [IsAFile $name] == 1 } {
		read_cmd_file $name
	    } else {
		AlertBox "\n $VERSION error: \n\n Macro file $name not found. \n"\
			"" "" "$VERSION" Ok
	    }
	}
    }

    .vcr.top.macro.m add command \
	    -label {Stop Macro Execution} \
	    -underline {0} \
	    -command {
	global cmd_start_time
	set cmd_start_time ""
	.vcr.middle.right.middle.label configure -text [center "" 60]
	.vcr.top.macro.m entryconfigure 0 -state normal
	.vcr.top.macro.m entryconfigure 1 -state disabled
    }


    # build widget .vcr.top.tools
    menubutton .vcr.top.tools \
	    -menu {.vcr.top.tools.m} \
	    -text {Tools} \
	    -underline {0}

    # build widget .vcr.top.tools.m
    menu .vcr.top.tools.m -tearoff 0

    .vcr.top.tools.m add command \
	    -command {start_tools_win} \
	    -label {Start Session Tools ...} \
	    -underline {0}

    .vcr.top.tools.m add command \
	    -command {stop_tools 1} \
	    -label {Stop Session Tools ...} \
	    -underline {1}

    .vcr.top.tools.m add checkbutton \
	    -label {Autostart Session Tools} \
	    -variable {session(auto_start)} \
	    -underline {0}


    # build widget .vcr.top.help
    menubutton .vcr.top.help \
	    -menu {.vcr.top.help.m} \
	    -text {Help} \
	    -underline {0}

    # build widget .vcr.top.help.m
    menu .vcr.top.help.m -tearoff 0

    .vcr.top.help.m add command \
	    -command {
	AlertBox "\n Help not supported yet. \n\n\
		See the man page (vcr.1 or vcr.ps) for details. \n\n" "" "" "$VERSION (Help)" Ok
    }\
	    -label {Help} \
	    -state {active} \
	    -underline {0}

    .vcr.top.help.m add separator

    .vcr.top.help.m add command \
	    -command {
	AlertBox "\n$VERSION\n\n\
		Copyright (C) 1997 ICSI Berkeley, University of Mannheim\n\
		All rights reserved.\n\n\
		This software is subject to the copyright agreement set forth in the COPYRIGHT file.\n\
		Please read and agree to all terms before using this software.\n\n\
		Report any problems to mbone-vcr@icsi.berkeley.edu\n\n\
		Wieland Holfelder\n\
		L15, 16\n\
		68131 Mannheim\n\n\
		Phone: +49-621-292-5679\n\
		Email: whd@pi4.informatik.uni-mannheim.de\n\
		Fax: +49-621-292-5745\n\
		URL: http://www.informatik.uni-mannheim.de/~whd\n\n"\
		"" "" "$VERSION (About)" Ok
    } \
	    -label {About MBone VCR} \
	    -underline {0}


    # pack widget .vcr.top
    pack append .vcr.top \
	    .vcr.top.session {left frame center} \
	    .vcr.top.options {left frame center} \
	    .vcr.top.speed   {left frame center} \
	    .vcr.top.record  {left frame center} \
	    .vcr.top.tools   {left frame center} \
	    .vcr.top.macro   {left frame center} \
	    .vcr.top.help    {right frame center}

    # build widget .vcr.middle
    frame .vcr.middle \
	    -borderwidth {1} \
	    -relief {raised}

    # build widget .vcr.middle.left
    frame .vcr.middle.left \
	    -borderwidth {1} \
	    -relief {raised}

    # build widget .vcr.middle.left.top
    frame .vcr.middle.left.top \
	    -borderwidth {2} \
	    -relief {raised}

    # build widget .vcr.middle.left.top.fill1
    frame .vcr.middle.left.top.fill1

    # build widget .vcr.middle.left.top.session_name
    frame .vcr.middle.left.top.session_name

    # build widget .vcr.middle.left.top.session_name.label
    label .vcr.middle.left.top.session_name.label \
	    -foreground {red} \
	    -text [center $session(name) 40]

    bind .vcr.middle.left.top.session_name.label <Double-Button-1> {
	if {[check_macro] == "1"} {return}
	if {$session(loaded) == 1} {
	    edit_session "0"
	} else {
	    open_session 0 "$VERSION (Open Session)"
	}
    }

    pack append .vcr.middle.left.top.session_name \
	    .vcr.middle.left.top.session_name.label {top frame center fill pady 5}

    # build widget .vcr.middle.left.top.fill2
    frame .vcr.middle.left.top.fill2

    # build widget .vcr.middle.left.media_list
    frame .vcr.middle.left.top.media_list

    # build widget .vcr.middle.left.top.media_list.scrollbar_y
    scrollbar .vcr.middle.left.top.media_list.scrollbar_y \
	    -highlightthickness {0} \
	    -command {.vcr.middle.left.top.media_list.listbox yview} \
	    -width {10} \
	    -relief {raised}

    # build widget .vcr.middle.left.media_list.listbox
    listbox .vcr.middle.left.top.media_list.listbox \
	    -highlightthickness {0} \
	    -borderwidth {2} \
	    -width {25} \
	    -height {5} \
	    -relief {groove} \
	    -selectmode {extended} \
	    -yscrollcommand {.vcr.middle.left.top.media_list.scrollbar_y set}

    bind .vcr.middle.left.top.media_list.listbox <Double-Button-1> {
	if {[check_macro] == "1"} {return}
	if { $session(loaded) == "1" } {
	    set x [.vcr.middle.left.top.media_list.listbox curselection]
	    fill_tmp_session
	    edit_media [expr $x + 1]
	} else {
	    if {[demo_version] == "0"} {
		open_session 1 "$VERSION (New Session)"
	    }
	}
    }

    bind .vcr.middle.left.top.media_list.listbox <Button-2> {
	if {[check_macro] == "1"} {return}
	set index [.vcr.middle.left.top.media_list.listbox nearest %y]
	set selected [.vcr.middle.left.top.media_list.listbox curselection]
	mute_media $index
	if { $selected != "" } {
	    .vcr.middle.left.top.media_list.listbox select set [lrange $selected 0 0] [lrange $selected [expr [llength $selected] - 1] end]
	}
    }

    bind .vcr.middle.left.top.media_list.listbox <Button-3> {
	if { [check_macro] == "1" } {return}
	.vcr.middle.left.top.media_list.listbox select clear 0 end
    }

    # pack widget .vcr.middle.left.top.media_list
    pack append .vcr.middle.left.top.media_list \
	    .vcr.middle.left.top.media_list.scrollbar_y {right frame center filly} \
	    .vcr.middle.left.top.media_list.listbox {top frame center expand fill}

    # build widget .vcr.middle.left.top.fill3
    frame .vcr.middle.left.top.fill3

    button .vcr.middle.left.top.mute \
	    -text { mute/unmute} \
	    -relief {raised} \
	    -borderwidth {2} \
	    -cursor {hand2} \
	    -pady {1} \
	    -padx {1} \
	    -command {
	if {[check_macro] == "1"} {return}
	set selected [.vcr.middle.left.top.media_list.listbox curselection]
	mute_media $selected
	if { $selected != "" } {
	    .vcr.middle.left.top.media_list.listbox select set [lrange $selected 0 0] [lrange $selected [expr [llength $selected] - 1] end]
	}

    }

    # build widget .vcr.middle.left.top.fill4
    frame .vcr.middle.left.top.fill4

    # build widget .vcr.middle.left.top.read_only
    frame .vcr.middle.left.top.read_only

    # build widget .vcr.middle.left.top.read_only.label
    label .vcr.middle.left.top.read_only.label \
	    -font $font(read_only) \
	    -text ""

    pack append .vcr.middle.left.top.read_only \
	    .vcr.middle.left.top.read_only.label {top frame center fill pady 5}

    # pack widget .vcr.middle.left.top
    pack append .vcr.middle.left.top \
	    .vcr.middle.left.top.fill1 {top frame center fill pady 2} \
	    .vcr.middle.left.top.session_name {top frame center fill padx 2} \
	    .vcr.middle.left.top.fill2 {top frame center fill pady 2} \
	    .vcr.middle.left.top.media_list {top frame center padx 10} \
	    .vcr.middle.left.top.fill3 {top frame center fill pady 5} \
	    .vcr.middle.left.top.mute {top frame center pady 5} \
	    .vcr.middle.left.top.fill4 {top frame center fill} \
	    .vcr.middle.left.top.read_only {top frame center fill pady 5}

    # build widget .vcr.middle.left.bottom
    frame .vcr.middle.left.bottom \
	    -borderwidth {2} \
	    -relief {raised}

    frame .vcr.middle.left.bottom.leds
    label .vcr.middle.left.bottom.leds.title -text {}

    label .vcr.middle.left.bottom.leds.led1 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    label .vcr.middle.left.bottom.leds.led2 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    label .vcr.middle.left.bottom.leds.led3 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    label .vcr.middle.left.bottom.leds.led4 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    label .vcr.middle.left.bottom.leds.led5 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    label .vcr.middle.left.bottom.leds.led6 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    label .vcr.middle.left.bottom.leds.led7 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    label .vcr.middle.left.bottom.leds.led8 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    label .vcr.middle.left.bottom.leds.led9 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    label .vcr.middle.left.bottom.leds.led10 \
	    -bitmap "$bmp(led)" \
	    -foreground "$colors(led,hidden)"

    pack append .vcr.middle.left.bottom.leds \
	    .vcr.middle.left.bottom.leds.title {top frame center expand} \
	    .vcr.middle.left.bottom.leds.led1 {left frame center expand} \
	    .vcr.middle.left.bottom.leds.led2 {left frame center expand} \
	    .vcr.middle.left.bottom.leds.led3 {left frame center expand} \
	    .vcr.middle.left.bottom.leds.led4 {left frame center expand} \
	    .vcr.middle.left.bottom.leds.led5 {left frame center expand} \
	    .vcr.middle.left.bottom.leds.led6 {left frame center expand} \
	    .vcr.middle.left.bottom.leds.led7 {left frame center expand} \
	    .vcr.middle.left.bottom.leds.led8 {left frame center expand} \
	    .vcr.middle.left.bottom.leds.led9 {left frame center expand} \
	    .vcr.middle.left.bottom.leds.led10 {left frame center expand}

    # pack widget .vcr.middle.left.bottom
    pack append .vcr.middle.left.bottom \
	    .vcr.middle.left.bottom.leds {top frame center expand fill}


    # pack widget .vcr.middle.left
    pack append .vcr.middle.left \
	    .vcr.middle.left.top {top frame center expand fill} \
	    .vcr.middle.left.bottom {bottom frame center expand fill}

    # build widget .vcr.middle.right
    frame .vcr.middle.right \
	    -highlightthickness 0 \
	    -borderwidth {1} \
	    -relief {raised}

    # build widget .vcr.middle.right.top
    frame .vcr.middle.right.top \
	    -highlightthickness 0 \
	    -borderwidth {1} \
	    -relief {sunken}

    # build widget .vcr.middle.right.top [NumDisplay]
    set num_display [NumDisplay .vcr.middle.right.top 18 5 1]

    # pack widget .vcr.middle.right.top
    pack append .vcr.middle.right.top \
	    $num_display {top frame center padx 10 pady 10}

    frame .vcr.middle.right.middle \
	    -borderwidth {0} \
	    -relief {flat}

    label .vcr.middle.right.middle.label \
	    -font $font(hint) \
	    -text ""

    pack append .vcr.middle.right.middle \
	    .vcr.middle.right.middle.label {top frame center}

    # build widget .vcr.middle.right.bottom
    frame .vcr.middle.right.bottom \
	    -borderwidth {1}

    # build widget .vcr.middle.right.bottom.skip_back
    button .vcr.middle.right.bottom.skip_back \
	    -bitmap "$bmp(goto_start)" \
	    -borderwidth {3} \
	    -cursor {hand2} \
	    -foreground "$colors(skip_back,normal)" \
	    -activeforeground "$colors(skip_back,normal)" \
	    -command {
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl goto start]
    }

    # build widget .vcr.middle.right.bottom.rew
    button .vcr.middle.right.bottom.rew \
	    -bitmap "$bmp(rew)" \
	    -borderwidth {3} \
	    -cursor {hand2} \
	    -foreground "$colors(rew,normal)" \
	    -activeforeground "$colors(rew,normal)" \
	    -command {
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl rew $session(ff_rew_factor)]
    }

    # build widget .vcr.middle.right.bottom.rec
    button .vcr.middle.right.bottom.rec \
	    -bitmap "$bmp(rec)" \
	    -borderwidth {3} \
	    -cursor {hand2} \
	    -state {disabled} \
	    -foreground "$colors(rec,normal)" \
	    -activeforeground "$colors(rec,normal)" \
	    -command { rec }

    # build widget .vcr.middle.right.bottom.stop
    button .vcr.middle.right.bottom.stop \
	    -bitmap "$bmp(stop)" \
	    -borderwidth {3} \
	    -cursor {hand2} \
	    -foreground "$colors(stop,normal)" \
	    -activeforeground "$colors(stop,normal)" \
	    -command { stop }

    # build widget .vcr.middle.right.bottom.play
    button .vcr.middle.right.bottom.play \
	    -bitmap "$bmp(play)" \
	    -borderwidth {3} \
	    -cursor {hand2} \
	    -foreground "$colors(play,normal)" \
	    -activeforeground "$colors(play,normal)" \
	    -command { play }

    # build widget .vcr.middle.right.bottom.ff
    button .vcr.middle.right.bottom.ff \
	    -bitmap "$bmp(ff)" \
	    -borderwidth {3} \
	    -cursor {hand2} \
	    -foreground "$colors(ff,normal)" \
	    -activeforeground "$colors(ff,normal)" \
	    -command {
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl ff $session(ff_rew_factor)]
    }

    # build widget .vcr.middle.right.bottom.skip_forw
    button .vcr.middle.right.bottom.skip_forw \
	    -bitmap "$bmp(goto_end)" \
	    -borderwidth {3} \
	    -cursor {hand2} \
	    -foreground "$colors(skip_forw,normal)" \
	    -activeforeground "$colors(skip_forw,normal)" \
	    -command {
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl goto end]
    }

    bind .vcr.middle.right.bottom.skip_back <Enter> {
	button_hint .vcr.middle.right.bottom.skip_back "goto start" "not available"
    }

    bind .vcr.middle.right.bottom.rew <Enter> {
	button_hint .vcr.middle.right.bottom.rew "rewind" "not available"
    }

    bind .vcr.middle.right.bottom.rec <Enter> {
	global session
	if { $session(loaded) == 1 } {
	    if {$session(read_only) != "" } {
		set text "recording disabled (session is read-only)"
	    } else {
		set text "recording disabled (use Record-menu to enable recording)"
	    }
	} else {
	    set text "not available"
	}
	button_hint .vcr.middle.right.bottom.rec "record" $text
    }

    bind .vcr.middle.right.bottom.stop <Enter> {
	button_hint .vcr.middle.right.bottom.stop "stop" "not available"
    }

    bind .vcr.middle.right.bottom.play <Enter> {
	button_hint .vcr.middle.right.bottom.play "play" "not available"
    }

    bind .vcr.middle.right.bottom.ff <Enter> {
	button_hint .vcr.middle.right.bottom.ff "forward" "not available"
    }

    bind .vcr.middle.right.bottom.skip_forw <Enter> {
	button_hint .vcr.middle.right.bottom.skip_forw "goto end" "not available"
    }

    bind .vcr.middle.right.bottom.skip_back	<Leave>	{print_msg ""}
    bind .vcr.middle.right.bottom.rew		<Leave>	{print_msg ""}
    bind .vcr.middle.right.bottom.rec		<Leave> {print_msg ""}
    bind .vcr.middle.right.bottom.stop		<Leave> {print_msg ""}
    bind .vcr.middle.right.bottom.play		<Leave> {print_msg ""}
    bind .vcr.middle.right.bottom.ff		<Leave> {print_msg ""}
    bind .vcr.middle.right.bottom.skip_forw	<Leave> {print_msg ""}

    # pack widget .vcr.middle.right.bottom
    pack append .vcr.middle.right.bottom \
	    .vcr.middle.right.bottom.skip_back {left frame center padx 10 pady 10 expand} \
	    .vcr.middle.right.bottom.rew {left frame center padx 10 pady 10 expand} \
	    .vcr.middle.right.bottom.rec {left frame center padx 10 pady 10 expand} \
	    .vcr.middle.right.bottom.stop {left frame center padx 10 pady 10 expand} \
	    .vcr.middle.right.bottom.play {left frame center padx 10 pady 10 expand} \
	    .vcr.middle.right.bottom.ff {left frame center padx 10 pady 10 expand} \
	    .vcr.middle.right.bottom.skip_forw {left frame center padx 10 pady 10 expand}

    # build widget .vcr.middle.right.index_ctrl
    frame .vcr.middle.right.index_ctrl \
	    -borderwidth {2}

    # build widget .vcr.middle.right.index_ctrl.index_mode
    frame .vcr.middle.right.index_ctrl.index_mode \
	    -borderwidth {2}

    # built widget .vcr.middle.right.index_ctrl.index_mode.indexbutton
    menubutton .vcr.middle.right.index_ctrl.index_mode.indexbutton \
	    -menu {.vcr.middle.right.index_ctrl.index_mode.indexbutton.m} \
	    -relief {groove} \
	    -padx {0} \
	    -pady {0} \
	    -text {Idxmode:}

    # build widget .vcr.middle.right.index_ctrl.index_mode.index_display
    label .vcr.middle.right.index_ctrl.index_mode.index_display \
	    -text {goto start/end} \
	    -relief {sunken} \
	    -borderwidth {1} \
	    -anchor {w} \
	    -width {20}

    # built widget .vcr.middle.right.index_ctrl.index_mode.indexbutton.m
    menu .vcr.middle.right.index_ctrl.index_mode.indexbutton.m -tearoff 0

    .vcr.middle.right.index_ctrl.index_mode.indexbutton.m add command \
	    -label "goto start/end" \
	    -command {set_goto_start_end}

    .vcr.middle.right.index_ctrl.index_mode.indexbutton.m add command \
	    -label "goto prev/next sync unit"\
	    -command {set_goto_prev_next_sync}

    .vcr.middle.right.index_ctrl.index_mode.indexbutton.m add command \
	    -label "goto prev/next speaker" \
	    -command {set_goto_prev_next_speaker}

    .vcr.middle.right.index_ctrl.index_mode.indexbutton.m add cascade \
	    -label "goto prev/next user index ..." \
	    -menu {.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.user_idx}

    menu .vcr.middle.right.index_ctrl.index_mode.indexbutton.m.user_idx -tearoff 0

    for { set i 1 } { $i <= 5 } { incr i } {
	.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.user_idx add command \
		-label "level $i" \
		-command "set_user_idx $i"
    }

    .vcr.middle.right.index_ctrl.index_mode.indexbutton.m add cascade \
	    -label "skip prev/next secs/mins..." \
	    -menu {.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.secs_mins}

    menu .vcr.middle.right.index_ctrl.index_mode.indexbutton.m.secs_mins -tearoff 0

    foreach i { 1 5 10 20 30 } {
	set label "secs"
	if { $i == 1} { set label "sec" }
	.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.secs_mins add command \
		-label "$i $label" \
		-command "set_skip_time $i sec"
    }

    foreach i { 1 5 10 20 30 } {
	set label "mins"
	if { $i == 1} { set label "min" }
	.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.secs_mins add command \
		-label "$i min" \
		-command "set_skip_time $i min"
    }

    .vcr.middle.right.index_ctrl.index_mode.indexbutton.m add cascade \
	    -label "blank skip to x gap..." \
	    -menu {.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.blank_skip}

    menu .vcr.middle.right.index_ctrl.index_mode.indexbutton.m.blank_skip -tearoff 0

    foreach i { 1 2 3 4 5 10 20 30 } {
	set label "sec"
	.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.blank_skip add command \
		-label "$i $label" \
		-command "set_blank_skip $i sec"
    }

    foreach i { 1 2 3 4 5 } {
	set label "min"
	.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.blank_skip add command \
		-label "$i min" \
		-command "set_blank_skip $i min"
    }

    .vcr.middle.right.index_ctrl.index_mode.indexbutton.m add cascade \
	    -label "auto blank skip x gaps..." \
	    -menu {.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.auto_blank_skip}

    menu .vcr.middle.right.index_ctrl.index_mode.indexbutton.m.auto_blank_skip -tearoff 0

    foreach i { 1 2 3 4 5 10 20 30 } {
	set label "sec"
	.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.auto_blank_skip add command \
		-label "$i $label" \
		-command "set_auto_blank_skip $i sec"
    }

    foreach i { 1 2 3 4 5 } {
	set label "min"
	.vcr.middle.right.index_ctrl.index_mode.indexbutton.m.auto_blank_skip add command \
		-label "$i min" \
		-command "set_auto_blank_skip $i min"
    }

    pack append .vcr.middle.right.index_ctrl.index_mode \
	    .vcr.middle.right.index_ctrl.index_mode.indexbutton {left frame center} \
	    .vcr.middle.right.index_ctrl.index_mode.index_display {left frame center}

    # build widget .vcr.middle.right.index_ctrl.media_desc
    frame .vcr.middle.right.index_ctrl.media_desc \
	    -borderwidth {2}

    # build widget .vcr.middle.right.index_ctrl.media_desc.mediabutton
    menubutton .vcr.middle.right.index_ctrl.media_desc.mediabutton \
	    -menu {.vcr.middle.right.index_ctrl.media_desc.mediabutton.m} \
	    -relief {groove} \
	    -padx {0} \
	    -pady {0} \
	    -text {Idxmedia:}

    # build widget .vcr.middle.right.index_ctrl.media_desc.media_display
    label .vcr.middle.right.index_ctrl.media_desc.media_display \
	    -text {- none -} \
	    -anchor {w} \
	    -relief {sunken} \
	    -borderwidth {1} \
	    -width {20}

    # build widget .vcr.middle.right.index_ctrl.media_desc.mediabutton.m
    menu .vcr.middle.right.index_ctrl.media_desc.mediabutton.m -tearoff 0

    update_index_menu

    # pack widget .vcr.middle.right.index_ctrl.media_desc
    pack append .vcr.middle.right.index_ctrl.media_desc \
	    .vcr.middle.right.index_ctrl.media_desc.mediabutton {left frame center} \
	    .vcr.middle.right.index_ctrl.media_desc.media_display {left frame center expand}


    # pack widget .vcr.middle.right.index_ctrl
    pack append .vcr.middle.right.index_ctrl \
	    .vcr.middle.right.index_ctrl.index_mode {left frame center pady 10 expand}\
	    .vcr.middle.right.index_ctrl.media_desc {left frame center pady 10 expand}

    frame .vcr.middle.right.main_index_ctrl
    button .vcr.middle.right.main_index_ctrl.toggle_button \
	    -text {Toggle indexcontrol panel} \
	    -relief {groove} \
	    -borderwidth {2} \
	    -padx {0} \
	    -pady {0} \
	    -highlightthickness 0 \
	    -command {
	toggle_index_ctrl
    }

    frame .vcr.middle.right.main_index_ctrl.fill
    pack append .vcr.middle.right.main_index_ctrl \
	    .vcr.middle.right.main_index_ctrl.toggle_button {frame center padx 10 fillx} \
	    .vcr.middle.right.main_index_ctrl.fill {frame center pady 10}


    # pack widget .vcr.middle.right
    pack append .vcr.middle.right \
	    .vcr.middle.right.top {top frame center expand padx 20 pady 20} \
	    .vcr.middle.right.middle {top frame center expand filly fillx} \
	    .vcr.middle.right.bottom {top frame center expand filly fillx} \
	    .vcr.middle.right.index_ctrl {top frame center padx 10 expand} \
	    .vcr.middle.right.main_index_ctrl {top frame center padx 20 expand}

    # pack widget .vcr.middle
    pack append .vcr.middle \
	    .vcr.middle.left {left frame center expand fill} \
	    .vcr.middle.right {right frame center expand fill}

    # build widget .vcr.position
    frame .vcr.position \
	    -highlightthickness {0} \
	    -borderwidth {2} \
	    -relief {raised}

    # build widget .vcr.position.slider
    scale .vcr.position.slider \
	    -highlightthickness {0} \
	    -borderwidth {1} \
	    -foreground {black} \
	    -font $font(slider) \
	    -label {Seconds} \
	    -length {300} \
	    -orient {horizontal} \
	    -relief {raised} \
	    -showvalue {0} \
	    -tickinterval {60} \
	    -to {10} \
	    -width {12}

    # bindings for the scale
    # (just stop at button press and continue at the point the scale took us...)
 
    bind .vcr.position.slider <ButtonPress-1> {
	global session
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl stop no_button_release]
    }
    
    bind .vcr.position.slider <ButtonRelease-1> {
	global session
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl goto ts [.vcr.position.slider get] no_stop]
    }

    bind .vcr.position.slider <ButtonPress-2> {
	global session
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl stop no_button_release]
    }
    
    bind .vcr.position.slider <ButtonRelease-2> {
	global session
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl goto ts [.vcr.position.slider get] no_stop]
    }

    bind .vcr.position.slider <ButtonPress-3> {
	global session
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl stop no_button_release]
    }
    
    bind .vcr.position.slider <ButtonRelease-3> {
	global session
	if {[check_macro] == "1"} {return}
	set session(state) [vcr_ctrl goto ts [.vcr.position.slider get] no_stop]
    }

    # build widget .vcr.position.total_length
    frame .vcr.position.total_length \
	    -borderwidth {2} \
	    -relief {ridge}

    # build widget .vcr.position.total_length.label
    label .vcr.position.total_length.label \
	    -padx {0} \
	    -pady {0} \
	    -foreground {black} \
	    -text {Total time:}

    # build widget .vcr.position.total_length.value
    label .vcr.position.total_length.value \
	    -padx {0} \
	    -pady {0} \
	    -foreground {black} \
	    -text {0 ms}

    # pack widget .vcr.position.total_length
    pack append .vcr.position.total_length \
	    .vcr.position.total_length.label {top frame center} \
	    .vcr.position.total_length.value {top frame center}

    bind .vcr.position.total_length <Button-1> {
	if {[check_macro] == "1"} {return}
	vcr_ctrl goto end
    }
    bind .vcr.position.total_length.label <Button-1> {
	if {[check_macro] == "1"} {return}
	vcr_ctrl goto end
    }
    bind .vcr.position.total_length.value <Button-1> {
	if {[check_macro] == "1"} {return}
	vcr_ctrl goto end
    }

    # pack widget .vcr.position
    pack append .vcr.position \
	    .vcr.position.slider {left frame center pady 0 expand fill} \
	    .vcr.position.total_length {left frame center fill}

    # build widget .vcr.bottom
    frame .vcr.bottom \
	    -borderwidth {1} \
	    -relief {raised}

    # build widget .vcr.bottom.copyright
    message .vcr.bottom.copyright \
	    -aspect {2000} \
	    -borderwidth {0} \
	    -font $font(copyright) \
	    -foreground {black} \
	    -padx {3} \
	    -pady {3} \
	    -relief {raised} \
	    -text {Copyright (C) 1997 ICSI Berkeley, University of Mannheim}

    # pack widget .vcr.bottom
    pack append .vcr.bottom \
	    .vcr.bottom.copyright {top frame center}

    # pack widget .vcr
    pack append .vcr \
	    .vcr.top {top frame center fill} \
	    .vcr.middle {top frame center expand fill} \
	    .vcr.position {top frame center fill} \
	    .vcr.bottom {top frame center fill}

    # pack widget .
    pack append . \
	    .title {top frame center fillx} \
	    .vcr {top frame center expand fill}

    clear_session
    clear_tmp_session

}

# User defined procedures

proc adios {} {
    quit_vcr
}


# Procedure: quit_vcr
proc quit_vcr {} {
    global {VERSION}
    global session

    if { [ close_session "Quit" ] == "1" } {
	destroy .
    }
}

proc load_session {{session_file ""}} {
    global VERSION session

    if {$session(loaded) == 1} {
	set_init
	set session(loaded) [vcr_ctrl delete]
	clear_media
	clear_session
	clear_tmp_session
	update_main_win
    }

    set session_file [check_file_ext $session_file]
    if { [read_session_file $session_file 1] == 0 } {
	set session(loaded) "0"
	clear_media
	clear_session
	clear_tmp_session
	update_main_win
	return "0"
    } else {
	clear_media
	update_session
	if { $session(rec_enabled) == "1" } {
	    set session(rec_enabled) [vcr_ctrl set_rec_enabled]
	}
	update_main_win
	return "1"
    }
}

proc open_session { {new 0} {msg ""} } {
    global VERSION session tmp_session media

    if {$session(loaded) == 1} {
	stop_tools 1
	set_init
	set session(loaded) [vcr_ctrl delete]
	clear_media
	clear_session
	clear_tmp_session
	update_main_win
    }

    if {$new == 0} {
	set name [FSBox "Select MBone VCR Session" "" "" "" "*.vcr" "$msg"]
	if { ($name != "") } {
	    set name [check_file_ext $name]
	    if { [IsAFile $name] == 1 } {
		if {[read_session_file $name] == 0} {
		    set_init
		    set session(loaded) "0"
		    clear_media
		    clear_session
		    clear_tmp_session
		} else {
		    clear_media
		    update_session
		    if { $session(rec_enabled) == "1" } {
			set session(rec_enabled) [vcr_ctrl set_rec_enabled]
		    }
		}
		update_main_win
	    } else {
		AlertBox "\n $VERSION error: \n\n File $name not found. \n"\
			"" "" "$VERSION" Ok
	    }
	}
    } else {
	edit_session "1"
    }
    return
}

proc close_session {{msg "Close"}} {
    global VERSION session

    set ret 0
    set_init
    if {($session(loaded) == 1) && ($session(saved) == "0")} {
	set ret [AlertBox "\n $VERSION warning: \n\n Session \"$session(name)\" \
		not saved. \n\n" "" "" "$VERSION ($msg Session)" " $msg " " Save and $msg " " Cancel "]
	if { $ret == "2" } { return "0" }
	if { $ret == "1" } {
	    if { [save_session] == 0 } {
		return "0"
	    }
	}
    }

    stop_tools 1

    if { $session(loaded) == "1" } {
	set session(loaded) [vcr_ctrl delete]
    }
    clear_media
    clear_session
    clear_tmp_session
    update_main_win
    return "1"
}

proc enable_recording {} {
    global session VERSION
    if { $session(loaded) == 1} {
	if { $session(read_only) != "" } {
	    AlertBox "\n Sorry, this session is read-only! \n"\
		    "" "" "$VERSION (Info)" Ok
	    set session(rec_enabled) 0
	} else {
	    set session(rec_enabled) [vcr_ctrl set_rec_enabled]
	}
    } else {
	set session(rec_enabled) 1
    }
    if {$session(rec_enabled) != 0} {
	# for some reason [vcr_ctrl set_rec_enabled] sometimes returns nonsense 
	# (like after#11) ?!? therefore we check for !=0 and set session(rec_enabled) 
	# explicitly to 1
	set session(rec_enabled) 1
	.vcr.middle.right.bottom.rec configure -state normal
    }
}

proc disable_recording {} {
    global session
    if { $session(state) == "recording" } {
	set session(state) [vcr_ctrl stop]
    }
    if {$session(loaded)==1} {
	set session(rec_enabled) [vcr_ctrl set_rec_disabled]
    } else {
	set session(rec_enabled) 0
    }

    if {$session(rec_enabled) != 1} {
	# for some reason [vcr_ctrl set_rec_disabled] sometimes returns nonsense 
	# (like after#11) ?!? therefore we check for !=1 and set session(rec_enabled) 
	# explicitly to 0
	set session(rec_enabled) 0
	.vcr.middle.right.bottom.rec configure -state disabled
    }
}


# Procedure: SetXbm
proc SetXbm {} {
  global interactive bmp
  global bmp_goto_start bmp_rew bmp_rec bmp_stop bmp_play bmp_ff bmp_goto_end
  global bmp_next_speaker bmp_next_ts bmp_next_time bmp_next_blank
  global bmp_prev_speaker bmp_prev_ts bmp_prev_time bmp_prev_blank
  global bmp_prev_index_1 bmp_prev_index_2 bmp_prev_index_3 bmp_prev_index_4 bmp_prev_index_5
  global bmp_next_index_1 bmp_next_index_2 bmp_next_index_3 bmp_next_index_4 bmp_next_index_5

    if { $interactive == 1 } {
	# change to external bitmap files for interactive mode
	set bmp(led)		{@../xbmp/led.xbm}
	set bmp(goto_start)	{@../xbmp/goto_start.xbm}
	set bmp(rew)		{@../xbmp/rew.xbm}
	set bmp(rec)		{@../xbmp/rec.xbm}
	set bmp(stop)		{@../xbmp/stop.xbm}
	set bmp(play)		{@../xbmp/play.xbm}
	set bmp(ff)		{@../xbmp/ff.xbm}
	set bmp(goto_end)	{@../xbmp/goto_end.xbm}
	set bmp(next_index_1)	{@../xbmp/next_index_1.xbm}
	set bmp(next_index_2)	{@../xbmp/next_index_2.xbm}
	set bmp(next_index_3)	{@../xbmp/next_index_3.xbm}
	set bmp(next_index_4)	{@../xbmp/next_index_4.xbm}
	set bmp(next_index_5)	{@../xbmp/next_index_5.xbm}
	set bmp(next_speaker)	{@../xbmp/next_speaker.xbm}
	set bmp(next_ts)	{@../xbmp/next_ts.xbm}
	set bmp(next_time)	{@../xbmp/next_time.xbm}
	set bmp(next_blank)	{@../xbmp/next_blank.xbm}
	set bmp(prev_index_1)	{@../xbmp/prev_index_1.xbm}
	set bmp(prev_index_2)	{@../xbmp/prev_index_2.xbm}
	set bmp(prev_index_3)	{@../xbmp/prev_index_3.xbm}
	set bmp(prev_index_4)	{@../xbmp/prev_index_4.xbm}
	set bmp(prev_index_5)	{@../xbmp/prev_index_5.xbm}
	set bmp(prev_speaker)	{@../xbmp/prev_speaker.xbm}
	set bmp(prev_ts)	{@../xbmp/prev_ts.xbm}
	set bmp(prev_time)	{@../xbmp/prev_time.xbm}
	set bmp(prev_blank)	{@../xbmp/prev_blank.xbm}
    } else {
	# change to internal bitmaps for noninteractive mode
	set bmp(led)		{bmp_led}
	set bmp(goto_start)	{bmp_goto_start}
	set bmp(rew)		{bmp_rew}
	set bmp(rec)		{bmp_rec}
	set bmp(stop)		{bmp_stop}
	set bmp(play)		{bmp_play}
	set bmp(ff)		{bmp_ff}
	set bmp(goto_end)	{bmp_goto_end}

	set bmp(next_index_1)	{bmp_next_index_1}
	set bmp(next_index_2)	{bmp_next_index_2}
	set bmp(next_index_3)	{bmp_next_index_3}
	set bmp(next_index_4)	{bmp_next_index_4}
	set bmp(next_index_5)	{bmp_next_index_5}
	set bmp(next_speaker)	{bmp_next_speaker}
	set bmp(next_ts)	{bmp_next_ts}
	set bmp(next_time)	{bmp_next_time}
	set bmp(next_blank)	{bmp_next_blank}

	set bmp(prev_index_1)	{bmp_prev_index_1}
	set bmp(prev_index_2)	{bmp_prev_index_2}
	set bmp(prev_index_3)	{bmp_prev_index_3}
	set bmp(prev_index_4)	{bmp_prev_index_4}
	set bmp(prev_index_5)	{bmp_prev_index_5}
	set bmp(prev_speaker)	{bmp_prev_speaker}
	set bmp(prev_ts)	{bmp_prev_ts}
	set bmp(prev_time)	{bmp_prev_time}
	set bmp(prev_blank)	{bmp_prev_blank}
    }
}

proc update_main_win {} {
    global session READ_ONLY
    global cmd_start_time

    if { $session(loaded) == "1" } {

        .vcr.top.session.m entryconfigure 4 -state normal
        .vcr.top.session.m entryconfigure 5 -state normal
        .vcr.top.session.m entryconfigure 7 -state normal
        .vcr.top.session.m entryconfigure 8 -state normal
        .vcr.top.session.m entryconfigure 10 -state normal
        .vcr.top.session.m entryconfigure 11 -state normal

	.vcr.top.speed.selection entryconfigure 0 -state normal
	.vcr.top.speed.selection entryconfigure 1 -state normal
	.vcr.top.speed.selection entryconfigure 2 -state normal
	
	if { $session(read_only) == "" } {
	    .vcr.top.record.m entryconfigure 0 -state normal
	    .vcr.top.record.m entryconfigure 1 -state normal
	    .vcr.top.record.m entryconfigure 2 -state normal
	}
	.vcr.top.tools.m entryconfigure 0 -state normal

	if {$cmd_start_time == ""} {
	    .vcr.top.macro.m entryconfigure 0 -state normal
	    .vcr.top.macro.m entryconfigure 1 -state disabled
	} else {
	    .vcr.top.macro.m entryconfigure 0 -state disabled
	    .vcr.top.macro.m entryconfigure 1 -state normal
	}

	.vcr.middle.right.bottom.skip_back configure -state normal
	.vcr.middle.right.bottom.rew configure -state normal
	if {$session(rec_enabled) == 0} {
	    .vcr.middle.right.bottom.rec configure -state disabled
	} else {
	    .vcr.middle.right.bottom.rec configure -state normal
	}
	.vcr.middle.right.bottom.stop configure -state normal
	.vcr.middle.right.bottom.play configure -state normal
	.vcr.middle.right.bottom.ff configure -state normal
	.vcr.middle.right.bottom.skip_forw configure -state normal

	.vcr.middle.left.top.media_list.listbox delete 0 end

	for {set i 1} {$i <= $session(nb_of_media)} {incr i} {
	    if { [string length $session(media_list,$i,media_desc)] > 0 } {
		set x [format "%s (%s)" $session(media_list,$i,media_name) \
			$session(media_list,$i,media_desc)]
	    } else {
		set x $session(media_list,$i,media_name)
	    }
	    .vcr.middle.left.top.media_list.listbox insert end $x
	}

    } else {

	.vcr.middle.left.top.media_list.listbox delete 0 end

	set_init
	clear_session
	clear_tmp_session

        .vcr.top.session.m entryconfigure 4 -state disabled
        .vcr.top.session.m entryconfigure 5 -state disabled
        .vcr.top.session.m entryconfigure 7 -state disabled
        .vcr.top.session.m entryconfigure 8 -state disabled
	.vcr.top.session.m entryconfigure 10 -state disabled
	.vcr.top.session.m entryconfigure 11 -state disabled

	.vcr.top.speed.selection entryconfigure 0 -state disabled
	.vcr.top.speed.selection entryconfigure 1 -state disabled
	.vcr.top.speed.selection entryconfigure 2 -state disabled

	.vcr.top.record.m entryconfigure 0 -state disabled
	.vcr.top.record.m entryconfigure 1 -state disabled
	.vcr.top.record.m entryconfigure 2 -state disabled

	.vcr.top.tools.m entryconfigure 0 -state disabled

	.vcr.top.macro.m entryconfigure 0 -state normal
	.vcr.top.macro.m entryconfigure 1 -state disabled

	.vcr.middle.right.bottom.skip_back configure -state disabled
	.vcr.middle.right.bottom.rew configure -state disabled
	.vcr.middle.right.bottom.rec configure -state disabled
	.vcr.middle.right.bottom.stop configure -state disabled
	.vcr.middle.right.bottom.play configure -state disabled
	.vcr.middle.right.bottom.ff configure -state disabled
	.vcr.middle.right.bottom.skip_forw configure -state disabled

	.vcr.middle.right.index_ctrl.media_desc.mediabutton.m delete 0 last
	.vcr.middle.right.index_ctrl.media_desc.mediabutton.m add command \
		-label "- none -" \
		-command {
	    global session
	    set session(index_media) 0
	    if {[winfo exists .index_ctrl] == "1"} {
		.index_ctrl.middle.media_desc.media_display configure -text "- none -"
		.index_ctrl.middle.index_list.listbox delete 0 end
		.index_ctrl.middle.index_list.listbox insert end "- none -"
	    }
	}
    }

    .vcr.middle.left.top.session_name.label configure -text [center $session(name) 40]

    if {$session(saved) == "1"} {
	.vcr.top.session.m entryconfigure 4 -label {Save Session (not needed)}
    } else {
	.vcr.top.session.m entryconfigure 4 -label {Save Session (needed)}
    }


    if { $session(read_only) != "" } {
        .vcr.top.session.m entryconfigure 4 -state disabled
        .vcr.top.session.m entryconfigure 7 -label "View Session"
	if { $session(loaded) == "1" } {
	    .vcr.middle.left.top.read_only.label configure -text $READ_ONLY
	    set session(rec_enabled) [vcr_ctrl set_rec_disabled]
	    if {$session(rec_enabled) == 0} {
		.vcr.middle.right.bottom.rec configure -state disabled
	    } else {
		.vcr.middle.right.bottom.rec configure -state normal
	    }
	} else {
	    set session(rec_enabled) 0
	}
    } else {
        .vcr.top.session.m entryconfigure 7 -label "Edit Session"
        .vcr.middle.left.top.read_only.label configure -text ""
    }
    update_leds
    update_mute
}

# procedure to show window .starttools
global start_playing
proc start_tools_win {{args ""}} {
    global VERSION font session protocols start_playing

    set start_playing $args

    if {$session(loaded) != 1} { return }
    if { [winfo exists .starttools] == "1" } { return }

    if { $session(tools_started) != "" } {
	if { "$args" == "play" } {
	    set session(state) [vcr_ctrl play]
	    return
	}
    }

    # build widget .starttools
    toplevel .starttools  \
	    -relief {raised}
	
    # bindings
    bind .starttools <Enter> { focus %W }
    bind .starttools <Return> { tools_ok }
    bind .starttools <Escape> { tools_esc }
    
    # Window manager configurations
    global tkVersion
    wm positionfrom .starttools program
    wm sizefrom .starttools program
    wm title .starttools "$VERSION (Start Session Tools)"
    
    # build widget .starttools.top
    frame .starttools.top \
	    -borderwidth {2} \
	    -relief {raised}
    
    # build widget .starttools.top.title
    label .starttools.top.title \
	    -font $font(window_title) \
	    -height {2} \
	    -text {Select Session Tools to Start:}
    
    # pack widget .starttools.top
    pack append .starttools.top \
	    .starttools.top.title {expand frame center pady 5 padx 5}
    
    # build widget .starttools.middle
    frame .starttools.middle \
	    -borderwidth {2} \
	    -relief {raised}
    
    
    for {set i 1} { $i <= $session(nb_of_media) } {incr i} {
	set get_how2start [format "how2start_%s" \
		$protocols($session(media_list,$i,media_type))]
	set start_cmd [$get_how2start ""\
		$session(media_list,$i,media_cmd)\
		$session(media_list,$i,media_ip)\
		$session(media_list,$i,media_port)\
		$session(media_list,$i,media_ttl)\
		$session(media_list,$i,media_cid)\
		$session(name)]

	set session(media_list,$i,start_it) 0

	# build widget .starttools.middle.checkbutton$i
	frame .starttools.middle.checkbutton$i
	
	# build widget .starttools.middle.checkbutton$i.button
	checkbutton .starttools.middle.checkbutton$i.button \
		-text "$start_cmd" \
		-width {45} \
		-anchor {w} \
		-variable "session(media_list,$i,start_it)"
	pack append .starttools.middle.checkbutton$i \
		.starttools.middle.checkbutton$i.button {expand frame center pady 5}
    }
    
    for {set i 1} { $i <= $session(nb_of_media) } {incr i} {
	# pack widget .starttools.middle
	pack append .starttools.middle \
		.starttools.middle.checkbutton$i {top frame center pady 5 expand fill}
    }
    # build widget .starttools.bottom
    frame .starttools.bottom \
	    -borderwidth {2} \
	    -relief {raised}
    
    frame .starttools.bottom.hostname \
	    -borderwidth {2} \
	    -relief {groove}
    label .starttools.bottom.hostname.label \
	    -text "Start tools on host: "
    entry .starttools.bottom.hostname.entry \
	    -textvariable {hostname} \
	    -relief {sunken} \
	    -width {22}
    bind .starttools.bottom.hostname.entry <Return> {
	.starttools.bottom.ok invoke
    }
    
    pack append .starttools.bottom.hostname \
	    .starttools.bottom.hostname.label {left frame center padx 10} \
	    .starttools.bottom.hostname.entry {left frame center padx 10 expand}
    
    if { $start_playing == "play" } {
	set label "Start Tools and Play"
    } else {
	set label "Start Tools"
    }
    
    # build widget .starttools.bottom.ok
    button .starttools.bottom.ok \
	    -text "$label" \
	    -command {
	if { [start_tools $hostname] == 1 } { 
	    vcr_grab release .starttools
	    catch {destroy .starttools}
	}
    }
    
    # build widget .starttools.bottom.cancel
    button .starttools.bottom.cancel \
	    -text {Cancel} \
	    -command {
	if { "$hostname" == ""} { set hostname "localhost" }
	vcr_grab release .starttools
	catch {destroy .starttools}
    }
    
    # pack widget .starttools.bottom
    pack append .starttools.bottom \
	    .starttools.bottom.hostname {top frame center padx 10 pady 10 expand fill} \
	    .starttools.bottom.ok {left frame center padx 20 pady 10} \
	    .starttools.bottom.cancel {right frame center padx 20 pady 10}
    
    
    # pack widget .starttools
    pack append .starttools \
	    .starttools.top {top frame center padx 10 pady 10 expand fill} \
	    .starttools.middle {top frame center padx 20 pady 20 expand fill} \
	    .starttools.bottom {top frame center padx 10 pady 10 expand fill}
    
    vcr_grab .starttools
}

proc start_tools { {hostname ""} } {
    global session protocols PS_CMD
    global start_playing
    set error "0"

    set something_started 0
    for {set i 1} { $i <= $session(nb_of_media) } {incr i} {
	set start_it 0
	if { [info exist session(media_list,$i,start_it)] == 0 } {
	    set session(media_list,$i,media_pid) 0
	    set start_it 1
	} else {
	    if { $session(media_list,$i,start_it) == "1" } {
		set start_it 1
	    } else {
		set session(media_list,$i,media_pid) 0
	    }
	}
	if { $start_it == 1} {
	    set get_how2start [format "how2start_%s" \
		    $protocols($session(media_list,$i,media_type))]
	    set start_cmd [$get_how2start $hostname\
		    $session(media_list,$i,media_cmd)\
		    $session(media_list,$i,media_ip)\
		    $session(media_list,$i,media_port)\
		    $session(media_list,$i,media_ttl)\
		    $session(media_list,$i,media_cid)\
		    $session(name)]
	    if { $start_cmd == "" } {
		AlertBox "\n Sorry, don't know how to start media\
			\"$session(media_list,$i,media_name)\".\n\n\
			Function $get_how2start returns NULL.\n\n"\
			"" "" "$VERSION" Ok
		set error 1
	    } else {
		set cmd_list [split $start_cmd]
		if { [lindex $cmd_list 0] == "rsh" } {
		    set rsh_cmd [join [lrange $cmd_list 0 1]]
		    set media_cmd [join [lrange $cmd_list 2 end]]
		} else {
		    set media_cmd ""
		}
		set start_cmd "$start_cmd &"
		puts $start_cmd
		catch {set session(media_list,$i,media_pid) [eval exec $start_cmd]}
		if { $media_cmd != "" } {
		    set processes [eval exec "$rsh_cmd $PS_CMD"]
		    set p_list [split $processes "\n"]
		    set p_index [lsearch $p_list "*?:?? $media_cmd"]
		    if { $p_index != -1 } {
			set process [split [string trim [lindex $p_list $p_index]]]
			set session(media_list,$i,media_pid) [lindex $process 0]
		    } else {
			set session(media_list,$i,media_pid) 0
		    }
		}
		catch {unset start_cmd cmd_list rsh_cmd processes process p_list p_index}
		set something_started 1
	    }
	}
	set session(media_list,$i,start_it) 0
    }
    if {$error != 0} { return 0 }
    if { $something_started == 1 } {
	if { "$hostname" == "" } { set hostname "localhost" }
	set session(tools_started) $hostname
    }
    if { $start_playing == "play" } {
	set session(state) [vcr_ctrl play]
    }
    return 1
}


proc stop_tools {{ ask 0 }} {
    global session VERSION PS_CMD alertBox
    if {$session(loaded) != 1} { return }
    if { $session(tools_started) != "" } {
	set ret 1
	if { $ask == 1 } {
	    set ret [AlertBox "\n $VERSION: \n Do you want to close the session-tools? \n" \
		    "" "" "$VERSION (Close Session Tools)" "Yes" "No"]
	} else {
	    set ret 0
	}
	if { $ret == 0 } {
	    set old $alertBox(after)
	    set {alertBox(after)} {4}
	    AlertBox "\n $VERSION: \n Looking for open session-tools. \n\n\
		    Please wait ... \n\n" \
		    "" "" "$VERSION (Close Session Tools)" " OK "
	    set {alertBox(after)} $old
	    unset old
	    set rsh ""
	    set kill 0
	    if { $session(tools_started) != "localhost" } {
		set rsh "rsh $session(tools_started)"
	    }
	    for {set i 1} { $i <= $session(nb_of_media) } {incr i} {
		set processes [eval exec "$rsh $PS_CMD"]
		set p_list [split $processes "\n"]
		set p_index [lsearch $p_list "*$session(media_list,$i,media_pid) *"]
		if { $p_index == -1 } {
		    set session(media_list,$i,media_pid) 0
		} else {
		    set kill 1
		}
	    }
	    if { $kill == 1 } {
		if  { $ret == 0 } {
		    set rsh ""
		    if { $session(tools_started) != "localhost" } {
			set rsh "rsh $session(tools_started)"
		    }
		    for {set i 1} { $i <= $session(nb_of_media) } {incr i} {
			if { $session(media_list,$i,media_pid) > 0 } {
			    puts [string trim "$rsh kill $session(media_list,$i,media_pid)"]
			    catch {set session(media_list,$i,media_pid) [eval exec $rsh kill $session(media_list,$i,media_pid)]}
			    set session(media_list,$i,media_pid) 0
			}
		    }
		    set session(tools_started) ""
		}
	    } else {
		set session(tools_started) ""
		if { $ask == 1 } {
		    AlertBox "\n $VERSION: \n\n Sorry, couldn't find the session-tools, \n\
			    they were probably closed already. \n\n" \
			    "" "" "$VERSION (Close Session Tools)" " OK "
		}
	    }
	}
    }
}

proc tools_ok {} {
    if { [winfo exists .starttools.bottom.ok] == 1 } {
	.starttools.bottom.ok invoke
    }
}
proc tools_esc {} {
    if { [winfo exists .starttools.bottom.cancel] == 1 } {
	.starttools.bottom.cancel invoke
    }
}

proc toggle_index_ctrl { { new "" } } {
    global session
    if { $new == "" } {
	catch {.index_ctrl configure} error
	set error [string first "invalid command name" $error]
	if { $error == -1 } {
	    set new "0"
	} else {
	    set new "1"
	}
    }
    if { $new == "0" } {
	destroy .index_ctrl
	set session(index_ctrl) "0"
    } else {
	show_index_ctrl
    }
}


proc reread_session {} {
    global session
    set session_file $session(filename)
    close_session
    after 1000
    load_session $session_file
    enable_recording
}

proc blank_session {} {
    global session protocols VERSION

    if { $session(read_only) != "" } return

    if { [AlertBox "\n $VERSION warning. \n Do you really want to blank the session? \n" "" "" "$VERSION (Blank Session)" Yes Cancel] == "1"} {
	return
    }

    for { set i 1} { $i <= $session(nb_of_media) } { incr i } {
	set file "$session(data_dir)/$session(file_prefix)_$protocols($session(media_list,$i,media_type))_$i"
	catch {eval exec rm -f $file.rec}
	catch {eval exec rm -f $file.idx}
    }
    set session_file $session(filename)
    set_init
    close_session
    after 1000
    load_session $session_file
    enable_recording
}

proc demo_version {} {
    global DEMO_VERSION VERSION
    if { $DEMO_VERSION == "1" } {
	AlertBox "\n $VERSION warning: \n\n Sorry, this is not part of this Demo Version! \
		\n\n" "" "" "$VERSION" Ok
	return "1"
    }
    return "0"
}

proc center {{s ""} {i 0}} {
    if {$s == ""} {return ""}
    if {($i == 0) || ($i <= [string length $s])} {return [string trim $s]}
    if {[string length $s] >= $i} {return [string range $s 0 [expr $i-1]]}
    set i1 [expr floor(($i - [string length $s])/2)]
    set i2 [expr ceil(($i - [string length $s])/2)]
    set ret $s
    for {set i 1} {$i < $i1} { incr i} {set ret " $ret"}
    for {set i 1} {$i < $i2} { incr i} {set ret "$ret "}
    return $ret
}


proc press_button { {name ""} } {
    global session colors

    if { $name == "" } { return }

    if {$session(active_button) == $name} {return}

    if { ($name == "rew") || ($name == "rec") || ($name == "play") || ($name == "ff") } {
	.vcr.middle.right.bottom.$name configure -relief sunken
	if { $session(blink_mode) == "1" } {
	    set session(start_blinking) ""
	    set session(active_button) $name
	    blink .vcr.middle.right.bottom.$name $name \
		    $colors($name,active) \
		    $colors($name,blink)
	} else {
	    .vcr.middle.right.bottom.$name configure -foreground $colors($name,active)
	    .vcr.middle.right.bottom.$name configure -activeforeground $colors($name,active)
	    set session(active_button) $name
	    set session(start_blinking) "blink .vcr.middle.right.bottom.$name $name \
		    $colors($name,active) \
		    $colors($name,blink)"
	}
    }
}

proc release_button { {name ""} } {
    global session colors

    if {$name == ""} {
	if { $session(active_button) != "" } {
	    set name $session(active_button)
	    set session(active_button) ""
	} else {
	    return
	}
    }

    .vcr.middle.right.bottom.$name configure -relief raised
    if { $session(active_button) == $name } { set session(active_button) "" }
    .vcr.middle.right.bottom.$name configure -foreground $colors($name,normal)
    .vcr.middle.right.bottom.$name configure -activeforeground $colors($name,normal)
    set session(start_blinking) ""
}


proc blink { w name color1 color2 {interval 500} } {
    global session colors

    if { $session(active_button) == $name } {
	if { $session(blink_mode) == "1" } {
	    $w configure -foreground $color1
	    $w configure -activeforeground $color1
	    after $interval [list blink $w $name $color2 $color1 $interval]
	} else {
	    .vcr.middle.right.bottom.$name configure -foreground $colors($name,active)
	    .vcr.middle.right.bottom.$name configure -activeforeground $colors($name,active)
	    set session(start_blinking) "blink $w $name $color2 $color1 $interval"
	}
    }
}

proc vcr_grab { args } {
    global DEVELOPMENT_VERSION
    if { $DEVELOPMENT_VERSION == "0" } {
	catch "grab $args"
    }
}

proc set_goto_start_end {} {
    global index_values session bmp skip_time
    no_auto_blank_skip
    .vcr.middle.right.index_ctrl.index_mode.index_display configure -text "start/end"
    .vcr.middle.right.bottom.skip_back configure -bitmap "$bmp(goto_start)" \
	    -command {
	set session(state) [vcr_ctrl goto start]
    }
    bind .vcr.middle.right.bottom.skip_back <Enter> {
	button_hint .vcr.middle.right.bottom.skip_back "goto start" "not available"
    }

    .vcr.middle.right.bottom.skip_forw configure -bitmap "$bmp(goto_end)" \
	    -command {
	set session(state) [vcr_ctrl goto end]
    }
    bind .vcr.middle.right.bottom.skip_forw <Enter> {
	button_hint .vcr.middle.right.bottom.skip_forw "goto end" "not available"
    }

}

proc set_goto_prev_next_sync {} {
    global index_values session bmp skip_time
    no_auto_blank_skip
    .vcr.middle.right.index_ctrl.index_mode.index_display configure -text "prev/next sync unit"
    .vcr.middle.right.bottom.skip_back configure -bitmap $bmp(prev_ts)\
	    -command {
	if { [can_use_index] == 0 } { return }
	vcr_ctrl goto prev_index $session(index_media) $index_values(NEW_TS)
    }
    bind .vcr.middle.right.bottom.skip_back <Enter> {
	button_hint .vcr.middle.right.bottom.skip_back "goto previous sync unit" \
		"not available"
    }

    .vcr.middle.right.bottom.skip_forw configure -bitmap $bmp(next_ts)\
	    -command {
	if { [can_use_index] == 0 } { return }
	vcr_ctrl goto next_index $session(index_media) $index_values(NEW_TS)
    }
    bind .vcr.middle.right.bottom.skip_forw <Enter> {
	button_hint .vcr.middle.right.bottom.skip_forw "goto next sync unit" \
		"not available"
    }

}

proc set_goto_prev_next_speaker {} {
    global index_values session bmp skip_time
    no_auto_blank_skip
    .vcr.middle.right.index_ctrl.index_mode.index_display configure -text "prev/next speaker"
    .vcr.middle.right.bottom.skip_back configure -bitmap $bmp(prev_speaker)\
	    -command {
	if { [can_use_index] == 0 } { return }
	vcr_ctrl goto prev_index $session(index_media) $index_values(NEW_SP)
    }
    bind .vcr.middle.right.bottom.skip_back <Enter> {
	button_hint .vcr.middle.right.bottom.skip_back "goto previous speaker" \
		"not available"
    }

    .vcr.middle.right.bottom.skip_forw configure -bitmap $bmp(next_speaker)\
	    -command {
	if { [can_use_index] == 0 } { return }
	vcr_ctrl goto next_index $session(index_media) $index_values(NEW_SP)
    }
    bind .vcr.middle.right.bottom.skip_forw <Enter> {
	button_hint .vcr.middle.right.bottom.skip_forw "goto next speaker" \
		"not available"
    }

}
proc set_skip_time {i unit} {
    global index_values session bmp skip_time

    no_auto_blank_skip
    if {$unit == "min"} {
	set label "mins"
	if { $i == 1 } { set label "min" }
	set skip_time [expr $i * 60]
    } else {
	set label "secs"
	if { $i == 1 } { set label "sec" }
	set skip_time $i
    }

    .vcr.middle.right.index_ctrl.index_mode.index_display configure -text "prev/next $i $label"

    .vcr.middle.right.bottom.skip_back configure -bitmap "$bmp(prev_time)" \
	    -command {
	global session skip_time
	set ts [expr [.vcr.position.slider get]-$skip_time]
	if { $ts >= 0 } {
	    set session(state) [vcr_ctrl goto ts $ts]
	} else {
	    set session(state) [vcr_ctrl goto start]
	}
    }
    bind .vcr.middle.right.bottom.skip_back <Enter> "
	button_hint .vcr.middle.right.bottom.skip_back \"skip backwards $i $unit\" \
		\"not available\" 
    "


    .vcr.middle.right.bottom.skip_forw configure -bitmap "$bmp(next_time)" \
	    -command {
	global session skip_time
	set ts [expr [.vcr.position.slider get]+$skip_time]
	if { [expr $ts * 1000] < [vcr_ctrl get_length] } {
	    set session(state) [vcr_ctrl goto ts $ts]
	} else {
	    set session(state) [vcr_ctrl goto end]
	}
    }
    bind .vcr.middle.right.bottom.skip_forw <Enter> "
	button_hint .vcr.middle.right.bottom.skip_forw \"skip forward $i $unit\" \
		\"not available\"
    "


}

proc set_blank_skip {i unit} {
    global index_values session bmp blank_skip_time

    no_auto_blank_skip
    if {$unit == "min"} {
	set label "min"
	set blank_skip_time [expr $i * 60000]
    } else {
	set label "sec"
	set blank_skip_time [expr $i * 1000]
    }

    .vcr.middle.right.index_ctrl.index_mode.index_display configure -text "blank skip to $i $label gap"

    .vcr.middle.right.bottom.skip_back configure -bitmap "$bmp(prev_blank)" \
	    -command {
	global session
	if { [can_use_index] == 0 } { return }
	set session(state) [vcr_ctrl goto prev_blank $session(index_media) $blank_skip_time]
    }
    bind .vcr.middle.right.bottom.skip_back <Enter> {
	button_hint .vcr.middle.right.bottom.skip_back "blank skip backwards" \
		"not available"
    }


    .vcr.middle.right.bottom.skip_forw configure -bitmap "$bmp(next_blank)" \
	    -command {
	global session
	if { [can_use_index] == 0 } { return }
	set session(state) [vcr_ctrl goto next_blank $session(index_media) $blank_skip_time]
    }
    bind .vcr.middle.right.bottom.skip_forw <Enter> {
	button_hint .vcr.middle.right.bottom.skip_forw "blank skip forward" \
		"not available"
    }


}

proc no_auto_blank_skip {} {
    vcr_ctrl no_auto_blank_skip
    set auto_blank_skip_time 0
}

proc set_auto_blank_skip {i unit} {
    global index_values session bmp auto_blank_skip_time

    if { ($i > 0 ) && ([can_use_index] == 0) } { return }

    if {$unit == "min"} {
	set label "min"
	set auto_blank_skip_time [expr $i * 60000 ]
    } else {
	set label "sec"
	set auto_blank_skip_time [expr $i * 1000]
    }

    vcr_ctrl set_auto_blank_skip $session(index_media) $auto_blank_skip_time

    .vcr.middle.right.index_ctrl.index_mode.index_display configure -text "auto blank skip $i $label gaps"

    .vcr.middle.right.bottom.skip_back configure -bitmap "$bmp(prev_blank)" \
	    -command {
	global session
	if { [can_use_index] == 0 } { return }
	set session(state) [vcr_ctrl goto prev_blank $session(index_media) $auto_blank_skip_time]
    }
    bind .vcr.middle.right.bottom.skip_back <Enter> {
	button_hint .vcr.middle.right.bottom.skip_back "blank skip backwards" \
		"not available"
    }


    .vcr.middle.right.bottom.skip_forw configure -bitmap "$bmp(next_blank)" \
	    -command {
	global session
	if { [can_use_index] == 0 } { return }
	set session(state) [vcr_ctrl goto next_blank $session(index_media) $auto_blank_skip_time]
    }
    bind .vcr.middle.right.bottom.skip_forw <Enter> {
	button_hint .vcr.middle.right.bottom.skip_forw "blank skip forward" \
		"not available"
    }


}

proc set_user_idx {i} {
    global index_values session bmp
    no_auto_blank_skip
    .vcr.middle.right.index_ctrl.index_mode.index_display configure -text "prev/next index $i"

    .vcr.middle.right.bottom.skip_back configure -bitmap "$bmp(prev_index_$i)" \
	    -command "goto_prev_idx $i"
    bind .vcr.middle.right.bottom.skip_back <Double-1> "
	.vcr.middle.right.bottom.skip_back configure -command \"goto_prev_idx $i dbl\"
	.vcr.middle.right.bottom.skip_back invoke
	.vcr.middle.right.bottom.skip_back configure -command \"goto_prev_idx $i\"
    "
    bind .vcr.middle.right.bottom.skip_back <Enter> "
	button_hint .vcr.middle.right.bottom.skip_back \"goto previous user level index $i\" \
		\"not available\"
    "

    .vcr.middle.right.bottom.skip_forw configure -bitmap "$bmp(next_index_$i)" \
	    -command "goto_next_idx $i"
    bind .vcr.middle.right.bottom.skip_forw <Double-1> "
	.vcr.middle.right.bottom.skip_forw configure -command \"goto_next_idx $i dbl\"
	.vcr.middle.right.bottom.skip_forw invoke
	.vcr.middle.right.bottom.skip_forw configure -command \"goto_next_idx $i\"
    "
    bind .vcr.middle.right.bottom.skip_forw <Enter> "
	button_hint .vcr.middle.right.bottom.skip_forw \"goto next user level index $i\" \
		\"not available\"
    "
}

proc goto_prev_idx {i {dbl ""}} {
    global index_values session bmp
    if { [can_use_index] == 0 } { return }
    vcr_ctrl goto prev_index $session(index_media) $index_values(USER$i) $dbl
}
proc goto_next_idx {i {dbl ""}} {
    global index_values session bmp
    if { [can_use_index] == 0 } { return }
    vcr_ctrl goto next_index $session(index_media) $index_values(USER$i) $dbl
}


proc mute_media {{selected ""}} {
    global session
    if { $selected == "" } { return }
    for {set i 1} { $i <= [llength $selected] } { incr i } {
	set listindex [lrange $selected [expr $i - 1] [expr $i - 1]]
	set index [expr $listindex + 1]
	if { $session(media_list,$index,media_mute) == "0" } {
	    set session(media_list,$index,media_mute) "1"
	    vcr_ctrl mute_media $index
	    set tmp_str [.vcr.middle.left.top.media_list.listbox get $listindex]
	    set tmp_str "<$tmp_str>"
	    .vcr.middle.left.top.media_list.listbox delete $listindex
	    .vcr.middle.left.top.media_list.listbox insert $listindex $tmp_str
	} else {
	    set session(media_list,$index,media_mute) "0"
	    vcr_ctrl unmute_media $index
	    set tmp_str [.vcr.middle.left.top.media_list.listbox get $listindex]
	    set tmp_str [string range $tmp_str 1 [expr [string length $tmp_str] - 2]]
	    .vcr.middle.left.top.media_list.listbox delete $listindex
	    .vcr.middle.left.top.media_list.listbox insert $listindex $tmp_str
	}
    }
}

proc update_mute {} {
    global session
    for {set i 1} {$i <= $session(nb_of_media)} {incr i} {
	if { $session(media_list,$i,media_mute) == "1" } {
	    vcr_ctrl mute_media $i
	    set tmp_str [.vcr.middle.left.top.media_list.listbox get [expr $i -1]]
	    if { [string index $tmp_str 0] != "<" } {
		set tmp_str "<$tmp_str>"
		.vcr.middle.left.top.media_list.listbox delete [expr $i -1]
		.vcr.middle.left.top.media_list.listbox insert [expr $i -1] $tmp_str
	    }
	}
    }
}

proc set_init {} {
    global init session
    set init(loop_mode) $session(loop_mode)
    set init(start_recording) $session(start_recording)
    set init(rec_enabled) $session(rec_enabled)
    set init(ff_rew_factor) $session(ff_rew_factor)
    set init(indicate) $session(indicate)
    set init(auto_start) $session(auto_start)
    set init(rec_timeout) $session(rec_timeout)
}

# initialize fonts
proc InitFonts {} {
    global font
    set font(copyright)		{-*-Helvetica-Bold-R-Normal--*-100-75-75-*-*-*-*}
    set font(default)		{-*-Helvetica-Bold-R-Normal--*-120-75-75-*-*-*-*}
    set font(hint)		{-*-Helvetica-Medium-O-Normal--*-120-75-75-*-*-*-*}
    set font(read_only)		{-*-Helvetica-Bold-R-Normal--*-100-75-75-*-*-*-*}
    set font(slider)		{-*-Helvetica-Bold-R-Normal--*-100-75-75-*-*-*-*}
    set font(vcr_title)		{-*-Helvetica-Medium-R-Normal--*-180-75-75-*-*-*-*}
    set font(window_title)	{-*-Helvetica-Bold-R-Normal--*-140-75-75-*-*-*-*}
}

# initialize global variables
proc InitGlobals {} {

    global init
    set init(loop_mode) "0"
    set init(rec_enabled) "1"
    set init(start_recording) "0"
    set init(ff_rew_factor) "25"
    set init(indicate) "0"
    set init(auto_start) "0"
    set init(rec_timeout) "30"

    global READ_ONLY
    set READ_ONLY " (session is read-only)"

    # must correspond to "enum media_fmts" in MBoneMedia.h !
    global protocols
    set protocols(0) "none"
    set protocols(1) "rtp"
    set protocols(2) "vat"
#for future use:
#    set protocols(3) "wb" 

    # see VCRFile.h for definition
    global index_values
    set index_values(NEW_TS)  1
    set index_values(NEW_SP)  2
    set index_values(NEW_FMT) 4
    set index_values(USER1)   8
    set index_values(USER2)   16
    set index_values(USER3)   32
    set index_values(USER4)   64
    set index_values(USER5)   128

    global leds
    set leds(1) 0
    set leds(2) 0
    set leds(3) 0
    set leds(4) 0
    set leds(5) 0
    set leds(6) 0
    set leds(7) 0
    set leds(8) 0
    set leds(9) 0
    set leds(10) 0

    global colors
    set colors(led,play)	"limegreen"
    set colors(led,rec)		"red"
    set colors(led,hidden)	"grey85"
    set colors(led,normal)	"black"

    set colors(skip_back,normal)	"black"
    set colors(skip_back,active)	"yellow"
    set colors(skip_back,blink)	"grey"

    set colors(rew,normal)	"black"
    set colors(rew,active)	"blue"
    set colors(rew,blink)	"grey40"

    set colors(rec,normal)	"black"
    set colors(rec,active)	"red"
    set colors(rec,blink)	"black"
    set colors(rec,disabled)	"grey40"

    set colors(stop,normal)	"black"
    set colors(stop,active)	"black"
    set colors(stop,blink)	"black"

    set colors(play,normal)	"black"
    set colors(play,active)	"limegreen"
    set colors(play,blink)	"grey40"

    set colors(ff,normal)	"$colors(rew,normal)"
    set colors(ff,active)	"$colors(rew,active)"
    set colors(ff,blink)	"$colors(rew,blink)"

    set colors(skip_forw,normal)	"$colors(skip_back,normal)"
    set colors(skip_forw,active)	"$colors(skip_back,active)"
    set colors(skip_forw,blink)		"$colors(skip_back,blink)"

    global {auto_blank_skip_time}
    set auto_blank_skip_time 0

    clear_media
    clear_session
    clear_tmp_session

    global session_file
    set session_file ""

    # initialize global variables for index_ctrl
    global idx_add_mode
    set idx_add_mode "list_and_file"

    global idx_delete_mode
    set idx_delete_mode "list_and_file"

    global index_level
    set index_level 1

    global max_indices
    set max_indices 100

    global add_to_list
    set add_to_list 1

    global index_list
    set index_list(0) {}

    global new_index_list
    set new_index_list {}

    global nb_of_indices
    set nb_of_indices(0) 0

    global where2start
    set where2start 1

    global init_file
    set init_file "~/.vcr.tcl"

    global cmd_file
    set cmd_file "~/.vcr.cmd"

    global cmd_fd
    set cmd_fd 0

    global cmd_start_time
    set cmd_start_time ""

    global sched
    set sched(sec) 0
    set sched(min) 0
    set sched(hour) 0

    global wait_mode
    set wait_mode 0
    
    global start_playing
    set start_playing ""

}

proc check_macro {} {
    global cmd_start_time cmd_file VERSION
    if { $cmd_start_time != "" } {
	set ret [AlertBox "\n$VERSION:\n\n Macro [file tail $cmd_file] is still running, \n\
		Interrupt the macro execution and run \n\
		your command or continue with the macro? \n" "" "" "$VERSION (Macro)" Interrupt Continue]
	if {$ret == 0} {
	    set cmd_start_time ""
	    print_msg "macro interrupted."
	    after 5000 print_msg ""
	    .vcr.top.macro.m entryconfigure 0 -state normal
	    .vcr.top.macro.m entryconfigure 1 -state disabled
	    return 0
	} else {
	    return 1
	}
    }
    return 0
}

proc cmd_error {cmd_file line} {
    global cmd_start_time VERSION
    set ret [AlertBox "\n$VERSION:\n\n Error in macro \"[file tail $cmd_file]\" \n\
	    Unknown vcr macro command or syntax error in: \n\
	    \"$line\" \n\
	    Ignore the command and continue with the macro \n\
	    or cancel the macro execution? \n" "" "" "$VERSION (Macro)" Ignore Cancel]
    if {$ret == "1"} {
	set cmd_start_time ""
	print_msg "macro canceled."
	after 5000 print_msg ""
	.vcr.top.macro.m entryconfigure 1 -state normal
	.vcr.top.macro.m entryconfigure 0 -state disabled
    }
    return $ret
}


proc read_cmd_file {{file ""} } {
    global cmd_file cmd_fd cmd_start_time wait_mode sched 
    if { $cmd_fd != 0 } {
	catch {close $cmd_fd}
	set cmd_fd 0
    }
    set cmd_file $file
    if { [file exist $cmd_file] == 1 } {
	vcr_ctrl set_ttl_warning 0
	set cmd_fd [open $cmd_file r]
	set cmd_start_time [string trim [eval exec "date +\"%D-%T\""] "\""]
	set sched(sec) 0
	set sched(min) 0
	set sched(hour) 0
	set sched(day) 0
	set sched(month) 0
	set sched(year) 0
	.vcr.top.macro.m entryconfigure 0 -state disabled
	.vcr.top.macro.m entryconfigure 1 -state normal
	set wait_mode 0
	read_next_cmd $cmd_start_time $cmd_fd
	vcr_ctrl set_ttl_warning 1
    } else {
	print_msg "$cmd_file not found."
	after 5000 print_msg ""
    }
}

proc read_next_cmd { {my_time ""} {my_fd ""} } {
    global session index_values cmd_file cmd_start_time sched cmd_fd wait_mode

    if { $my_fd == 0 } {
	return
    }
    if { [string compare $my_time $cmd_start_time] != 0 } {
	catch {close $my_fd}
	if { $my_fd == $cmd_fd } { set cmd_fd 0	}
	return
    }

    if { $wait_mode == 1 } {
	print_msg "[format "%s: next cmd in %02d:%02d:%02d" [file tail $cmd_file] \
		$sched(hour) $sched(min) $sched(sec)]"
	if { $sched(sec) > 0 } {
	    set sched(sec) [expr $sched(sec)-1]
	    after 1000 read_next_cmd $my_time $my_fd
	    return
	} elseif { $sched(min) > 0 } {
	    set sched(min) [expr $sched(min) - 1]
	    set sched(sec) 59
	    after 1000 read_next_cmd $my_time $my_fd
	    return
	} elseif { $sched(hour) > 0 } {
	    set sched(hour) [expr $sched(hour) - 1]
	    set sched(min) 59
	    set sched(sec) 59
	    after 1000 read_next_cmd $my_time $my_fd
	    return
	} elseif { $sched(day) > 0 } {
	    set sched(day) [expr $sched(day) - 1]
	    set sched(hour) 23
	    set sched(min) 59
	    set sched(sec) 59
	    after 1000 read_next_cmd $my_time $my_fd
	    return
	}
    } elseif { $wait_mode == 0 } {
	if { $sched(day) > 0 } {
	    set sched(day) [expr $sched(day) - 1]
	    after [expr int(24*60*60*1000)] read_next_cmd $my_time $my_fd
	    return
	} elseif { $sched(hour) > 0 } {
	    after [expr int($sched(hour)*60*60*1000)] read_next_cmd $my_time $my_fd
	    set sched(hour) 0
	    return
	} elseif { $sched(min) > 0 } {
	    after [expr int($sched(min)*60*1000)] read_next_cmd $my_time $my_fd
	    set sched(min) 0
	    return
	} elseif { $sched(sec) > 0 } {
	    after [expr int($sched(sec)*1000)] read_next_cmd $my_time $my_fd
	    set sched(sec) 0
	    return
	}
    }
    set wait_mode 0

    set line [get_next_line $my_fd]
    if {$line != ""} {
	print_msg "[file tail $cmd_file]: $line"
	set cmd_list [split $line]
	set cmd [lindex $cmd_list 0]
	set params [lrange $cmd_list 1 end]
	case [string tolower $cmd] in {
	    {load} {
		load_session [lindex $params 0]
		if { $session(loaded) != 1 } {
		    set cmd_start_time ""
		    print_msg "[file tail $cmd_file]: could not load session \"[lindex $params 0]\""
		    after 5000 print_msg ""
		    .vcr.top.macro.m entryconfigure 0 -state normal
		    .vcr.top.macro.m entryconfigure 1 -state disabled
		}
	    }
	    {unload} {
		if { $session(auto_start) == "1" && $session(tools_started) != "" } {
		    stop_tools
		}
		close_session 1
	    }
	    {start_tools} {
		start_tools [lindex $params 0]
	    }
	    {stop_tools} {
		stop_tools
	    }
	    {play} {
		if { $session(auto_start) == "1" && $session(tools_started) == "" } {
		    start_tools
		} else {
		    set session(state) [vcr_ctrl play]
		}
		set session(state) [vcr_ctrl play]
	    }
	    {rec} {
		set session(state) [vcr_ctrl rec [join $params]]
	    }
	    {stop} {
		set session(state) [vcr_ctrl stop]
	    }
	    {goto} {
		case [string tolower [lindex $params 0]] {
		    {end} {
			set session(state) [vcr_ctrl goto end]
		    }
		    {start} {
			set session(state) [vcr_ctrl goto start]
		    }
		    {next_index} {
			if { "[lindex $params 1]" != "" } {
			    set idx [lindex $params 1]
			} else {
			    set idx 1
			}
			set session(state) [vcr_ctrl goto next_index $session(index_media) $index_values(USER$idx)]
		    }
		    {prev_index} {
			if { "[lindex $params 1]" != "" } {
			    set idx [lindex $params 1]
			} else {
			    set idx 1
			}
			set session(state) [vcr_ctrl goto prev_index $session(index_media) $index_values(USER$idx)]
		    }
		    {prev_blank} {
			set session(state) [vcr_ctrl goto prev_blank $session(index_media) [lindex $params 1]]
		    }
		    {next_blank} {
			set session(state) [vcr_ctrl goto next_blank $session(index_media) [lindex $params 1]]
		    }
		    {default} {
			set goto_ts(sec) 0
			set goto_ts(min) 0
			set goto_ts(hour) 0
			
			set x [split [lindex $params 0] ":"]
			set i [expr [llength $x] - 1]
			foreach  j [lrange {{sec} {min} {hour}} 0 $i]  {
			    set goto_ts($j) [lindex $x $i]
			    set i [expr $i - 1]
			}
			while { $goto_ts(sec) >= 60 } {
			    set goto_ts(min) [expr $goto_ts(min) + 1]
			    set goto_ts(sec) [expr $goto_ts(sec) - 60]
			}
			while { $goto_ts(min) >= 60 } {
			    set goto_ts(hour) [expr $goto_ts(hour) + 1]
			    set goto_ts(min) [expr $goto_ts(min) - 60]
			}
			print_msg "[format "%s: goto %02d:%02d:%02d" [file tail $cmd_file] \
				$goto_ts(hour) $goto_ts(min) $goto_ts(sec)]"
			
			set ts [expr $goto_ts(hour) * 3600 + $goto_ts(min) * 60 + $goto_ts(sec)]
			set session(state) [vcr_ctrl goto ts $ts]
		    }
		}
	    }
	    {idx_media} {
		set session(index_media) [lindex $params 0]
	    }
	    {auto_blank} {
		set session(state) [vcr_ctrl set_auto_blank_skip $session(index_media) [lindex $params 0]]
	    }
	    {speed} {
		case [string tolower [lindex $params 0]] {
		    {normal} {
			set session(speed) 1
		    }
		    {double} {
			set session(speed) 2
		    }
		    {half} {
			set session(speed) 0.5
		    }
		    {default} {
			if {[cmd_error $cmd_file $line] == "1"} return
		    }
		}
		vcr_ctrl speed $session(speed)
	    }
	    {blink} {
		case [string tolower [lindex $params 0]] {
		    {on} {
			set session(blink_mode) 1
			if { $session(start_blinking) != "" } {
			    eval "$session(start_blinking)"
			}
		    }
		    {off} {
			set session(blink_mode) 0
		    }
		    {default} {
			if {[cmd_error $cmd_file $line] == "1"} return
		    }
		}
	    }
	    {loop} {
		case [string tolower [lindex $params 0]] {
		    {on} {
			set session(loop_mode) 1
		    }
		    {off} {
			set session(loop_mode) 0
		    }
		    {default} {
			if {[cmd_error $cmd_file $line] == "1"} return
		    }
		}
	    }
	    {auto_start} {
		case [string tolower [lindex $params 0]] {
		    {on} {
			set session(auto_start) 1
		    }
		    {off} {
			set session(auto_start) 0
		    }
		    {default} {
			if {[cmd_error $cmd_file $line] == "1"} return
		    }
		}
	    }
	    {rec_enabled} {
		case [string tolower [lindex $params 0]] {
		    {on} {
			enable_recording
			update_main_win
		    }
		    {off} {
			disable_recording
			update_main_win
		    }
		}
	    }
	    {start_rec} {
		case [string tolower [lindex $params 0]] {
		    {on_signal} {
			set session(start_recording) 0
		    }
		    {no_signal} {
			set session(start_recording) 1
		    }
		    {default} {
			if {[cmd_error $cmd_file $line] == "1"} return
		    }
		}
	    }
	    {rec_timeout} {
		set session(rec_timeout) [lindex $params 0]
		vcr_ctrl set_rec_timeout [lindex $params 0]
	    }
	    {ff_rew_factor} {
		if { [llength $params] == 1 } {
		    set session(ff_rew_factor) [lindex $params 0]
		    vcr_ctrl set_ff_rew_factor [lindex $params 0]
		} else {
		    if {[cmd_error $cmd_file $line] == "1"} return
		}
	    }
	    {ff} {
		if { [llength $params] == 0 } {
		    set value $session(ff_rew_factor)
		} else {
		    set value [lindex $params 0]
		}
		set session(state) [vcr_ctrl goto ts [expr [.vcr.position.slider get]+$value]]
	    }
	    {rew} {
		if { [llength $params] == 0 } {
		    set value $session(ff_rew_factor)
		} else {
		    set value [lindex $params 0]
		}
		set session(state) [vcr_ctrl goto ts [expr [.vcr.position.slider get]-$value]]
	    }
	    {mute} {
		mute_media [join $params]
	    }
	    {wait} {
		if {[llength $params] > 0} {
		    set sched(sec) 0
		    set sched(min) 0
		    set sched(hour) 0
		    set sched(day) 0
		    set sched(month) 0
		    set sched(year) 0

		    set x [split [lindex $params 0] ":"]
		    set i [expr [llength $x] - 1]
		    foreach  j [lrange {{sec} {min} {hour}} 0 $i]  {
			set sched($j) [lindex $x $i]
			set i [expr $i - 1]
		    }
		    while { $sched(sec) >= 60 } {
			set sched(min) [expr $sched(min) + 1]
			set sched(sec) [expr $sched(sec) - 60]
		    }
		    while { $sched(min) >= 60 } {
			set sched(hour) [expr $sched(hour) + 1]
			set sched(min) [expr $sched(min) - 60]
		    }
		    set wait_mode 1
		    print_msg "[format "%s: next cmd in %02d:%02d:%02d" [file tail $cmd_file] \
			    $sched(hour) $sched(min) $sched(sec)]"
		    after 0 read_next_cmd $my_time $my_fd
		    return
		} else {
		    if {[cmd_error $cmd_file $line] == "1"} return
		}
	    }
	    {at} {
		if {[llength $params] > 0} {
		    set date [string trim [eval exec "date +\"%D|%T\""] "\""]
		    set y [split $date "|"]
		    set x [split [lindex $y 0] "/"]
		    set i 0
		    foreach  j {{month} {day} {year}} {
			set now($j) [expr int([string trimleft [lindex $x $i] 0])]
			incr i
		    }
		    set x [split [lindex $y 1] ":"]
		    set i 0
		    foreach  j {{hour} {min} {sec}}  {
			set now($j) [expr int([string trimleft [lindex $x $i] 0])]
			incr i
		    }
		    set sched(sec) $now(sec)
		    set sched(min) $now(min)
		    set sched(hour) $now(hour)
		    set sched(month) $now(month)
		    set sched(day) $now(day)
		    set sched(year) $now(year)

		    if {[llength $params] >= 1} {
			set x [split [lindex $params 0] ":"]
			set i [expr [llength $x] - 1]
			if { $i == 2} {
			    foreach  j [lrange {{sec} {min} {hour}} 0 $i]  {
				set sched($j) [expr int([string trimleft [lindex $x $i] 0])]
				set i [expr $i - 1]
			    }
			    if {[llength $params] == 2} {
				set x [split [lindex $params 1] "/"]
				set i [expr [llength $x] - 1]
				if { $i == 2 } {
				    foreach  j [lrange {{year} {day} {month}} 0 $i]  {
					set sched($j) [expr int([string trimleft [lindex $x $i] 0])]
					set i [expr $i - 1]
				    }
				} else {
				    if {[cmd_error $cmd_file $line] == "1"} {
					return
				    } else {
					# ignore this line and read the next cmd
					set sched(sec) 0
					set sched(min) 0
					set sched(hour) 0
					set sched(day) 0
					set sched(month) 0
					set sched(year) 0
					after 1 read_next_cmd $my_time $my_fd
					return
				    }
				}
			    }

			    set months { {31} {28} {31} {30} {31} {30} {31} {31} {30} {31} {30} {31} }

			    if {    ($sched(sec) < 60) && ($sched(sec) >= 0) &&\
				    ($sched(min) < 60) && ($sched(min) >= 0) &&\
				    ($sched(hour) < 24) && ($sched(hour) >= 0) &&\
				    ($sched(month) <= 12) && ($sched(month) >=1 ) &&\
				    ($sched(day) > 0) && ($sched(year)>=0) && \
				    ($sched(day) <= [lindex $months [expr $sched(month)-1]] ) } {

				set x1 [format "%d%02d%02d" $now(year) $now(month) $now(day)]
				set y1 [format "%d%02d%02d" $sched(year) $sched(month) $sched(day)]

				set x2 [format "%d%02d%02d" $now(hour) $now(min) $now(sec)]
				set y2 [format "%d%02d%02d" $sched(hour) $sched(min) $sched(sec)]

				if {($x1<=$y1) || (($x1==$y1)&&($x2<=$y2))} {
				    set x [format "%02d:%02d:%02d %02d/%02d/%02d" \
					    $sched(hour) $sched(min) $sched(sec) \
					    $sched(month) $sched(day) $sched(year)]
				    print_msg "[file tail $cmd_file]: next cmd at $x"
				    set sched(year) [expr $sched(year) - $now(year)]
				    set sched(month) [expr $sched(month) - $now(month)]
				    set sched(day) [expr $sched(day) - $now(day)]
				    set sched(hour) [expr $sched(hour) - $now(hour)]
				    set sched(min) [expr $sched(min) - $now(min)]
				    set sched(sec) [expr $sched(sec) - $now(sec)]
				    if { $sched(sec) < 0 } {
					set sched(min) [expr $sched(min) - 1]
					set sched(sec) [expr 60 + $sched(sec)]
				    }
				    if { $sched(min) < 0 } {
					set sched(hour) [expr $sched(hour) - 1]
					set sched(min) [expr 60 + $sched(min)]
				    }
				    if { $sched(hour) < 0 } {
					set sched(day) [expr $sched(day) - 1]
					set sched(hour) [expr 24 + $sched(hour)]
				    }
				    if { $sched(day) < 0 } {
					set sched(month) [expr $sched(month) - 1]
					set month [expr $now(month) + $sched(month)]
					if { $month > 12 } { set month [expr $month - 12] }
					if { $month < 1 } { set month [expr $month + 12] }
					set sched(day) [expr [expr [lindex $months $month] - 1] + $sched(day)]
				    }
				    if { $sched(month) < 0 } {
					set sched(year) [expr $sched(year) - 1]
					set sched(month) [expr 12 + $sched(month)]
				    }
				    if { $sched(year) < 0 } {
					set sched(year) [expr 100 - $now(year) - $sched(year)]
				    }

				    set month [expr $now(month)]
				    for { set i 0} { $i < $sched(month) } {incr i} {
					if { $month >= 12 } {
					    set month [expr $month - 12]
					}
					set sched(day) [expr $sched(day) + [lindex $months $month]]
					set month [expr $month + 1]
				    }
				    set sched(day) [expr $sched(year) * 365 + $sched(day) ]
				    set wait_mode 0
				    after 0 read_next_cmd $my_time $my_fd
#				    puts "$now(hour):$now(min):$now(sec) $now(month)/$now(day)/$now(year)"
#				    puts "$sched(hour):$sched(min):$sched(sec) $sched(month)/$sched(day)/$sched(year)"
				    return
				} else {
				    print_msg "[file tail $cmd_file]: sorry, too late for \"at $params\"."
				    set sched(sec) 0
				    set sched(min) 0
				    set sched(hour) 0
				    set sched(day) 0
				    set sched(month) 0
				    set sched(year) 0
				    set cmd_start_time ""
				    return
				}
			    }
			}
		    }
		}
		set sched(sec) 0
		set sched(min) 0
		set sched(hour) 0
		set sched(day) 0
		set sched(month) 0
		set sched(year) 0
		if {[cmd_error $cmd_file $line] == "1"} {
		    set cmd_start_time ""
		    return
		}
	    }
	    {quit} {
		quit_vcr
		return
	    }
	    {default} {
		if {[cmd_error $cmd_file $line] == "1"} return
	    }
	}
	# we do this instead of a loop to be able to interupt the execution.
	after 1 read_next_cmd $my_time $my_fd
    } else {
	catch {close $my_fd}
	if { $my_fd == $cmd_fd } { set cmd_fd 0	}
	set cmd_start_time ""
	if { "[file tail $cmd_file]" != ".vcr.cmd" } {
	    print_msg "macro \"[file tail $cmd_file]\" done."
	    after 5000 print_msg ""
	} else {
	    print_msg ""
	}
	.vcr.top.macro.m entryconfigure 0 -state normal
	.vcr.top.macro.m entryconfigure 1 -state disabled
    }
}

proc print_msg { {msg ""} } {
    .vcr.middle.right.middle.label configure -text "[center "$msg" 80]"
}

proc button_hint { {w ""} {msg1 ""} {msg2 ""}} {
    after 1500 "button_hint_do_it $w \"$msg1\" \"$msg2\""
}

proc button_hint_do_it { {w ""} {msg1 ""} {msg2 ""} } {
    if { [winfo containing [winfo pointerx $w] [winfo pointery $w] ] == "$w" } {    
	if {[lindex [$w configure -state] 4] != "disabled"} {
	    if { "$msg1" != "" } {
		print_msg "$msg1"
	    }
	} elseif { "$msg2" != "" } {
	    print_msg "$msg2"
	}
    }
}


proc set_led { {media_nb 0} {state 0} } {
    global colors leds session
    if { $session(indicate) == 0 } { return }
    if { ( $media_nb>0 ) && ( $media_nb <= 10 ) } {
	if { $state == 1 } {
	    .vcr.middle.left.bottom.leds.led$media_nb configure -foreground $colors(led,play)
	    after 100 clear_led $media_nb
	} elseif { $state == 2 } {
	    .vcr.middle.left.bottom.leds.led$media_nb configure -foreground $colors(led,rec)
	    after 100 clear_led $media_nb
	} else {
	    clear_led $media_nb
	}
	set leds($media_nb) $state
    }
}

proc clear_led { {media_nb 0}} {
    global colors leds session
    if { $session(indicate) == 0 } { return }
    if { ( $media_nb>0 ) && ( $media_nb <= 10 ) } {
	.vcr.middle.left.bottom.leds.led$media_nb configure -foreground $colors(led,normal)
	set leds($media_nb) 0
    }
}

proc update_leds {} {
    global leds colors session
    if { $session(indicate) == 1 } {
	.vcr.middle.left.bottom.leds.title configure -text {Record/Playback LEDs:}
	for { set i 1 } { $i <= 10 } { incr i } { set_led $i $leds($i) }
    } else {
	.vcr.middle.left.bottom.leds.title configure -text {}
	for { set i 1 } { $i <= 10 } { incr i } {
	    .vcr.middle.left.bottom.leds.led$i configure -foreground $colors(led,hidden)
	}
    }
}

proc check_file_ext { file } {
    if { [file extension $file] != ".vcr" } {
	set file ${file}.vcr
    }
    return $file
}

# Parse commandline parameters
proc parse_cmd_line {} {
    global argc argv init session_file init_file cmd_file VERSION

    set tmpArgv ""

    for {set counter 1} {$counter < $argc} {incr counter 1} {
	set test [string tolower [lindex $argv $counter]]

	if { ([string match "-c*" $test] == 1) && ([string match "-co*" $test] == 0) } {
	    # -c[md] (but not -color !)
	    incr counter 1
	    set cmd_file "[lindex $argv $counter]"
	    if {[file exists $cmd_file] == "0"} {
		puts "$VERSION error:"
		puts "  cmd file \"$cmd_file\" not found.\n"
		exit
	    }

	} elseif { [string match "-h*" $test] == 1} {
	    # -h[elp]
	    usage
	    exit

	} elseif { [string match "-i" $test] == 1} {
	    # -f
	    incr counter 1
	    set init_file "[lindex $argv $counter]"
	    if {[file exists $init_file] == "0"} {
		puts "$VERSION error:"
		puts "  init file \"$init_file\" not found.\n"
		exit
	    }

	} elseif { [string match "-s" $test] == 1} {
	    # -s
	    incr counter 1
	    set session_file "[lindex $argv $counter]"
	    set session_file [check_file_ext $session_file]
	    if {[file exists $session_file] == "0"} {
		puts "$VERSION error:"
		puts "  session file \"$session_file\" not found.\n"
		exit
	    }
	} elseif { [string match "-v" $test] == 1} {
	    puts "\n$VERSION\n"
	    exit
	} else {
	    lappend tmpArgv [lindex $argv $counter]
	}
    }

    set argv $tmpArgv
    set argc [llength $tmpArgv]

    if {([string match "-*" $tmpArgv] == 0) && ($session_file == "") && ($argc == 1) } {
	set session_file $tmpArgv
	set session_file [check_file_ext $session_file]
	if {[file exists $session_file] == "0"} {
	    puts "$VERSION error:"
	    puts "  session file \"$session_file\" not found.\n"
	    exit
	}
	set argv ""
	set argc 0
    }

    if { $argc > 0 } {
	global VERSION
	puts "$VERSION error:"
	puts "  invalid argument(s): \"$argv\"\n"
	usage
	exit
    }
    unset counter
    unset tmpArgv

    # if a session was specified load it now
    if { $session_file != "" } {
	load_session $session_file
    } else {
	update_main_win
    }
    
    
    # if an init file was specified load it now
    if {[file exists $init_file] == "1"} {
	puts "$init_file"
	catch "source $init_file" error_msg
	if {$error_msg != ""} {
	    global errorInfo
	    puts "$VERSION error:"
	    puts "  error loading init file: \"$init_file\"."
	    puts "  \n$error_msg\n\nstack trace:\n$errorInfo\n"
	    exit
	}
    }
    
    # if a cmd file was specified load it now
    if {[file exists $cmd_file] == "1"} {
	read_cmd_file $cmd_file
    }

}


proc rec {} {
    global session 
    if { $session(state) != "playing" && $session(state) != "recording"} {
	if {[check_macro] == "1"} {return}
	if {[demo_version] == "0"} {
	    set session(state) [vcr_ctrl rec $session(start_recording)]
	    if {$session(state) == "recording" } {
		.vcr.top.speed.selection entryconfigure 0 -state disabled
		.vcr.top.speed.selection entryconfigure 1 -state disabled
		.vcr.top.speed.selection entryconfigure 2 -state disabled
		
		.vcr.top.session.m entryconfigure 0 -state disabled
		.vcr.top.session.m entryconfigure 1 -state disabled
		.vcr.top.session.m entryconfigure 2 -state disabled
		.vcr.top.session.m entryconfigure 4 -state disabled
		.vcr.top.session.m entryconfigure 5 -state disabled
		.vcr.top.session.m entryconfigure 7 -state disabled
		.vcr.top.session.m entryconfigure 8 -state disabled
		.vcr.top.session.m entryconfigure 10 -state disabled
		.vcr.top.session.m entryconfigure 11 -state disabled
		
		.vcr.middle.right.bottom.skip_back configure -state disabled
		.vcr.middle.right.bottom.rew configure -state disabled
		.vcr.middle.right.bottom.play configure -state disabled
		.vcr.middle.right.bottom.ff configure -state disabled
		.vcr.middle.right.bottom.skip_forw configure -state disabled
		
		set_index_rec
	    }
	}
    }
}

proc play {} {
    global session 
    if { $session(state) != "playing" && $session(state) != "recording"} {
	if {[check_macro] == "1"} {return}
	if { $session(auto_start) == "1" && $session(tools_started) == "" } {
	    start_tools_win play
	} else {
	    set session(state) [vcr_ctrl play]
	}
    }
}

proc stop {} {
    global session 
    if {[check_macro] == "1"} {return}
    if { $session(state) == "recording" } {
	.vcr.top.speed.selection entryconfigure 0 -state normal
	.vcr.top.speed.selection entryconfigure 1 -state normal
	.vcr.top.speed.selection entryconfigure 2 -state normal
	
	.vcr.middle.right.bottom.skip_back configure -state normal
	.vcr.middle.right.bottom.rew configure -state normal
	.vcr.middle.right.bottom.play configure -state normal
	.vcr.middle.right.bottom.ff configure -state normal
	.vcr.middle.right.bottom.skip_forw configure -state normal
	
	.vcr.top.session.m entryconfigure 0 -state normal
	.vcr.top.session.m entryconfigure 1 -state normal
	.vcr.top.session.m entryconfigure 2 -state normal
	.vcr.top.session.m entryconfigure 4 -state normal
	.vcr.top.session.m entryconfigure 5 -state normal
	.vcr.top.session.m entryconfigure 7 -state normal
	.vcr.top.session.m entryconfigure 8 -state normal
	.vcr.top.session.m entryconfigure 10 -state normal
	.vcr.top.session.m entryconfigure 11 -state normal
	
	set_index_normal
    }
    release_button
    set session(state) [vcr_ctrl stop]
}

# eof






